/**
 * @class Sb.HeaderManager
 * Utility class for headers
 * @singleton
 */
Ext.define('Sb.HeaderManager', function () {
	var headers = new Ext.data.JsonStore({
			storeId: 'headers',
			fields: [
				'name', 'time', 'path', 'flashWidth', 'flashHeight',
				'keywords', 'relevance', 'isCustom', 'format', 'areas', 'imageSize'
			],
			applySort: function () {
				var index, record;

				Ext.data.JsonStore.superclass.applySort.call(this);

				index = this.find('name', 'fallback');
				if (index !== -1) {
					headers.suspendEvents();
					record = headers.getAt(index);
					headers.removeAt(index);
					headers.insert(0, record);
					headers.resumeEvents();
					headers.fireEvent('datachanged', headers);
				}
			},
			listeners: {
				load: function () {
					headers.applySort();
				}
			}
		});

	headers.on('remove', function (store, record) {
		var imageName = record.get('name'),
			imagePath = Sb.model.HeaderWidget.buildImagePath(imageName),
			fallback = store.getById('system.fallback'),
			design = Sb.Editor.model.design,
			siteBackground = design.getBackground('site');

		Sb.WidgetTypeManager.get('header').cache.each(function (header) {
			if (header.get('imagePath') === imagePath) {
				header.unserialize({
					imageName: fallback.get('name'),
					imagePath: fallback.get('path')
				});
				if (!Sb.Editor.Ui.editComponent || header !== Sb.Editor.Ui.editComponent.model) {
					header.save();
				}
			}
		}, this);

		if (siteBackground.image.indexOf('Header/' + imageName) !== -1) {
			siteBackground.type = 'none';
			design.setBackground('site', siteBackground);
			design.save();
		}

		Sb.SiteService.deleteHeader(imageName, function (result) {
			if (result) {
				Sb.Editor.Ui.mainWindow.statusBar.setMessageInfo(
					Sb.WidgetTypeManager.get('header').getTranslation('imageWasDeleted')
				);
			} else {
				Sb.Editor.Ui.mainWindow.restore();
				Sb.Editor.Ui.mainWindow.statusBar.setMessageError(
					Sb.WidgetTypeManager.get('header').getTranslation('imageDeleteFailed')
						.replace('{headerName}', record.get('name'))
				);
			}
		});
	});

	headers.multiSort([{
		field: 'isCustom',
		direction: 'DESC'
	}, {
		field: 'relevance',
		direction: 'DESC'
	}]);

	return {
		singleton: true,
		headers: headers,

		loadHeaders: function () {
			if (this.headers.getCount() === 0) {
				this.loadSystemHeaders();
				this.loadSiteHeaders();
			}
		},

		loadSystemHeaders: function () {
			Sb.SystemService.getHeaders(function (result) {
				Ext.each(result, function (data) {
					data.id = 'system.' + data.name;
					data.isCustom = false;
					data.path = Sb.Config.headersUrl + '/' + data.name + '/preview.' + data.format;
				});

				this.headers.loadData(result, true);
			}, this);
		},

		loadSiteHeaders: function () {
			Sb.SiteService.getHeaders(function (result) {
				Ext.each(result, function (data) {
					data.id = 'site.' + data.name;
					data.isCustom = true;
					var name = Sb.utils.Flash.isFlash(data.name)
						? data.name
						: 'thumbnail_' + data.name;
					data.path = Sb.Editor.model.getUrl() + '/' + Sb.model.HeaderWidget.buildImagePath(name) +
						'?' + data.time;
				});
				this.headers.loadData(result, true);
			}, this);
		},

		/**
		 * @param {String} name Banner's name
		 * @return {String} Format of the image like "jpg"
		 */
		getHeaderFormat: function (name) {
			return this.headers.getAt(this.headers.find('name', name)).get('format');
		}
	};
});
