/**
 * @class Sb.Limits
 * Utility class for chekc website limits
 * @singleton
 */
Ext.define('Sb.Limits', {
	singleton: true,

	/**
	 * Returns all widgets limits with current values
	 *
	 * @param {Sb.model.Site} siteModel
	 * @returns {Object}
	 */
	getWidgetLimits: function (siteModel) {
		var listUsedWidgets = {},
			widgetsList = {};

		siteModel.rootPage.applySelfAndRecursive(function (page) {
			Sb.Limits.usedWidgets(page.widget, listUsedWidgets);
		});

		Ext.iterate(siteModel.get('limits'), function (widgetType, limit) {
			if (Sb.Config.defaultLimits[widgetType] === limit
				|| widgetType === 'eshop_basket'
				|| widgetType === 'max_pages_number'
			) {
				return;
			}

			if (widgetType === 'eshop_catalog') {
				limit = Boolean(limit);
			}

			widgetsList[widgetType] = {
				count: listUsedWidgets[widgetType] || 0,
				limit: limit
			};
		}, this);

		return widgetsList;
	},

	isUpsellWidgets: function (siteModel, widgetType, widgets) {
		var listUsedWidgets = {},
			limit = siteModel.get('limits')[widgetType];

		if (limit === undefined) {
			return false;
		}

		listUsedWidgets[widgetType] = 0;
		widgets.each(function (widget) {
			Sb.Limits.usedWidgets(widget, listUsedWidgets);
		});

		return (limit > -1) && (listUsedWidgets[widgetType] >= limit);
	},

	/**
	 * Returns true if site limits is default
	 *
	 * @param siteModel
	 * @returns {boolean}
	 */
	isDefaultLimits: function (siteModel) {
		var result = true,
			currentLimits = siteModel.get('limits');

		Ext.iterate(Sb.Config.defaultLimits, function (name, value) {
			if (currentLimits[name] !== value) {
				result = false;
				return false;
			}
		});
		return result;
	},

	/**
	 * Check widgets on exceeded limits
	 *
	 * @param {Sb.model.Site} siteModel
	 * @param {Array} [widgets] List of widgets for which should be check limits
	 * @return {Object}
	 */
	checkWidgets: function (siteModel, widgets) {
		var isExceeded = false,
			listUsedWidgets = {},
			widgetsList = {};

		widgets = (undefined === widgets) ? [] : Ext.isArray(widgets) ? widgets : [widgets];
		siteModel.rootPage.applySelfAndRecursive(function (page) {
			Sb.Limits.usedWidgets(page.widget, listUsedWidgets);
		});

		Ext.iterate(siteModel.get('limits'), function (widgetType, limit) {
			if (limit == -1
				|| (!Ext.isEmpty(widgets) && widgets.indexOf(widgetType) == -1)
				|| undefined === listUsedWidgets[widgetType]
				|| listUsedWidgets[widgetType] <= limit) {
				return;
			}

			isExceeded = true;
			widgetsList[widgetType] = {
				count: listUsedWidgets[widgetType],
				limit: limit
			};
		}, this);

		return {
			isExceeded: isExceeded,
			widgets: widgetsList
		};
	},

	usedWidgets: function (widget, listUsedWidgets) {
		var type = widget.get('type');
		if (listUsedWidgets[type] === undefined) {
			listUsedWidgets[type] = 0;
		}
		if (widget.isConfiguredProperties()) {
			listUsedWidgets[type]++;
		}
		widget.widgets.each(function (widget) {
			this.usedWidgets(widget, listUsedWidgets);
		}, this);
	},

	/**
	 * Check website pages limit
	 *
	 * @param {Sb.model.Site} siteModel
	 * @return {Object}
	 */
	checkPages: function (siteModel) {
		var countPages = 0;

		siteModel.rootPage.applyRecursive(function (page) {
			if (page.get('sitePageTypeId') == Sb.model.SitePage.LINK_TYPE) {
				return;
			}

			var masterWidget = page.getMasterWidget();
			if (!(masterWidget instanceof Sb.model.BlogPostWidget)) {
				countPages++;
			}
		});

		return {
			isExceeded: -1 != siteModel.get('limits').max_pages_number
				&& countPages > siteModel.get('limits').max_pages_number,
			limit: siteModel.get('limits').max_pages_number,
			count: countPages
		};
	},

	/**
	 * Check additional locales limit
	 *
	 * @param {Sb.model.Site} site
	 * @param {String[]} locales
	 * @return {Object}
	 */
	checkLocales: function (site, locales) {
		var limit = site.get('limits').max_additional_locales,
			additionalLocales = locales.slice(1);

		return {
			isExceeded: limit !== -1 && additionalLocales.length > limit,
			isReached: limit !== -1 && additionalLocales.length >= limit,
			locales: additionalLocales.slice(limit),
			limit: limit
		};
	}
});