/**
 * @class Sb.controller.Blogs
 * @extends Sb.Controller
 *
 * Blogs controller
 */
Ext.define('Sb.controller.Blogs', {
	extend: 'Sb.Controller',

	/**
	 * Create a new blog post in blog.
	 * @param {Sb.model.BlogWidget} blog
	 * @param {Function} [cb] A function to be called after the post was created.
	 * @param {Object} [scope] The scope (<code>this</code> reference) in which the callback function is executed.
	 */
	createPost: function (blog, cb, scope) {
		Sb.PageService.createSlug(Sb.Locale.get('newBlogPostPage'), function (slug) {
			var postPage,
				container,
				post,
				date = new Date(),
				styles = {
					padding: '5px 10px 5px 10px',
					margin: '0px 0px 0px 0px',
					background: '',
					backgroundColor: 'transparent',
					backgroundPosition: 'top left',
					backgroundStretch: 'tile',
					backgroundOpacity: '100',
					borderRadius: '0 0 0 0',
					boxShadow: 'none',
					textColor: 'inherit',
					textStroke: false,
					linkColor: 'inherit',
					linkStroke: false,
					h1Color: 'inherit',
					h1Stroke: false,
					h2Color: 'inherit',
					h2Stroke: false
				},
				textStyles = Ext.apply({}, {padding: '0px 10px 0px 10px'}, styles);

			postPage = Sb.Editor.getController('Pages').createPage({
				title		: Sb.Locale.get('newBlogPostPage'),
				slug		: slug,
				slugIsAuto	: '1',
				hidden		: true
			});

			container = postPage.widget.widgets.find(function (container) {
				return container.data.region == 'content';
			});

			post = new Sb.model.BlogPostWidget({
				createdTimestamp: Math.round(date.getTime() / 1000) - date.getTimezoneOffset() * 60,
				styles: styles,
				visibilityMode: 'all',
				widgets			: [{
					type	: 'text',
					content	: '',
					visibilityMode: 'all',
					styles:	textStyles
				}, {
					type	: 'blog_post_cut'
				}, {
					type	: 'text',
					content	: '',
					visibilityMode: 'all',
					styles:	textStyles
				}]
			});
			container.widgets.insert(0, post);

			if (blog.isAllowComments()) {
				var commenting = new Sb.model.CommentingWidget({
					visibilityMode: 'all',
					styles: textStyles
				});
				container.widgets.insert(1, commenting);
				commenting.set('thread', commenting.generateThreadId());
			}

			blog.ownerPage.insert(blog.ownerPage.pages.getCount(), postPage);
			Ext.callback(cb, scope, [post]);
		}, this);
	},

	/**
	 * Gets list of blog posts
	 * @param {Sb.model.BlogWidget} blog
	 * @return {Ext.util.MixedCollection} Returns an Ext.util.MixedCollection of the blog posts
	 */
	getPosts: function (blog) {
		var posts = new Ext.util.MixedCollection();
		blog.ownerPage.pages.each(function (page) {
			var post = page.widget.queryBy(function (widget) {
				return widget instanceof Sb.model.BlogPostWidget;
			})[0];
			if (!post) {
				return true;
			}
			posts.add(post);
		}, this);

		posts.sort('DESC', function (post1, post2) {
			var v1 = post1.get('createdTimestamp'),
				v2 = post2.get('createdTimestamp');

			return v1 > v2 ? 1 : (v1 < v2 ? -1 : 0);
		});

		return posts;
	},

	/**
	 * Add commenting widget to blog post page
	 * @param {Sb.model.BlogPostWidget} post
	 */
	addCommentingToPost: function (post) {
		if (this.findCommentingOnPost(post)) {
			return;
		}

		var commenting = Sb.WidgetTypeManager.get('commenting');
		post.ownerWidget.createChild(
			commenting,
			post.ownerWidget.widgets.indexOf(post) + 1
		);

		if (Sb.Limits.checkWidgets(Sb.Editor.model, commenting.name).isExceeded) {
			Sb.Editor.Ui.mainWindow.statusBar.setMessageWarning(
				Sb.Locale.get(
					'moduleLimitExceeded',
					commenting.getTranslation('title')
				)
			);
		}
	},

	/**
	 * Remove commenting widget from blog post page
	 * @param {Sb.model.BlogPostWidget} post
	 */
	removeCommentingFromPost: function (post) {
		var commenting = this.findCommentingOnPost(post);
		if (!commenting) {
			return;
		}

		post.ownerWidget.removeChild(commenting);
	},

	/**
	 * Find commenting widget on blog post page
	 * @param {Sb.model.BlogPostWidget} post
	 */
	findCommentingOnPost: function (post) {
		return post.ownerWidget.widgets.find(function (widget) {
			return widget instanceof Sb.model.CommentingWidget;
		});
	}
});
