/**
 * @class Sb.editor.MainWindow
 * @extends Ext.Window
 */
Ext.define('Sb.editor.MainWindow', {
	extend: 'Ext.Window',

	/**
	 * @cfg {Array} widgets
	 * <code>widgets</code> will be used as <code>{@link Sb.editor.WidgetsPanel#items items}</code> for the widgets panel.
	 */

	// private
	initComponent: function () {
		function menuItemOver(menu, e, menuItem) {
			if (menuItem && menuItem.tooltip) {
				Sb.Editor.Ui.mainWindow.setHelperMessage(menuItem.tooltip);
			}
		}

		/**
		 * Link to status bar
		 * @type Sb.editor.StatusBar
		 * @property statusBar
		 */
		this.statusBar = new Sb.editor.StatusBar();

		/**
		 * @type Sb.dashboard.Manager
		 * @property dashboardManager
		 */
		this.dashboardManager = new Sb.dashboard.Manager();

		/**
		 * How many pixels of window will be always visible in X axis
		 * @type Number
		 * @property visibleOffsetX
		 */
		this.visibleOffsetX = 45;

		/**
		 * How many pixels of window will be always visible in Y axis
		 * @type Number
		 * @property visibleOffsetY
		 */
		this.visibleOffsetY = 27;

		/**
		 * Permanent helper message
		 * @type String
		 * @property permanentHelperMessage
		 */
		this.permanentHelperMessage = '';

		/**
		 * Helper message height
		 * @type Number
		 * @property helperMessageHeight
		 */
		this.helperMessageHeight = 0;

		var plugins = [];
		if (Sb.Config.general.facebookApplicationUrl) {
			/**
			 * Facebook Drawer
			 * @type Ext.Window
			 * @property fbDrawer
			 */
			this.fbDrawer = new Ext.ux.plugins.WindowDrawer({
				side		: 's',
				animate		: true,
				margin		: 23,
				overlap		: 21,
				baseCls		: 'sb-editor-panel-fbdrawer',
				shadow		: false,
				layout		: 'table',
				layoutConfig: { columns: 3 },
				items		: [{
					cellCls		: 'sb-editor-panel-fbdrawer-text-cell',
					xtype		: 'component',
					html		: Sb.Locale.get('facebookPromotionText')
				}, {
					cellCls		: 'sb-editor-panel-fbdrawer-button-cell',
					xtype		: 'button',
					cls			: 'sb-btn',
					minWidth	: 75,
					text		: Sb.Locale.get('yesButtonTitle'),
					scope		: this,
					handler		: function () {
						this.fbDrawer.hide();
						Sb.Editor.startFacebookIntegration();
						Ext.state.Manager.set('isNotShowFBIntegration', true);
					}
				}, {
					cellCls		: 'sb-editor-panel-fbdrawer-button-cell',
					xtype		: 'button',
					cls			: 'sb-btn',
					minWidth	: 75,
					text		: Sb.Locale.get('noButtonTitle'),
					scope		: this,
					handler		: function () {
						this.fbDrawer.hide();
						Ext.state.Manager.set('isNotShowFBIntegration', true);
					}
				}]
			});

			plugins.push(this.fbDrawer);
		}

		Ext.apply(this, {
			baseCls: 'sb-editor-panel',
			manager: Sb.Editor.windowManager.getManager(8),
			title: Sb.Config.general.product + ' ' + Sb.Config.general.version,
			width: 626,
			resizable: false,
			closable: false,
			shadow: false,
			modal: true,
			items: {
				xtype: 'tabpanel',
				headerCfg: {
					cls: 'sb-editor-main-navigation'
				},
				border: false,
				items: [{
					xtype: 'widgetspanel',
					tabCls: 'sb-editor-widgets-tab',
					items: this.widgets
				}, {
					xtype: 'stylepanel',
					tabCls: 'sb-editor-style-tab'
				}, {
					xtype: 'pagespanel',
					tabCls: 'sb-editor-pages-tab'
				}, {
					xtype: 'documentspanel',
					tabCls: 'sb-editor-documents-tab'
				}, {
					xtype: 'settingspanel',
					ref: 'settingsPanel',
					tabCls: 'sb-editor-settings-tab'
				}],
				onStripMouseDown: function (e) {
					if (e.button !== 0) {
						return;
					}
					e.preventDefault();
					var t = this.findTargets(e);
					if (!t.item) {
						return;
					}
					if (t.item != this.activeTab) {
						this.setActiveTab(t.item);
					} else {
						this.setActiveTab(null);
					}
				},
				listeners: {
					scope: this,
					tabchange: this.onTabChange,
					beforetabchange: this.onBeforeTabChange
				}
			},
			plugins : plugins,
			tbar: this.statusBar,
			bbar: [{
				xtype: 'tbtext',
				cls: 'sb-helper-text'
			}],
			buttonAlign: 'left',
			fbar: [{
					id: 'sb-editor-show-dashboard',
					text: Sb.Locale.get('dashboard'),
					tooltip: Sb.Editor.isDemoMode()
						? Sb.Locale.get('dashboardMenuItemDisabled')
						: Sb.Locale.get('dashboardMenuItemDescription'),
					disabled: Sb.Editor.isDemoMode() || !this.dashboardManager.isAvailable(),
					scope: this,
					handler: this.showDashboard
				},
				Sb.editor.Snapshot.generateSnapshotButton('save'),
				Sb.editor.Snapshot.generateSnapshotButton('load'), {
					text: Sb.Locale.get('help'),
					tooltip: Sb.Locale.get('helpMenuItemDescription', Sb.Config.general.product),
					menu: new Ext.menu.Menu({
						items: [{
							id: 'sb-editor-help-btn',
							text: Sb.Locale.get('openUserGuide'),
							tooltip: Sb.Locale.get('helpDescription'),
							handler: function () {
								var articleUrl		= '';
								var activeWidget	= Sb.Editor.Ui.editComponent;
								if (null != activeWidget) {
									// Active widget
									articleUrl		= 'Widgets/' + activeWidget.xtype.toUpperCaseFirst();
								} else {
									// Panel and it`s tabs
									articleUrl		= 'Panel';
									var activeTab	= this.tabs.getActiveTab();
									if (null != activeTab) {
										var activeTabKey	= activeTab.itemId;
										if (null != activeTabKey) {
											articleUrl		+= '/' + activeTabKey.toUpperCaseFirst();
										}
										if (null != activeTab.tabs) {
											activeTabKey	= activeTab.tabs.getActiveTab().itemId;
											if (null != activeTabKey) {
												articleUrl	+= '/' + activeTabKey.toUpperCaseFirst();
											}
										}
									}
								}
								Sb.Editor.openHelp(articleUrl);
							}.createDelegate(this)
						}, {
							text: Sb.Locale.get('gettingStarted'),
							tooltip: Sb.Locale.get('gettingStartedDescription'),
							handler: Sb.Editor.showGettingStarted,
							hidden: !Sb.Config.gettingStartedDemoEnabled
						}, {
							text: Sb.Locale.get('supportButton'),
							tooltip: Sb.Locale.get('supportButtonDescription'),
							hidden: !Sb.Config.supportUrl,
							handler: function () {
								window.open(
									Sb.Config.supportUrl.replace('%%SITE_UUID%%', Sb.Editor.model.get('uuid'))
								);
							}
						}],
						cls: 'sb-editor-panel-menu',
						listeners: {
							scope: this,
							mouseover: menuItemOver,
							mouseout: this.restoreHelperMessage
						}
					}),
					cls: 'sb-editor-panel-menu-button'
				}, {
					cls: 'sb-editor-panel-menu-button',
					text: Sb.Locale.get('more'),
					tooltip: Sb.Locale.get('moreMenuItemDescription'),
					menu: new Ext.menu.Menu({
						items: this.moreMenuItems(),
						cls: 'sb-editor-panel-menu',
						listeners: {
							scope: this,
							mouseover: menuItemOver,
							mouseout: this.restoreHelperMessage
						}
					})
				}, '->', {
					id: 'sb-editor-minimize-btn',
					iconCls: 'sb-editor-minimize-btn-icon',
					text: Sb.Locale.get('hidePanel').toUpperCase(),
					tooltip: Sb.Locale.get('hidePanelDescription'),
					scope: this,
					handler: this.toggleMinimize
				}
			]
		});

		Ext.each(this.items.items, function (item) {
			if (item.xtype) {
				item.itemId = item.xtype.replace('panel', '');
			}
		});

		Sb.editor.MainWindow.superclass.initComponent.apply(this, arguments);

		/**
		 * Main navigation
		 * @type Ext.TabPanel
		 * @property tabs
		 */
		this.tabs = this.items.item(0);

		// Override quick tips to show it in helper
		Ext.override(Ext.QuickTip, {
			show: function () {
				Sb.Editor.Ui.mainWindow.setHelperMessage(this.activeTarget.text);
			},
			delayShow: function () {
				this.show();
			},
			hide: function () {
				if (!this.activeTarget) {
					return;
				}
				Sb.Editor.Ui.mainWindow.restoreHelperMessage();
				delete this.activeTarget;
			},
			delayHide: function () {
				this.hide();
			}
		});
	},

	moreMenuItems: function () {
		var menuItems = [{
			text: Sb.Locale.get('startOver'),
			tooltip: Sb.Locale.get('startOverDescription'),
			handler: function () {
				Sb.editor.UiContext.tryToSwitch(
					function () {
						Sb.Editor.Ui.mainWindow.tabs.setActiveTab(null);
						Ext.Msg.show({
							title    : Sb.Locale.get('titleConfirmStartingOver'),
							msg      : Sb.Locale.get('msgConfirmStartingOver'),
							closable : false,
							buttons  : Ext.Msg.YESNO,
							fn       : function (btn) {
								if (btn == 'yes') {
									Sb.Editor.startOver(false);
								}
							}
						});
					}
				);
			}
		}];

		if (Sb.Config.allowDeleteSite) {
			menuItems.push({
				text: Sb.Locale.get('removeSite'),
				tooltip: Sb.Locale.get('removeSiteDescription'),
				hidden: Sb.Editor.isDemoMode(),
				handler: function () {
					Sb.editor.UiContext.tryToSwitch(
						function () {
							Sb.Editor.Ui.mainWindow.tabs.setActiveTab(null);
							var message = Sb.Locale.get('msgConfirmRemovingSite');
							if (Sb.Config.general.deletePublishedSiteFiles) {
								message += '<br>' + Sb.Locale.get('msgConfirmPublishedSiteRemoval');
							}
							Ext.Msg.show({
								title    : Sb.Locale.get('titleConfirmRemovingSite'),
								msg      : message,
								closable : false,
								buttons  : Ext.Msg.YESNO,
								fn       : function (btn) {
									if (btn == 'yes') {
										Sb.Loading.updateMessage(Sb.Locale.get('loaderMsgRemovingSite'));
										Sb.Loading.show();
										Sb.SiteService['delete'](function () {
											window.location = Sb.Config.extension.urlToRedirectAfterDelete;
										});
									}
								}
							});
						}
					);
				}
			});
		}

		return menuItems;
	},

	// private
	onRender: function (ct, position) {
		var controlPanelLink = Sb.Config.general.controlPanelLink;

		Sb.editor.MainWindow.superclass.onRender.call(this, ct, position);

		this.publishButton = new Ext.SplitButton({
			renderTo: this.header,
			cls: 'sb-btn-action sb-publish',
			iconCls: 'sb-publish-icon',
			template: new Ext.Template(
				'<div id="{4}" class="x-btn {3}">',
				'<div class="button-wrapper"><button type="{0}"></button></div>',
				'<div class="split-wrapper"><em class="{2} x-unselectable" unselectable="on"></em></div>',
				'</div>'),
			scope: this,
			handler: this.publishSiteStart,
			menu: new Ext.menu.Menu({
				cls: 'sb-publish-menu',
				items: {
					text: Sb.Locale.get('mobileSimulatorTitle'),
					scope: this,
					handler: this.previewSiteStart
				},
				listeners: {
					show: function (menu) {
						menu.el.setStyle('min-width', (menu.ownerCt.getWidth() - 4) + 'px');
					}
				}
			})
		});

		if (controlPanelLink) {
			new Ext.Button({
				renderTo: this.header,
				text: Sb.Locale.get('controlPanelButtonText'),
				tooltip: Sb.Locale.get('controlPanelButtonTooltip'),
				cls: 'sb-btn-action sb-control-panel-button',
				iconCls: 'sb-publish-icon',
				handler: function() {
					window.location = controlPanelLink + (controlPanelLink.split('?').length == 1 ? '?' : '&') +
						'site_uuid=' + Sb.Editor.model.get('uuid');
				}
			});
		}
	},

	// private
	afterRender: function () {
		Sb.editor.MainWindow.superclass.afterRender.call(this);

		this.updatePublishButton();

		var tabsCount = this.tabs.items.length;
		var	tabElements = this.tabs.el.select('li:not(.x-tab-edge)');
		var firstTabId = tabElements.first().getAttribute('id');
		var lastTabId = tabElements.last().getAttribute('id');

		var tabsMargins = -20 * (tabsCount - 1); // 4 tabs by -20px margin-left
		var delta = this.tabs.header.dom.offsetWidth - tabsMargins + 12; //12 - width of shadows of side tabs to hide
		tabElements.each(function (tab) {
			delta -= tab.getWidth();
		});
		tabElements.each(function (tab) {
			tab = Ext.get(tab.id);
			tab.setStyle({
				width: (tab.getWidth() + delta / tabsCount) + 'px',
				marginLeft: '-20px'
			});

			tab.on('mouseenter', function (e) {
				var target = e.getTarget('li');

				if (target.id == firstTabId) {
					tabElements.setStyle({
						marginLeft: '-22px'
					});
					target.style.marginRight = '8px';
				} else if (target.id == lastTabId) {
					tabElements.setStyle({
						marginLeft: '-22px'
					});
					target.style.marginLeft = '-14px';
				} else {
					tabElements.setStyle({
						marginLeft: '-23px',
						marginRight: 0
					});
					target.style.marginLeft = '-16px';
					target.style.marginRight = '5px';
				}
			}, this);

			tab.on('mouseleave', function () {
				tabElements.setStyle({
					marginLeft: '-20px',
					marginRight: 0
				});
			}, this);

			tab.on('mousedown', function () {
				tabElements.setStyle({
					marginLeft: '-20px',
					marginRight: 0
				});
			}, this);
		}, this);

		this.tabs.items.each(function (tab) {
			tab.un('titlechange', this.tabs.onItemTitleChanged, this.tabs);
		}, this);

		this.mask.setOpacity(0);
		var topMessageBoxElement = Sb.TopMessageBox.getEl();
		this.alignTo(Ext.getBody(), 'tr-tr', [11, -7 + (topMessageBoxElement ? topMessageBoxElement.getHeight() : 0)]);

		this.clearHelperMessage();

		this.mask.on('click', function () {
			this.tabs.setActiveTab(null);
		}, this);

		/**
		 * Width for restoring
		 * @type Number
		 * @property restoreWidth
		 */
		this.restoreWidth = this.getWidth();

		this.header.on('dblclick', this.toggleMinimize, this);
	},

	beforeShow: function () {
		Sb.editor.MainWindow.superclass.beforeShow.call(this);
		this.mask[this.tabs.getActiveTab() instanceof Sb.editor.BaseFormPanel ? 'show' : 'hide']();
	},

	/**
	 * Shortcut for performing an {@link #minimize} or {@link #restore} based on the current state of the panel.
	 * @return {Sb.editor.MainWindow} this
	 */
	toggleMinimize: function () {
		this[this.minimized ? 'restore' : 'minimize']();
		return this;
	},

	/**
	 * Create and show dashboard window
	 */
	showDashboard: function () {
		if (!this.dashboardManager || this.dashboardManager.isDestroyed) {
			this.dashboardManager = new Sb.dashboard.Manager();
		}
		this.dashboardManager.showWindow();
	},

	/**
	 * Minimize a panel.
	 * @return {Sb.editor.MainWindow} this
	 */
	minimize: function () {
		if (!Sb.editor.UiContext.isMayBeSwitchedImmediately()) {
			Sb.editor.UiContext.tryToSwitch(this.minimize.createDelegate(this));
			return this;
		}

		this.tabs.setActiveTab(null);
		this.minimized = true;
		this.addClass('sb-editor-panel-minimized');
		this.restoreWidth = this.getWidth();
		this.setWidth(this.header.child('.sb-editor-panel-header-text').getWidth() + 60);
		this.setPosition(this.getPosition()[0] + this.restoreWidth - this.getWidth());
		Ext.getCmp('sb-editor-minimize-btn').setText(Sb.Locale.get('showPanel').toUpperCase());

		this.fireEvent('minimize', this);
		return this;
	},

	/**
	 * Restores a panel back to its original size and position prior to being minimized.
	 * @return {Sb.editor.MainWindow} this
	 */
	restore: function () {
		this.removeClass('sb-editor-panel-minimized');
		this.minimized = false;
		this.setPosition(this.getPosition()[0] - this.restoreWidth + this.getWidth());
		this.setWidth(this.restoreWidth);
		Ext.getCmp('sb-editor-minimize-btn').setText(Sb.Locale.get('hidePanel').toUpperCase());

		this.doConstrain(true);
		this.fireEvent('restore', this);
		return this;
	},

	/**
	 * Fires before the active tab changes. Handlers return false in order to cancel the tab change.
	 * @param {Ext.TabPanel} tabPanel Active tabpanel
	 * @param {Ext.Panel} newTab The tab being activated
	 * @param {Ext.Panel} currentTab The current active tab
	 */
	onBeforeTabChange: function (tabPanel, newTab, currentTab) {
		if (Sb.editor.UiContext.isMayBeSwitchedImmediately()) {
			return true;
		}
		Sb.editor.UiContext.tryToSwitch(tabPanel.setActiveTab.createDelegate(tabPanel, [newTab]));
		return false;
	},

	/**
	 * Fires after the active tab has changed.
	 * @param {Ext.TabPanel} tabPanel
	 * @param {Ext.Panel} tab
	 */
	onTabChange: function (tabPanel, tab) {
		this.mask[tab instanceof Sb.editor.BaseFormPanel ? 'show' : 'hide']();

		this.setHelperMessageHeight(0, true);
		this.statusBar.clear();

		if (tab) {
			if (tab.hidden) {
				tab.show();
			}
			if (tab.helperMessage) {
				this.setHelperMessage(tab.helperMessage, true);
			}
		} else {
			tabPanel.layout.activeItem.hide();
			tabPanel.layout.activeItem = null;
			this.clearHelperMessage();
		}

		this.restore();
	},

	/**
	 * Shows window with message for Try Before Buy scenarious
	 */
	showTBBMessage: function (title, message) {
		var win = new Sb.Window({
			title	: title,
			modal	: true,
			width	: 500,
			items	: {
				xtype: 'box',
				html: message
			},
			buttons	: [{
				text: Sb.Locale.get('Close'),
				handler: function () {
					win.close();
				}
			}]
		});
		win.show();
		win.items.first().getEl().update(message, true);
	},

	preparePublishing: function () {
		this.statusBar.clear();
		this.tabs.setActiveTab(null);
		Ext.getBody().mask().addClass('sb-editor-mask');

		if (!this.progressBar || this.progressBar.isDestroyed) {
			this.progressBar = new Sb.editor.MainWindow.FakeProgressBar({
				renderTo: this.getTopToolbar().container
			});
			this.getTopToolbar().add(this.progressBar);
		}
	},

	/**
	 * Publish site start, that initiates publish site finish action.
	 */
	publishSiteStart: function () {
		if (!Sb.editor.UiContext.isMayBeSwitchedImmediately()) {
			Sb.editor.UiContext.tryToSwitch(this.publishSiteStart.createDelegate(this));
			return;
		}
		Sb.Editor.model.updateUploadedImages();

		this.preparePublishing();

		var pagesCount = 0;
		Sb.Editor.model.rootPage.applyRecursive(function (){
			pagesCount++;
		});
		this.progressBar.wait({
			duration	: 1000 + Sb.Config.pagePublicationTime * 1000 * pagesCount,
			iterval		: 80,
			localeKey	: 'publishingSite'
		});

		Sb.model.apiProvider.runAfterLastCall(this.publishSiteFinish, this);
	},

	/**
	 * Actual publish site work is here, that initiated by publish site start.
	 */
	publishSiteFinish: function () {
		var site		= Sb.Editor.model,
			publishUrl	= site.get('publishWebSiteUrl'),
			publishLink	= '<a href="' + publishUrl + '" target="_blank">' + site.get('publishWebSiteUrl_UTF8') + '</a>',
			statusBar	= this.statusBar;
		Sb.SiteService.publish(function (data, e) {
			this.progressBar.gotoEnd({
				localeKey: 'publishingSite',
				fn: function () {
					this.progressBar.destroy();
					Ext.getBody().unmask();
					var msg = {};
					if (data) {
						if (data._subStatusText) {
							this.showTBBMessage(data._subStatusText, data._statusText);
						} else {
							if (2 != data._statusLevel) {
								// Not error, site was published

								var statForm = Ext.getDom('statForm');
								if (!statForm) {
									statForm = document.createElement('form');
									statForm.id = 'statForm';
									statForm.style.cssText = 'display: none; ' +
										'position: absolute; z-index: 15000; left: -10000px; top: -10000px;';
									statForm.method = 'POST';
									document.body.appendChild(statForm);
								}
								Ext.Ajax.request({
									url: '//stat.llsitebuilder.com/1/',
									params: {
										licenseKeyNumber: Sb.Config.licenseKeyNumber,
										pa4wpUrl: self.location.href,
										webSiteUrl: publishUrl,
										wstCode: Sb.Config.initialWstCode,
										isCustom: (Sb.Config.isCustomInitialWst) ? 1 : 0,
										wpbVersion: Sb.Config.general.version
									},
									form: statForm,
									isUpload: true
								});
							}

							switch (data._statusLevel) {
								case 0:
								case 1:
									msg.text = Sb.Locale.get('siteSuccessfullyPublished', publishLink);
									if (Sb.Config.general.facebookApplicationUrl
										&& !(site.get('isFacebookIntegration') && site.get('isPublished')
											&& Ext.state.Manager.get('isNotShowFBIntegration')
										)
									) {
										this.fbDrawer.show();
									}
									site.set('isPublished', true);
									break;
								case 2:
									this.progressBar.publishResult = data;
									msg = {text: data._statusText, level: Sb.editor.StatusBar.ERROR};
									break;
								case 3:
									this.progressBar.publishResult = data;
									msg = {
										text	: Sb.Locale.get('sitePublishedWithWarnings', publishLink),
										level	: Sb.editor.StatusBar.WARNING,
										closeOn	: 'message-control'
									};
									break;
							}
							if (data._messagesDetailed.length) {
								msg.text += ' <a href="#" onclick="javascript:Sb.Editor.showPublishErrorDetails();">' +
									Sb.Locale.get('publishSeeDetails') + '</a>';
							}
							statusBar.setMessage(msg);
						}
					}
					Sb.WidgetService.removeUnusedUploadedFiles(
						Sb.Config.path.imageFolder, Sb.Editor.model.getUploadedImagesSources()
					);
				},
				scope: this
			});
		}, this);
	},

	previewSiteStart: function () {
		if (!Sb.editor.UiContext.isMayBeSwitchedImmediately()) {
			Sb.editor.UiContext.tryToSwitch(this.previewSiteStart.createDelegate(this));
			return;
		}

		this.preparePublishing();

		this.progressBar.wait({
			duration	: 10000,
			iterval		: 80,
			localeKey	: 'previewingSite'
		});

		Sb.model.apiProvider.runAfterLastCall(this.previewSiteFinish, this);
	},

	previewSiteFinish: function () {
		Sb.SiteService.preview(function (data, e) {
			this.progressBar.gotoEnd({
				localeKey: 'previewingSite',
				fn: function () {
					this.progressBar.destroy();
					Ext.getBody().unmask();
					Sb.Editor.Ui.mobileSimulator.show();
				},
				scope: this
			});
		}, this);
	},

	/**
	 * Update helpers's message.
	 * @param {String} msg The text to display in helper.
	 * @param {Boolean} [isPermanent=false] Is the text will be permanent?
	 */
	setHelperMessage: function (msg, isPermanent) {
		this.setHelperMessageHeight(0);

		if (isPermanent) {
			this.permanentHelperMessage = msg;
		}
		this.bottomToolbar.items.items[0].setText(msg);

		var height = this.getHelperMessageHeight();
		if (height > this.helperMessageHeight) {
			this.helperMessageHeight = height;
		}
		this.setHelperMessageHeight(this.helperMessageHeight);
	},

	/**
	 * Restore helper message from previous message
	 */
	restoreHelperMessage: function () {
		this.setHelperMessage(this.permanentHelperMessage);
	},

	/**
	 * Clear helper's message.
	 */
	clearHelperMessage: function () {
		this.permanentHelperMessage = Sb.Locale.get('helperDefaultMessage', Sb.Config.general.product);
		this.setHelperMessage(this.permanentHelperMessage);
	},

	/**
	 * Get helpers's message area height.
	 * @return {Number} Height of message area.
	 */
	getHelperMessageHeight: function () {
		return this.bottomToolbar.el.child('table').getHeight();
	},

	/**
	 * Set helpers's message area height.
	 * @param {Number} height Height of message area.
	 * @param {Boolean} [setModelHeight] If set height to model height too.
	 */
	setHelperMessageHeight: function (height, setModelHeight) {
		if (setModelHeight) {
			this.helperMessageHeight = height;
		}
		this.bottomToolbar.el.child('table').setHeight((height === 0) ? '' : height);
	},

	initDraggable: function () {
		this.dd = new Sb.editor.MainWindow.DD(this);
	},

	/**
	 * Make editor panel always visible, even after browser resize
	 *
	 * @param {Boolean} isWholeWindowVisible Do we force to show whole window on screen?
	 */
	doConstrain: function (isWholeWindowVisible) {
		if (undefined === isWholeWindowVisible) {
			isWholeWindowVisible = false;
		}

		var containerSize		= this.container.getSize();
		var windowSize			= this.getSize();
		var windowPosition		= newWindowPosition = this.getPosition();
		var delta				= {
			x			: containerSize.width - (isWholeWindowVisible ? windowSize.width : this.visibleOffsetX),
			y			: containerSize.height - (isWholeWindowVisible ? windowSize.height : this.visibleOffsetY)
		};
		delta.x					= delta.x < 0 ? 0 : delta.x;
		delta.y					= delta.y < 0 ? 0 : delta.y;

		var distanceToVisible	= {
			fromRight	: windowPosition[0] - delta.x,
			fromBottom	: windowPosition[1] - delta.y
		};

		var isNeedToMove = {
			toLeft		: (distanceToVisible.fromRight > 0),
			toTop		: (distanceToVisible.fromBottom > 0)
		};

		if (isNeedToMove.toLeft || isNeedToMove.toTop) {
			if (isNeedToMove.toLeft) {
				newWindowPosition[0] = delta.x;
			}
			if (isNeedToMove.toTop) {
				newWindowPosition[1] = delta.y;
			}
			this.setPosition(newWindowPosition);
		}
	},

	createGhost: function () {
		return this.container.createChild({
			cls: 'x-panel-ghost sb-editor-panel-ghost', cn: {}
		}).setSize(this.getSize());
	},

	updatePublishButton: function() {
		var upsellCls = 'sb-publish-upsell',
			isUpsell = false;

		if (
			Sb.Limits.checkWidgets(Sb.Editor.model).isExceeded
			|| Sb.Limits.checkPages(Sb.Editor.model).isExceeded
			|| Sb.Limits.checkLocales(Sb.Editor.model, Sb.Editor.model.get('locales')).isExceeded
		) {
			isUpsell = true;
		}

		if (isUpsell) {
			this.publishButton
				.addClass(upsellCls)
				.setText(Sb.Locale.get('publishUpsell'))
				.setTooltip(Sb.Locale.get('publishUpsellDescription'));
		} else {
			this.publishButton
				.removeClass(upsellCls)
				.setText(Sb.Locale.get('publish'))
				.setTooltip(Sb.Locale.get('publishDescription'));
		}
	}
});

/**
 * Class for main window drag & drop
 * @class Sb.editor.MainWindow.DD
 * @extends Ext.dd.DD
 */
Ext.define('Sb.editor.MainWindow.DD', {
	extend: 'Ext.Window.DD',

	startDrag: function () {
		var w = this.win;
		w.fireEvent('ghost', []);
		this.proxy = w.ghost(w.initialConfig.cls);
		var windowSize = w.el.getSize();
		var widthOffset = -(windowSize.width - w.visibleOffsetX);
		var heightOffset = -(windowSize.height - w.visibleOffsetY);
		var headerSize = w.header.getSize();
		this.constrainTo(w.container, {
			top: -(headerSize.height - w.visibleOffsetY),
			bottom: heightOffset,
			left: widthOffset,
			right: widthOffset
		});
		this.iframeFix = Ext.DomHelper.append(document.body, {tag: 'div', cls: 'draggable-iframe-overlay'});
	},

	endDrag: function(e) {
		document.body.removeChild(this.iframeFix);
		Sb.editor.MainWindow.DD.superclass.endDrag.call(this);
	}
});
