/**
 * @class Sb.editor.PagesPanel
 * @extends Sb.editor.BaseFormPanel
 * Site pages panel
 * @constructor
 * @param {Object} config The config object
 * @xtype pagespanel
 */
Ext.define('Sb.editor.PagesPanel', {
	extend: 'Sb.editor.BaseFormPanel',
	xtype: 'pagespanel',
	currentPage: null,

	// private
	initComponent: function () {
		this.nodesValidity = {};

		this.treeSelectionModel = new Ext.tree.DefaultSelectionModel({
			listeners: {
				scope: this,
				beforeselect: function () {
					this.slugField.onBlur();
				},
				selectionchange: function (sm, node) {
					if (!node) {
						return;
					}

					var isLinkPage = node.attributes.model.get('sitePageTypeId') == Sb.model.SitePage.LINK_TYPE;
					this.browserTitleField.setVisible(!isLinkPage);
					this.slugField.setVisible(!isLinkPage);
					this.urlField.setVisible(isLinkPage);
					this.urlField.allowBlank = !isLinkPage;
					this.targetField.setVisible(isLinkPage);
					this.descriptionBox.setVisible(!isLinkPage);
					this.protectedField.setVisible(!isLinkPage);
					Ext.getCmp('openPageButton').setDisabled(isLinkPage);

					this.currentPage = node.attributes.model;
					this.protectedField.suspendEvents();
					this.suspendChanges = true;
					this.getForm().setValues(Ext.apply({
						'protected': this.currentPage.isProtected()
					}, this.currentPage.data));
					this.protectedField.resumeEvents();
					this.suspendChanges = false;
				}
			}
		});

		Ext.apply(this, {
			cls: 'sb-editor-pages-panel',
			title: Sb.Locale.get('pagesTitle'),
			helperMessage: Sb.Locale.get('pagesDescription'),
			tabTip: Sb.Locale.get('pagesDescriptionOnHover'),
			labelAlign: 'top',
			layout: 'column',
			autoHeight: true,
			defaults: {
				border: false,
				buttonAlign: 'left'
			},
			items: [{
				columnWidth: 0.6,
				ref: 'pagesTree',
				xtype: 'treepanel',
				baseCls: 'sb-pages-tree',
				autoScroll: true,
				animate: true,
				containerScroll: true,
				enableDD: true,
				lines: false,
				loader: new Ext.tree.TreeLoader({
					doPreload: function (node) {
						node.beginUpdate();
						node.attributes.model.pages.each(function (page) {
							var masterWidget = page.getMasterWidget(),
								allowChildren = true,
								pageType = masterWidget ? masterWidget.get('type').camelize().toLowerCase() :
									(page.get('sitePageTypeId') == Sb.model.SitePage.LINK_TYPE ? 'link' : 'text'),
								data = page.data,
								cls = [],
								attr,
								isEmpty,
								newNode;

							if (masterWidget instanceof Sb.model.BlogPostWidget) {
								return true;
							}
							if (masterWidget instanceof Sb.model.BlogWidget
								|| page.get('sitePageTypeId') == Sb.model.SitePage.LINK_TYPE
							) {
								allowChildren = false;
							}

							if (data.hidden) {
								cls.push('sb-pages-tree-node-hidden');
							}
							if (page.isProtected()) {
								cls.push('sb-pages-tree-node-protected');
							}
							attr = {
								expanded		: true,
								id				: data.identity,
								text			: data.title,
								iconCls			: 'sb-' + pageType + '-icon',
								cls				: cls.join(' '),
								model			: page,
								maxDepth		: Sb.Config.pageMaxLevel,
								allowChildren	: allowChildren,
								uiProvider		: Sb.editor.PagesTreeNodeUI
							};
							isEmpty	= page.pages.getCount() === 0;
							newNode = isEmpty ? new Ext.tree.TreeNode(attr) : new Ext.tree.AsyncTreeNode(attr);

							node.appendChild(newNode);
						});
						node.endUpdate();
						return true;
					}
				}),
				rootVisible: false,
				root: {
					id: Sb.Editor.model.rootPage.data.identity,
					model: Sb.Editor.model.rootPage,
					loaded: true
				},
				selModel: this.treeSelectionModel,
				listeners: {
					movenode: function (tree, node, oldParent, newParent, index) {
						newParent.attributes.model.insert(index, node.attributes.model);
						Sb.editor.UiContext.registerChanges();
						node.select();
					},
					startdrag: function (panel, node) {
						var el = panel.dragZone.proxy.el;
						el.disableShadow();
						el.setXY(el.getXY());
						var ghost = panel.dragZone.proxy.ghost;
						ghost.addClass('sb-pages-tree-drag-ghost');
						ghost.setWidth(Ext.get(node.ui.wrap).getWidth());
					},
					beforemovenode: function (tree, node, oldParent, newParent) {
						return node.getUI().allowMoveToNode(newParent);
					}
				},
				tbar: {
					items: {
						xtype: 'languagebuttons',
						ref: '../../lngButtons',
						value: Sb.Editor.siteView.getCurrentSiteLocale(),
						data: Sb.Editor.model.get('locales'),
						listeners: {
							scope: this,
							selectionchange: function (cmp) {
								this.changeLocale(cmp.getValue());
								this.updateTree();
							}
						}
					}
				}
			}, {
				columnWidth: 0.4,
				layout: 'form',
				baseCls: 'sb-pages-page-properties',
				layoutConfig: {
					labelSeparator: ''
				},
				defaults: {
					validateOnBlur: false
				},
				items: [{
					ref: '../titleField',
					xtype: 'textfield',
					fieldLabel: Sb.Locale.get('pageTitle'),
					name: 'title',
					maxLength: 255,
					allowBlank: false,
					anchor: '100%',
					msgTarget: 'under',
					listeners: {
						scope: this,
						valid: this.onChange,
						focus: function () {
							Sb.Editor.Ui.mainWindow.setHelperMessage(Sb.Locale.get('messagePageTitle'));
						}
					}
				}, {
					ref: '../browserTitleField',
					xtype: 'textarea',
					emptyClass: '',
					fieldLabel: Sb.Locale.get('pageBrowserTitle'),
					name: 'browserTitle',
					maxLength: 255,
					anchor: '100%',
					style: 'height: 34px',
					msgTarget: 'under',
					listeners: {
						scope: this,
						valid: this.onChange,
						focus: function (field) {
							if (field.getValue() === '') {
								field.setRawValue(field.emptyText);
							}
							Sb.Editor.Ui.mainWindow.setHelperMessage(Sb.Locale.get('messagePageBrowserTitle'));
						}
					}
				}, {
					ref: '../slugField',
					xtype: 'textfield',
					fieldLabel: Sb.Locale.get('pageSlug'),
					name: 'slug',
					maxLength: 255,
					regex: /^[a-z0-9\-_]+$/i,
					maskRe: /[a-z0-9\-_]/i,
					anchor: '100%',
					msgTarget: 'under',
					validator: this.pageSlugValidator.createDelegate(this),
					listeners: {
						scope: this,
						valid: this.onChange,
						blur: function (field) {
							var node, page;
							if (field.getValue() !== '') {
								return;
							}
							node = this.treeSelectionModel.getSelectedNode();
							if (null === node) {
								return;
							}
							page = node.attributes.model;
							if (page.get('sitePageTypeId') == Sb.model.SitePage.LINK_TYPE) {
								return;
							}
							if (page.set('slugIsAuto', '1')) {
								Sb.editor.UiContext.registerChanges();
							}
							this.createSlug(page, page.data.title);
						}
					}
				}, {
					ref: '../urlField',
					xtype: 'textfield',
					fieldLabel: Sb.Locale.get('pageUrl'),
					name: 'url',
					maxLength: 255,
					anchor: '100%',
					msgTarget: 'under',
					validator: this.pageSlugValidator.createDelegate(this),
					listeners: {
						scope: this,
						invalid: this.markTreeNodeAsInvalid,
						valid: this.onChange
					}
				}, {
					ref: '../targetField',
					xtype: 'combo',
					fieldLabel: Sb.Locale.get('pageTarget'),
					name: 'target',
					anchor: '100%',
					msgTarget: 'under',
					typeAhead: true,
					triggerAction: 'all',
					lazyRender: true,
					mode: 'local',
					value: '',
					store: new Ext.data.ArrayStore({
						fields: ['value', 'title'],
						data: [
							['', Sb.Locale.get('linkEditDialogTargetSelf')],
							['_blank', Sb.Locale.get('linkEditDialogTargetBlank')]
						]
					}),
					valueField: 'value',
					displayField: 'title',
					listeners: {
						scope: this,
						select: this.onChange
					}
				}, {
					ref: '../descriptionBox',
					xtype: 'box',
					cls: 'x-form-item',
					html: '<a href="#PageDescriptionAndKeywords">' +
						Ext.util.Format.htmlEncode(Sb.Locale.get('pageEditDescriptionAndKeywords')) + '</a>',
					listeners	: {
						scope: this,
						afterrender: function () {
							this.el.child('a').on('click', this.showDescriptionAndKeywordsWindow, this, {stopEvent: true});
						}
					}
				}, {
					ref: '../hiddenField',
					xtype: 'checkbox',
					name: 'hidden',
					hideLabel: true,
					boxLabel: Sb.Locale.get('pageHidden'),
					listeners: {
						check: this.onChange.createDelegate(this),
						afterrender: function () {
							var fn = Sb.Editor.Ui.mainWindow.setHelperMessage.createDelegate(
								Sb.Editor.Ui.mainWindow,
								[Sb.Locale.get('hiddenPageTooltip')]
							);
							this.el.on('mouseenter', fn);
							this.el.on('mouseleave', Sb.Editor.Ui.mainWindow.restoreHelperMessage, Sb.Editor.Ui.mainWindow);

							this.labelEl.on('mouseenter', fn);
							this.labelEl.on('mouseleave', Sb.Editor.Ui.mainWindow.restoreHelperMessage, Sb.Editor.Ui.mainWindow);
						}
					}
				}, {
					ref			: '../protectedField',
					xtype		: 'checkbox',
					name		: 'protected',
					itemCls		: 'sb-pages-protected-field',
					hideLabel	: true,
					boxLabel	: [
						Sb.Locale.get('passwordProtectPageTitle'), '<a href="#setCredentials">',
						Ext.util.Format.htmlEncode(Sb.Locale.get('passwordProtectPageSetCredentials')), '</a>'
					].join(' '),
					listeners	: {
						check		: this.protectPage.createDelegate(this),
						afterrender	: function (fld) {
							var fn = Sb.Editor.Ui.mainWindow.setHelperMessage.createDelegate(
								Sb.Editor.Ui.mainWindow,
								[Sb.Locale.get('passwordProtectPageDescription')]
							);
							fld.el.on('mouseenter', fn);
							fld.el.on('mouseleave', Sb.Editor.Ui.mainWindow.restoreHelperMessage, Sb.Editor.Ui.mainWindow);

							fld.labelEl.on('mouseenter', fn);
							fld.labelEl.on('mouseleave', Sb.Editor.Ui.mainWindow.restoreHelperMessage, Sb.Editor.Ui.mainWindow);
							fld.labelEl.child('a').on('click', function (e) {
								e.stopEvent();
								this.protectPage(fld);
							}, this);
						}.createDelegate(this)
					}
				}]
			}, {
				columnWidth: 0.6,
				defaults: {
					xtype: 'container',
					cls: 'sb-pages-page-button x-panel-btns'
				},
				items: [{
					items: [{
						xtype: 'splitbutton',
						id: 'addPageButton',
						text: Sb.Locale.get('addPage'),
						tooltip: Sb.Locale.get('addPageTooltip'),
						iconCls: 'sb-add-icon',
						handler : this.createPage.createDelegate(this, ['text']),
						menu: {
							items: [{
								text: Sb.Locale.get('addLink'),
								tooltip: Sb.Locale.get('addLinkTooltip'),
								iconCls: 'sb-link-icon',
								handler: this.createPage.createDelegate(this, ['link'])
							}],
							listeners: {
								scope: this,
								mouseover: function (menu, e, menuItem) {
									if (menuItem && menuItem.tooltip) {
										Sb.Editor.Ui.mainWindow.setHelperMessage(menuItem.tooltip);
									}
								},
								mouseout: function () {
									Sb.Editor.Ui.mainWindow.restoreHelperMessage();
								}
							}
						}
					}, {
						xtype: 'box',
						id: 'addPageButtonUpsellIcon',
						cls: 'sb-add-page-upsell-icon',
						hidden: true
					}]
				}, {
					items: [{
						xtype: 'button',
						text: Sb.Locale.get('deletePage'),
						tooltip: Sb.Locale.get('deletePageTooltip'),
						iconCls: 'sb-delete-icon',
						handler : this.deletePage.createDelegate(this)
					}]
				}]
			}, {
				columnWidth: 0.4,
				layout: 'form',
				baseCls: 'sb-pages-page-buttons',
				buttons: [{
					id: 'openPageButton',
					text: Sb.Locale.get('openPage'),
					tooltip: Sb.Locale.get('openPageTooltip'),
					iconCls: 'sb-open-page-icon',
					scope: this,
					handler: function () {
						Sb.Editor.siteView.setCurrentPage(
							this.treeSelectionModel.getSelectedNode().attributes.model
						);
					}
				}]
			}]
		});

		/**
		 * Object for saving last entered credentials for prefilling it later
		 * @property lastSavedCredentials
		 * @type Object
		 */
		this.lastSavedCredentials = {};

		Sb.editor.PagesPanel.superclass.initComponent.apply(this, arguments);
	},

	pageSlugValidator: function (slug) {
		if (slug === '') {
			return true;
		}
		var page = this.treeSelectionModel.getSelectedNode().attributes.model,
			msg = true;
		if (Sb.Config.invalidSlugs.indexOf(slug.toLowerCase()) !== -1
			|| page.parentPage === Sb.Editor.model.rootPage
			&& (
				Sb.Config.reservedSlugs.indexOf(slug.toLowerCase()) !== -1
				|| (/^[a-z]{2}-[a-z]{2}$/).test(slug.toLowerCase())
			)
		) {
			return Sb.Locale.get('pageSlugReserved');
		}
		page.parentPage.pages.each(function (curPage) {
			if (page !== curPage && curPage.data.slug === slug) {
				msg = Sb.Locale.get('pageSlugDuplicate');
				return false;
			}
		});
		return msg;
	},

	/**
	 * Activate handler
	 */
	onActivate: function () {
		this.memento = Sb.Editor.model.rootPage.saveToMemento();

		this.currentPage = Sb.Editor.siteView.currentPage;
		this.changeLocale(Sb.Editor.siteView.getCurrentSiteLocale());
		this.updateLngButtons();
		this.updateTree();
		this.updateAddPageButton();

		Sb.editor.PagesPanel.superclass.onActivate.apply(this, arguments);
	},

	changeLocale: function (localeCode) {
		Sb.Editor.model.rootPage.applySelfAndRecursive(function (page) {
			page.changeLocale(localeCode);
		});
	},

	updateLngButtons: function () {
		var locales = Sb.Editor.model.get('locales');
		this.lngButtons
			.setData(locales)
			.setValue(Sb.Editor.siteView.getCurrentSiteLocale())
			.setVisible(locales.length > 1);
	},

	updateTree: function () {
		var rootNode = this.pagesTree.getRootNode();

		rootNode.reload();
		rootNode.findChildBy(function (node) {
			return node.attributes.model.get('identity') === this.currentPage.get('identity');
		}, this, true).select();
	},

	markTreeNodeAsInvalid: function (field) {
		this.nodesValidity[this.treeSelectionModel.getSelectedNode().id] = false;
	},

	areAllNodesValid: function () {
		var res = true;
		Ext.iterate(this.nodesValidity, function (nodeId, isFormValid) {
			if (!isFormValid) {
				res = false;
				return false;
			}
		});
		return res;
	},

	/**
	 * Change handler
	 *
	 * @param {Ext.form.Field} field
	 */
	onChange: function (field) {
		var node = this.treeSelectionModel.getSelectedNode(),
			page = node.attributes.model;
		this.nodesValidity[node.id] = true;
		switch (field.name) {
			case 'title':
				node.setText(Ext.util.Format.htmlEncode(
					Ext.util.Format.ellipsis(field.getValue(), 27 - node.getDepth() * 2)
				));
				if (this.browserTitleField.getValue() === '') {
					this.browserTitleField.setRawValue(field.getValue());
				}
				this.browserTitleField.emptyText = field.getValue();

				if (!this.suspendChanges && page.get('slugIsAuto') === '1' &&
					this.lngButtons.getValue() === Sb.Editor.model.get('locale')
				) {
					this.createSlug(page, field.getValue());
				}
				break;

			case 'slug':
				if (!this.suspendChanges && field.getValue() !== '' && page.data.slugIsAuto === '1') {
					if (page.set('slugIsAuto', 0)) {
						Sb.editor.UiContext.registerChanges();
					}
				}
				break;

			case 'hidden':
				node.ui[(field.getValue() ? 'add' : 'remove') + 'Class']('sb-pages-tree-node-hidden');
				break;
		}

		if (!this.suspendChanges && page.set(field.name, field.getValue())) {
			Sb.editor.UiContext.registerChanges();
		}
	},

	createSlug: function (page, title) {
		Sb.PageService.createSlug(title, function (slug) {
			var re = new RegExp('^' + Ext.escapeRe(slug) + '(?:-(\\d+))?$', 'ig');
			if (re.test(page.get('slug'))) {
				return;
			}

			if (page.set('slug', slug)) {
				page.makeSlugUnique();
				Sb.editor.UiContext.registerChanges();
			}
			if (page === this.treeSelectionModel.getSelectedNode().attributes.model) {
				this.slugField.setRawValue(page.get('slug'));
			}
		}, this);
	},

	/**
	 * Create page by type
	 * @param {String} type
	 */
	createPage: function (type) {
		var checkPages,
			countPages = this.getCountPages(),
			mainLocaleCode = Sb.Editor.model.get('locale'),
			currentNode,
			newPage,
			newPageNode;

		if (-1 !== Sb.Config.maxPagesNumber) {
			if (countPages + 1 > Sb.Config.maxPagesNumber) {
				Sb.Editor.Ui.mainWindow.statusBar.setMessageError(
					Sb.Locale.get('maxPagesNumberExceeded', countPages, Sb.Config.maxPagesNumber)
				);
				return;
			}
		}

		if (mainLocaleCode !== this.lngButtons.getValue()) {
			this.lngButtons.setValue(mainLocaleCode);
			this.changeLocale(mainLocaleCode);
			this.updateTree();
		}

		switch (type) {
			case 'text':
				newPage = Sb.Editor.getController('Pages').createPage();
				this.addTextWidget(newPage, 'content');
				this.addTextWidget(newPage, 'column1');
				this.addTextWidget(newPage, 'column2');
				break;

			case 'link':
				newPage = new Sb.model.SitePage({
					sitePageTypeId: Sb.model.SitePage.LINK_TYPE
				});
				break;
		}

		newPage.set('title', Sb.Locale.get('newTextPage'));
		newPage.set('url', 'http://example.com');
		newPageNode = new Ext.tree.TreeNode({
			id				: newPage.get('identity'),
			text			: newPage.get('title'),
			iconCls			: 'sb-' + type + '-icon',
			model			: newPage,
			maxDepth		: Sb.Config.pageMaxLevel,
			allowChildren	: type != 'link',
			uiProvider		: Sb.editor.PagesTreeNodeUI
		});

		currentNode = this.treeSelectionModel.getSelectedNode();
		currentNode.parentNode.insertBefore(newPageNode, currentNode.nextSibling);
		currentNode.parentNode.attributes.model.insert(currentNode.parentNode.indexOf(newPageNode), newPage);
		if (type == 'text') {
			this.createSlug(newPage, newPage.get('title'));
			newPage.set('slugIsAuto', '1');
		}
		Sb.editor.UiContext.registerChanges();
		newPageNode.select();

		this.updateAddPageButton();
		checkPages = Sb.Limits.checkPages(Sb.Editor.model);
		if (checkPages.isExceeded) {
			Sb.Editor.Ui.mainWindow.statusBar.setMessageWarning(
				Sb.Locale.get(
					'maxPagesNumberLimitExceeded',
					checkPages.count,
					checkPages.limit
				)
			);
			Sb.Editor.Ui.mainWindow.updatePublishButton();
		}
	},

	/**
	 * Add new text widget with empty content
	 * @param {Sb.model.SitePage} page
	 * @param {String} region
	 */
	addTextWidget: function (page, region) {
		var widgetContainer = page.widget.widgets.find(function (container) {
			return container.data.region === region;
		});

		var contentWidget = new Sb.model.TextWidget({
			styles: {
				padding: '0px 10px 0px 10px',
				margin: '0px 0px 0px 0px'
			},
			visibilityMode: 'all',
			content: ''
		});

		widgetContainer.widgets.insert(0, contentWidget);
	},

	/**
	 * Delete page from model and tree
	 */
	deletePage: function () {
		var rootNode = this.pagesTree.getRootNode();
		var nodeForDelete = this.treeSelectionModel.getSelectedNode();
		if (!nodeForDelete) {
			return;
		}
		if ((nodeForDelete.parentNode === rootNode)
			&& (nodeForDelete.attributes.model.get('sitePageTypeId') != Sb.model.SitePage.LINK_TYPE)
			&& (rootNode.childNodes.reduce(function (count, node) {
				return count + (node.attributes.model.get('sitePageTypeId') == Sb.model.SitePage.LINK_TYPE ? 0 : 1);
			}, 0) === 1)
		) {
			Sb.Editor.Ui.mainWindow.statusBar.setMessageInfo(Sb.Locale.get('cannotRemoveAllPages'));
			return;
		}

		var nodeForSelect = nodeForDelete.nextSibling || nodeForDelete.previousSibling || nodeForDelete.parentNode;

		//:INFO: workaround for bug #44214
		var c = this.pagesTree.body.dom;
		var offset = c.scrollHeight - c.scrollTop - c.clientHeight - nodeForDelete.ui.wrap.clientHeight;
		if (offset < 0) {
			c.scrollTop += offset;
		}

		nodeForDelete.parentNode.attributes.model.remove(nodeForDelete.attributes.model);
		nodeForDelete.parentNode.removeChild(nodeForDelete);

		nodeForDelete.attributes.model.applySelfAndRecursive(function (page) {
			if (page.data.identity === Sb.Editor.siteView.currentPage.data.identity) {
				Sb.Editor.siteView.setCurrentPage(
					nodeForSelect.attributes.model
				);
			}
		});
		nodeForSelect.select();

		this.updateAddPageButton();
		Sb.Editor.Ui.mainWindow.updatePublishButton();

		Sb.editor.UiContext.registerChanges();
	},

	/**
	 * Open protect page dialog or unprotect page
	 *
	 * @param {Ext.form.Field} field
	 */
	protectPage: function (field) {
		var node		= this.treeSelectionModel.getSelectedNode(),
			page		= node.attributes.model,
			data		= page.data,
			saveHandler = function (dlg) {
				var credentials	= {
						login		: dlg ? dlg.loginField.getValue() : null,
						password	: dlg ? dlg.passwordField.getValue() : null
					},
					isSaved = page.unserialize(credentials);
				if (isSaved) {
					node.ui[(page.isProtected() ? 'add' : 'remove') + 'Class']('sb-pages-tree-node-protected');
					Sb.editor.UiContext.registerChanges();
					if (credentials.login) {
						this.lastSavedCredentials = credentials;
					}
				}
			}.createDelegate(this);

		if (!field.getValue()) {
			saveHandler();
			return;
		}

		(new Sb.editor.PageProtectDialog({
			login			: data.login || this.lastSavedCredentials.login,
			password		: data.password || this.lastSavedCredentials.password,
			okHandler		: saveHandler,
			cancelHandler	: function () {
				if (field.getValue() !== page.isProtected()) {
					field.setValue(page.isProtected());
				}
			}
		})).show();
	},

	showDescriptionAndKeywordsWindow: function () {
		if (!this.descriptionAndKeywordsWin) {
			this.descriptionAndKeywordsWin = new Sb.Window({
				title: Sb.Locale.get('pageDescriptionAndKeywords'),
				width: 400,
				modal: true,
				resizable: false,
				closeAction: 'hide',
				layout: 'form',
				labelAlign: 'top',
				items: [{
					ref: 'descriptionField',
					xtype: 'textarea',
					fieldLabel: Sb.Locale.get('pageDescription'),
					name: 'description',
					msgTarget: 'under',
					maxLength: 255,
					anchor: '100%'
				}, {
					ref: 'keywordsField',
					xtype: 'textarea',
					fieldLabel: Sb.Locale.get('pageKeywords'),
					name: 'keywords',
					msgTarget: 'under',
					maxLength: 255,
					anchor: '100%'
				}],
				buttons: [{
					text: Sb.Locale.get('ok'),
					scope: this,
					handler: function () {
						var page = this.treeSelectionModel.getSelectedNode().attributes.model,
							descriptionField =  this.descriptionAndKeywordsWin.descriptionField,
							keywordsField = this.descriptionAndKeywordsWin.keywordsField,
							descriptionChanged, keywordsChanged;

						if (!descriptionField.isValid() || !keywordsField.isValid()) {
							return;
						}

						descriptionChanged = page.set('description', descriptionField.getValue());
						keywordsChanged = page.set('keywords', keywordsField.getValue());
						if (descriptionChanged || keywordsChanged) {
							Sb.editor.UiContext.registerChanges();
						}

						this.descriptionAndKeywordsWin.hide();
					}
				}, {
					text: Sb.Locale.get('cancelButtonTitle'),
					scope: this,
					handler: function () {
						this.descriptionAndKeywordsWin.hide();
					}
				}]
			});
		}

		var page = this.treeSelectionModel.getSelectedNode().attributes.model;
		this.descriptionAndKeywordsWin.descriptionField.emptyText = Sb.Editor.model.getTranslation(
			this.lngButtons.getValue(), 'description'
		);
		this.descriptionAndKeywordsWin.descriptionField.setValue(page.data.description);
		this.descriptionAndKeywordsWin.keywordsField.emptyText = Sb.Editor.model.getTranslation(
			this.lngButtons.getValue(), 'keywords'
		);
		this.descriptionAndKeywordsWin.keywordsField.setValue(page.data.keywords);

		this.descriptionAndKeywordsWin.show();
	},

	/**
	 * OK button handler.
	 *
	 * @param {Ext.Button} btn
	 */
	okHandler: function (btn) {
		this.suspendChanges = true;
		if (!this.form.isValid() || !this.areAllNodesValid()) {
			Sb.Editor.Ui.mainWindow.statusBar.setMessageError(
				Sb.Locale.get('fillRequiredFields')
			);
			this.suspendChanges = false;
			return;
		}
		this.suspendChanges = false;
		Sb.editor.PagesPanel.superclass.okHandler.call(this, btn);
	},

	/**
	 * Checks that at least 1 page is visible
	 */
	isAllPagesHidden: function () {
		var firstVisibleNode = this.pagesTree.root.findChildBy(
			function (child) {
				return !child.attributes.model.data.hidden;
			},
			this
		);
		return !firstVisibleNode;
	},

	/**
	 * Save handler.
	 * @param {Function} afterSaveCallbacks Handler, that will be run after save
	 */
	saveHandler: function (afterSaveCallbacks) {
		if (this.isAllPagesHidden()) {
			Sb.Editor.Ui.mainWindow.statusBar.setMessageError(
				Sb.Locale.get('allPagesHidden')
			);
			Ext.callback(afterSaveCallbacks.failure);
			return;
		}

		this.changeLocale(Sb.Editor.siteView.getCurrentSiteLocale());

		Sb.Editor.model.rootPage.save({
			success: function () {
				Ext.callback(afterSaveCallbacks.success);
				Sb.Editor.Ui.mainWindow.statusBar.setMessage(
					Sb.Locale.get('messageChangesSaved')
				);
			},
			failure: afterSaveCallbacks.failure
		});
	},

	/**
	 * Discard handler
	 */
	discardHandler: function () {
		Sb.Editor.Ui.mainWindow.statusBar.clear();
		this.changeLocale(Sb.Editor.siteView.getCurrentSiteLocale());
		Sb.Editor.model.rootPage.restoreFromMemento(this.memento);
	},

	/**
	 * Returns count pages
	 *
	 * @return {Number}
	 */
	getCountPages: function () {
		var countPages = -1;
		this.pagesTree.getRootNode().cascade(function () {
			countPages++;
		});
		return countPages;
	},

	updateAddPageButton: function () {
		var addPageButton = Ext.getCmp('addPageButton'),
			addPageButtonUpsellIcon = Ext.getCmp('addPageButtonUpsellIcon'),
			countPages = 0,
			maxPages = Sb.Editor.model.get('limits').max_pages_number,
			upsellClass = 'sb-add-page-upsell';

		Sb.Editor.model.rootPage.applyRecursive(function (page) {
			if (page.get('sitePageTypeId') == Sb.model.SitePage.LINK_TYPE) {
				return;
			}

			var masterWidget = page.getMasterWidget();
			if (!(masterWidget instanceof Sb.model.BlogPostWidget)) {
				countPages++;
			}
		});

		if (maxPages > countPages || maxPages == -1) {
			addPageButton.removeClass(upsellClass);
			addPageButtonUpsellIcon.hide();
		} else {
			addPageButton.addClass(upsellClass);
			addPageButtonUpsellIcon.show();
		}
	}
});