/**
 * @class Sb.editor.UiContext
 * <p>This singleton object, which represent user context.</p>
 * @singleton
 */
Ext.define('Sb.editor.UiContext', {
	singleton: true,

	/**
	 * Indicates that user data is changed
	 *
	 * @private
	 * @type boolean
	 * @property isUserDataChanged
	 */
	isUserDataChanged: false,

	/**
	 * Default options
	 *
	 * @private
	 * @type Object
	 * @property defaultOptions
	 */
	defaultOptions: {
		yesCallback: Ext.emptyFn,
		isYesDeferred: false,
		noCallback: Ext.emptyFn,
		isNoDeferred: false
	},

	/**
	 * Current context options
	 *
	 * @private
	 * @type Object
	 * @property options
	 */
	options: {},

	/**
	 * "Apply changes" dialog buttons
	 *
	 * @private
	 * @type Object
	 * @property buttons
	 */
	buttons: {},

	/**
	 * Icon class for yes button.
	 *
	 * @private
	 * @type String
	 * @property yesIconCls
	 */
	yesIconCls: 'sb-yes-icon',

	/**
	 * Icon class for no button.
	 *
	 * @private
	 * @type String
	 * @property noIconCls
	 */
	noIconCls: 'sb-no-icon',

	/**
	 * Icon class for cancel button.
	 *
	 * @private
	 * @type String
	 * @property cancelIconCls
	 */
	cancelIconCls: 'sb-cancel-icon',

	/**
	 * Icon class for loading button.
	 *
	 * @private
	 * @type String
	 * @property loadingIconCls
	 */
	loadingIconCls: 'sb-loading-icon',

	/**
	 * "Apply changes" dialog instance
	 *
	 * @private
	 * @type Ext.Window
	 * @property dialog
	 */
	dialog: null,

	/**
	 * Context switcher callback
	 *
	 * @private
	 * @type Function
	 * @property contextSwitcher
	 */
	contextSwitcher: Ext.emptyFn,

	/**
	 * Registers changes in user data
	 */
	registerChanges: function () {
		this.isUserDataChanged = true;
	},

	/**
	 * Sets new context
	 *
	 * @param {Object} [options]
	 */
	setNewContext: function (options) {
		this.isUserDataChanged = false;
		Ext.apply(this.options, options || {}, this.defaultOptions);
	},

	/**
	 * Returns TRUE if context may be switched immediately or FALSE otherwise.
	 *
	 * @return bool
	 */
	isMayBeSwitchedImmediately: function () {
		return !this.isUserDataChanged;
	},

	/**
	 * Tries to switch UI context.
	 * Returns TRUE if context is switched immediately or FALSE otherwise.
	 *
	 * @param {Function} contextSwitcher
	 * @return bool
	 */
	tryToSwitch: function (contextSwitcher) {
		if (this.isMayBeSwitchedImmediately()) {
			contextSwitcher.call();
			return true;
		}

		this.contextSwitcher = contextSwitcher;
		this.showDialog();
		return false;
	},

	/**
	 * Finishes deferred action
	 */
	finishDeferredAction: function () {
		this.dialog.hide();
		Ext.iterate(this.buttons, function (name, button) {
			button.enable();
			button.setIconClass(this[name + 'IconCls']);
		}, this);

		this.isUserDataChanged = false;
		this.contextSwitcher.call();
		this.contextSwitcher = Ext.emptyFn;
	},

	/**
	 * Cancels deferred action
	 */
	cancelDeferredAction: function () {
		this.dialog.hide();
		Ext.iterate(this.buttons, function (name, button) {
			button.enable();
			button.setIconClass(this[name + 'IconCls']);
		}, this);
	},

	/**
	 * Executes when user select saving changes
	 *
	 * @private
	 */
	yesButtonClicked: function () {
		if (this.options.isYesDeferred) {
			Ext.iterate(this.buttons, function (name, button) {
				button.disable();
			}, this);
			this.buttons.yes.setIconClass(this.loadingIconCls);
		}
		Ext.callback(this.options.yesCallback);
		if (!this.options.isYesDeferred) {
			this.finishDeferredAction();
		}
	},

	/**
	 * Executes when user select do not save changes
	 *
	 * @private
	 */
	noButtonClicked: function () {
		Ext.callback(this.options.noCallback);
		if (this.options.isNoDeferred) {
			Ext.iterate(this.buttons, function (name, button) {
				button.disable();
			});
			this.buttons.no.setIconClass(this.loadingIconCls);
			return;
		}
		this.finishDeferredAction();
	},

	/**
	 * Executes when user cancel operation
	 *
	 * @private
	 */
	cancelButtonClicked: function () {
		this.dialog.hide();
	},

	/**
	 * Shows "Apply changes" dialog
	 *
	 * @private
	 */
	showDialog: function () {
		if (this.dialog) {
			this.dialog.show();
			return;
		}

		var buttonNames = ['yes', 'no', 'cancel'];
		var buttons = [];
		var handleButton = function (name) {
			this.buttons[name].blur();
			this[name + 'ButtonClicked'].call(this);
		};
		Ext.each(buttonNames, function (name) {
			buttons.push(this.buttons[name] = new Ext.Button({
				iconCls: this[name + 'IconCls'],
				text: Ext.util.Format.htmlEncode(Sb.Locale.get(name + 'ButtonTitle')),
				handler: handleButton.createDelegate(this, [name]),
				hideMode: 'offsets'
			}));
		}, this);

		this.dialog = new Ext.Window({
			manager:	Sb.Editor.windowManager.getManager(9),
			renderTo:	Ext.getBody(),
			modal:		true,
			constrain:	true,
			draggable:	false,
			resizable:	false,
			closable:	false,
			shadow:		false,
			title:		Ext.util.Format.htmlEncode(Sb.Locale.get('applyChangesDialogTitle')),
			stateful:	false,
			shim:		true,
			buttonAlign:'center',
			width:		400,
			cls:		'x-window-dlg',
			plain:		true,
			html:		Ext.util.Format.htmlEncode(Sb.Locale.get('applyChangesDialogMessage')),
			footer:		true,
			onEsc:		this.buttons.cancel.handler,
			buttons:	buttons
		});
		this.dialog.focusEl = this.buttons.yes;

		var keyMap = this.dialog.getKeyMap();
		keyMap.on(27, handleButton.createDelegate(this, ['cancel']), this);
		keyMap.enable();

		this.dialog.on('show', function () {
			this.dialog.focusEl = this.buttons.yes;
		}, this);

		this.dialog.show();
	}
});
