/**
 * @class Sb.editor.WidgetsPanel
 * @extends Ext.Panel
 * Widgets panel
 * @constructor
 * @param {Object} config The config object
 * @xtype widgetspanel
 */
Ext.define('Sb.editor.WidgetsPanel', {
	extend: 'Ext.Panel',
	xtype: 'widgetspanel',

	initComponent: function () {
		var widgets = this.items || [];
		Ext.apply(this, {
			title: Sb.Locale.get('widgetsTitle'),
			helperMessage: Sb.Locale.get('widgetsDescription'),
			tabTip: Sb.Locale.get('widgetsDescriptionOnHover'),
			cls: 'sb-editor-widgets-panel',
			autoHeight: true,
			items: {
				layout: 'carousel',
				layoutConfig: {
					scrollIncrement: Ext.isIE ? 30 : 10
				},
				border: false,
				itemId: 'panel',
				items: widgets,
				lookupComponent: function (config) {
					if ('-' === config) {
						return new Sb.editor.WidgetsPanel.Separator();
					}
					config = (Ext.isString(config)) ? {type: config} : config;
					return new Sb.editor.WidgetsPanel.Button(config);
				},
				listeners: {
					afterrender: function (panel) {
						panel.el.on('mousewheel', function (e) {
							for (var i = 0; i < Math.abs(e.getWheelDelta()) * 4; i++) {
								this.layout['onScroll' + (e.getWheelDelta() > 0 ? 'Left' : 'Right') + 'Click'].call(this.layout);
							}
						}, panel);
					}
				}
			}
		});
		Sb.editor.WidgetsPanel.superclass.initComponent.apply(this, arguments);
	}
});

/**
 * @class Sb.editor.WidgetsPanel.Button
 * @extends Ext.BoxComponent
 */
Ext.define('Sb.editor.WidgetsPanel.Button', {
	extend: 'Ext.BoxComponent',

	overCls: 'sb-widgets-item-over',
	disabledClass: 'sb-widgets-item-disabled',
	disableReasonClass: '',

	constructor: function (config) {
		config.itemId = config.type;

		Sb.editor.WidgetsPanel.Button.superclass.constructor.call(this, config);

		/**
		 * Button`s widget type
		 * @type {Sb.widgetType.Base}
		 * @property widgetType
		 */
		this.type = Sb.WidgetTypeManager.get(config.type);
		this.type.on('isupsellchange', function (isUpsell) {
			this.setUpsell(isUpsell);
		}, this);
		Sb.WidgetTypeManager.registerComponent(this, this.type);
	},

	onRender: function (ct, position) {
		this.autoEl = Sb.Utils.getWidgetButtonCfg(this.type.name, this.getWidgetTitle());

		Sb.editor.WidgetsPanel.Button.superclass.onRender.call(this, ct, position);

		this.type.toggleButton();
		this.setUpsell(this.type.isUpsell);
	},

	getWidgetTitle: function() {
		var str = this.type.getTranslation('title'),
			pos = str.indexOf(' '),
			prev = 0;

		if (pos === -1) {
			return str;
		}

		while (pos != -1) {
			if (2 * pos > str.length) {
				if (pos + prev < str.length) {
					prev = pos;
				}
				break;
			}
			prev = pos;
			pos = str.indexOf(' ', prev + 1);
		}

		return str.slice(0, prev) + '<br>' + str.slice(prev + 1);
	},

	/**
	 * Sets the tooltip for this Button.
	 * @param {String} tooltip
	 */
	setTooltip: function (tooltip) {
		Ext.QuickTips.registerSelfAndChildren({
			target: this,
			text: tooltip
		});
	},

	onEnable: function () {
		this.createDD();
		this.dd.unlock();
		this.removeClass(this.disableReasonClass);
		this.setTooltip(this.type.getTranslation('description'));

		this.setUpsell(this.type.isUpsell);
		Sb.editor.WidgetsPanel.Button.superclass.onEnable.call(this);
	},

	onDisable: function () {
		if (this.dd) {
			this.dd.lock();
		}
		Sb.editor.WidgetsPanel.Button.superclass.onDisable.call(this);

		this.setUpsell(false);
		this.removeClass(this.disableReasonClass);

		if (this.type.disableReason) {
			if (this.type.disableReason == 'installed') {
				this.disableReasonClass = 'sb-widgets-item-installed';
				this.removeClass(this.disabledClass);
			} else {
				this.disableReasonClass = 'sb-widgets-item-' + this.type.name + '-disabled-' + this.type.disableReason;
			}
			this.addClass(this.disableReasonClass);
		}

		this.setTooltip(this.type.getTranslation('descriptionDisabled' + this.type.disableReason.toUpperCaseFirst()));
	},

	createDD: function () {
		if (!this.dd) {
			this.dd = new Ext.dd.DragSource(this.el, {
				buttonEl	: this.el,
				ddGroup		: 'widgetButtons',
				dragData	: {
					widgetType: this.type,
					widgetButton: this
				},
				scroll		: false,

				startDrag	: function () {
					Sb.Editor.siteView.onWidgetStartDrag();
					if (Sb.Editor.Ui.editComponent) {
						Sb.Editor.Ui.editComponent.completeEdit();
					}
					this.dragging = true;
					this.proxy.ghost.addClass(this.buttonEl.dom.className);
					this.proxy.update(this.buttonEl.dom.innerHTML);
					Ext.getBody().setStyle('cursor', 'default');
					this.dragData.widgetButton.removeClass(this.dragData.widgetButton.overCls);
				},

				endDrag		: function () {
					Ext.getBody().setStyle('cursor', 'auto');
					Sb.Editor.siteView.onWidgetEndDrag();
				},

				beforeDragDrop: function (target, e, id) {
					if (target.id != id) {
						return false;
					}

					var dragData = Ext.dd.DragDropMgr.dragCurrent.dragData;
					if (!dragData.container) { // drop to usual (not column) container?
						return true;
					}
					return (dragData.container.el.id == id);
				}
			});
			this.el.on('click', this.dd.handleMouseDown, this.dd);
		}
	},

	afterRender: function () {
		if (!this.disabled) {
			this.createDD();
		}
	},

	/**
	 * Update button view if widget is upsell
	 * @param {Boolean} isUpsell
	 */
	setUpsell: function (isUpsell) {
		if (!this.el) {
			return;
		}

		isUpsell = (isUpsell === true) && this.type.isAvailable();
		var upsellClass = 'sb-widgets-item-upsell',
			upsellEl;

		upsellEl = this.el.down('.' + upsellClass);
		if (isUpsell && !upsellEl) {
			this.el.createChild({
				cls: upsellClass
			});
		}
		if (!isUpsell && upsellEl) {
			upsellEl.remove();
		}
	}
});

/**
 * @class Sb.editor.WidgetsPanel.Separator
 * @extends Ext.BoxComponent
 */
Ext.define('Sb.editor.WidgetsPanel.Separator', {
	extend: 'Ext.BoxComponent',

	onRender: function (ct, position) {
		this.autoEl = {
			cls: 'sb-widgets-separator',
			cn: {
				tag: 'span'
			}
		};
		Sb.editor.WidgetsPanel.Separator.superclass.onRender.call(this, ct, position);
	}
});