/**
 * @class Sb.editor.component.Base
 * @extends Ext.Component
 * Base class for editor component - hovering, drag and drop, clicking
 * @constructor
 * @param {Object} config The config object
 */
Ext.define('Sb.editor.component.Base', {
	extend: 'Ext.Component',

	/**
	 * @cfg {String} baseCls
	 * The base CSS class to apply to this component's element (defaults to <code>'sb-component'</code>).
	 */
	baseCls: 'sb-widget',

	/**
	 * @cfg {Boolean} hasControls
	 * <p>True to display controls buttons, false to them (defaults to <code>true</code>).</p>
	 */
	hasControls: true,

	/**
	 * @cfg {Boolean} hasEditButton
	 * Is needed to create Edit button or not (defaults to <code>true</code>).
	 */
	hasEditButton: true,

	/**
	 * @cfg {Array} menu
	 * An array of items to be added to edit button's menu (defaults to undefined).
	 */

	/**
	 * @cfg {Boolean} hasEditPanel
	 * Is needed to create edit panel (defaults to <code>true</code>).
	 */
	hasEditPanel: true,

	/**
	 * @cfg {Boolean} hasStylePanel
	 * Is needed to create style panel (defaults to <code>false</code>).
	 */
	hasStylePanel: false,

	/**
	 * @cfg {Boolean} movable
	 * <p>True to allow to move the component, false to not (defaults to <code>true</code>).</p>
	 */
	movable: true,

	/**
	 * @cfg {Boolean} removable
	 * <p>True to display the 'Remove' button and allow to remove the component, false to hide
	 * the button and disallow removing the component (defaults to <code>true</code>).</p>
	 */
	removable: true,

	/**
	 * @cfg {Boolean} clickable
	 * <p>True to allow to click the component content, false to not (defaults to <code>true</code>).</p>
	 */
	clickable: true,

	/**
	 * @cfg {Boolean} overable
	 * <p>True to show frame and controls when the component is hovered, false to not (defaults to <code>true</code>).</p>
	 */
	overable: true,

	/**
	 * @cfg {Boolean} translatable
	 * <p>True if component is translatable (defaults to <code>false</code>).</p>
	 */
	translatable: false,

	/**
	 * @cfg {Boolean} removeConfirmation
	 * True to remove the component with confirmation
	 */
	removeConfirmation: true,

	isShowEditPanel: true,

	initComponent: function () {
		if (this.menu && Ext.isArray(this.menu)) {
			this.menu = new Ext.menu.Menu({
				cls: 'sb-menu-widget-actions',
				shadow: false,
				items: this.menu,
				listeners: {
					scope: this,
					mouseover: function (menu, e, menuItem) {
						if (menuItem.tooltip) {
							Sb.Editor.Ui.mainWindow.setHelperMessage(menuItem.tooltip);
						}
					},
					mouseout: function (menu, e, menuItem) {
						if (menuItem.tooltip) {
							Sb.Editor.Ui.mainWindow.restoreHelperMessage();
						}
					},
					show: function (menu) {
						this.editEl.arrow.ignoreNextClick = 0;
						this.editEl.arrow.ignoreNextHide = 0;
						Sb.Editor.Ui.isHoverDisabled = true;
					},
					hide: function (menu) {
						Sb.Editor.Ui.isHoverDisabled = false;
						this.editEl.arrow.ignoreNextClick = this.restoreActionsClick.defer(250, this);
						if (!this.editEl.arrow.ignoreNextHide) {
							this.removeOver.defer(250, this);
						}
					}
				}
			});
		}

		Sb.editor.component.Base.superclass.initComponent.call(this);
		this.removeTitle = this.removeTitle || Sb.Locale.get('removeWidgetTitle');
		this.removalConfirmationMessage	= Sb.Locale.get('confirmRemovalComponent');
		this.addEvents(
			'startover',		'removeover',
			'beforestartedit',	'startedit',	'completeedit',	'canceledit',	'hideedit',
			'remove'
		);
	},

	onRender: function () {
		this.autoEl = {
			cls: this.baseCls,
			cn: {
				cls: this.baseCls + '-inner',
				cn: [{
					cls: this.baseCls + '-controls',
					cn: {
						cls: this.baseCls + '-controls-wrap',
						cn: {
							cls: this.baseCls + '-controls-inner'
						}
					}
				}, {
					cls: this.baseCls + '-content'
				}]
			}
		};

		Sb.editor.component.Base.superclass.onRender.apply(this, arguments);

		this.innerEl = this.el.first();
		this.controlsEl = this.innerEl.first();
		this.controlsEl.setVisibilityMode(Ext.Element.DISPLAY);
		if (!this.hasControls) {
			this.controlsEl.addClass('sb-widget-controls-hidden');
		}
		this.contentElement = this.controlsEl.next();
		this.controlsTargetEl = this.controlsEl.first().first();
		this.controlsEl.hide();

		this.ddEl = this.controlsTargetEl.createChild({
			tag: 'span',
			cls: this.baseCls + '-control-item ' + this.baseCls + '-control-move',
			cn: {
				tag: 'span'
			}
		});

		var editElCls = this.baseCls + '-control-item ' + this.baseCls + '-control-edit';
		if (!this.menu) {
			editElCls += ' sb-widget-control-edit-no-menu';
		}
		this.editEl = this.controlsTargetEl.createChild({
			tag: 'span',
			cls: editElCls
		}).setVisibilityMode(Ext.Element.DISPLAY);
		this.editEl.button = this.editEl.createChild({
			tag: 'span',
			html: Sb.Locale.get('editWidgetTitle')
		});
		this.editEl.arrow = this.editEl.createChild({
			tag: 'i',
			cls: this.baseCls + '-control-dd-arrow'
		});
		if (!this.hasEditButton) {
			this.editEl.hide();
		}

		if (this.hasStylePanel) {
			this.stylePropertiesEl = this.controlsTargetEl.createChild({
				tag: 'span',
				cls: this.baseCls + '-control-item ' + this.baseCls + '-control-style-properties ' + this.baseCls + '-control-item-disabled',
				cn: {
					tag: 'span',
					html: Sb.Locale.get('propertiesStyleWidgetHide')
				}
			}).setVisibilityMode(Ext.Element.DISPLAY).hide();
		}
		// :TODO: 20131209: need to remove this initialization for such widget as columns
		this.propertiesEl = this.controlsTargetEl.createChild({
			tag: 'span',
			cls: this.baseCls + '-control-item ' + this.baseCls + '-control-properties ' + this.baseCls + '-control-item-disabled',
			cn: {
				tag: 'span',
				html: Sb.Locale.get('propertiesWidgetHide')
			}
		}).setVisibilityMode(Ext.Element.DISPLAY).hide();

		if (!this.movable) {
			this.ddEl.setVisibilityMode(Ext.Element.DISPLAY).hide();
			this.editEl.setStyle('background-image', 'none');
		}

		if (this.removable) {
			this.removeEl = this.controlsTargetEl.createChild({
				tag: 'span',
				cls: this.baseCls + '-control-item ' + this.baseCls + '-control-remove',
				cn: {
					tag: 'span',
					html: this.removeTitle
				}
			});
			if (!this.movable && !this.hasEditButton) {
				this.removeEl.setStyle('background-image', 'none');
			}
		}

		this.lngButtons = new Sb.form.LanguageButtons({
			renderTo: this.controlsTargetEl,
			listeners: {
				scope: this,
				selectionchange: function (cmp) {
					this.onCurrentLocaleChange(cmp.getValue(), this.currentLocale);
				}
			}
		});

		this.applyEl = this.controlsEl.createChild({
			tag: 'span',
			cls: this.baseCls + '-control-btn ' + this.baseCls + '-control-btn-apply',
			style: 'display: none',
			cn: {
				tag: 'span',
				cls: this.baseCls + '-control-btn-inner',
				cn: {
					tag: 'span',
					html: Sb.Locale.get('applyChangesTitle')
				}
			}
		});
		this.applyEl.setVisibilityMode(Ext.Element.DISPLAY).hide();

		this.reverseEl = this.controlsEl.createChild({
			tag: 'span',
			cls: this.baseCls + '-control-btn ' + this.baseCls + '-control-btn-reverse',
			style: 'display: none',
			cn: {
				tag: 'span',
				cls: this.baseCls + '-control-btn-inner',
				cn: {
					tag: 'span',
					html: Sb.Locale.get('reverseChangesTitle')
				}
			}
		});
		this.reverseEl.setVisibilityMode(Ext.Element.DISPLAY).hide();
	},

	/**
	 * @property {String} newLocale
	 * @property {String} oldLocale
	 */
	onCurrentLocaleChange: function (newLocale, oldLocale) {
		this.currentLocale = newLocale;
	},

	restoreActionsClick: function () {
		this.editEl.arrow.ignoreNextClick = 0;
	},

	restoreActionsMouseDown: function () {
		this.editEl.arrow.ignoreNextHide = 0;
	},

	getContentTarget: function () {
		return this.contentElement;
	},

	afterRender: function () {
		Sb.editor.component.Base.superclass.afterRender.call(this);
		this.initEvents();
	},

	initEvents: function () {
		Sb.Editor.windowManager.register(this, 1);

		if (this.clickable) {
			this.contentElement.on('click', this.onContentElementClick, this);
		}

		if (this.overable) {
			this.getEl().on('mouseenter', this.onMouseEnter, this);
			this.getEl().on('mouseleave', function () {
				this.completeOver(true);
			}, this);
		}

		this.controlsEl.on('click', Ext.emptyFn, this, {stopEvent: true});

		this.editEl.arrow.on('mousedown', function () {
			this.editEl.arrow.ignoreNextHide = this.restoreActionsMouseDown.defer(250, this);
		}, this);
		this.editEl.arrow.on('click', function () {
			if (!this.editEl.arrow.ignoreNextClick && Sb.Editor.Ui.editComponent != this && this.rendered && this.menu) {
				this.menu.show(this.editEl);
			}
		}, this);
		this.editEl.button.on('click', this.startEdit, this);

		this.propertiesEl.on('click', this.propertiesClickHandler, this);
		if (this.hasStylePanel) {
			this.stylePropertiesEl.on('click', this.stylePropertiesClickHandler, this);
		}

		if (this.removable) {
			this.removeEl.on('click', this.removeComponent, this);
		}
		this.applyEl.on('click', function () {
			this.completeEdit(true);
		}, this);
		this.reverseEl.on('click', this.cancelEdit, this);

		if (this.movable) {
			this.initDD();
		}
	},

	onMouseEnter: function (e) {
		e.stopPropagation();
		if (Sb.Editor.Ui.isDragActive()	|| Sb.Editor.Ui.isHoverDisabled) {
			return;
		}
		if (!this.showOverTask) {
			this.showOverTask = new Ext.util.DelayedTask(this.startOver, this);
		}
		this.showOverTask.delay(200);
	},

	initDD: function () {
		this.dd = new Sb.editor.component.Base.DD(this);
	},

	propertiesClickHandler: function () {
		var editPanel = this.getEditPanel();
		if (this.getShowEditPanelState()) {
			this.propertiesEl.removeClass(this.baseCls + '-control-item-disabled');
			this.propertiesEl.child('span').update(Sb.Locale.get('propertiesWidgetShow'));
			this.toggleEditPanelState(false);
			editPanel.hide();
			editPanel.offsets = null;
		} else {
			this.propertiesEl.addClass(this.baseCls + '-control-item-disabled');
			this.propertiesEl.child('span').update(Sb.Locale.get('propertiesWidgetHide'));
			this.toggleEditPanelState(true);
			editPanel.show(this);
		}
	},

	stylePropertiesClickHandler: function () {
		var stylePanel = this.getStylePanel();
		if (stylePanel.isVisible()) {
			this.stylePropertiesEl.removeClass(this.baseCls + '-control-item-disabled');
			this.stylePropertiesEl.child('span').update(Sb.Locale.get('propertiesStyleWidgetShow'));
			Sb.Editor.Ui.isShowStylePanel = false;
			stylePanel.hide();
			stylePanel.offsets = null;
		} else {
			this.stylePropertiesEl.addClass(this.baseCls + '-control-item-disabled');
			this.stylePropertiesEl.child('span').update(Sb.Locale.get('propertiesStyleWidgetHide'));
			Sb.Editor.Ui.isShowStylePanel = true;
			stylePanel.show(this);
		}
	},

	removeComponent: function () {
		if (!this.removeConfirmation) {
			this.removeComponentHandler();
			return;
		}

		Ext.Msg.show({
			title    : Sb.Locale.get('confirmRemoval'),
			msg      : this.removalConfirmationMessage,
			closable : false,
			buttons  : Ext.Msg.YESNO,
			scope    : this,
			fn       : function (btn) {
				if (btn == 'yes') {
					this.removeComponentHandler();
				}
			}
		});
	},

	removeComponentHandler: function () {
		if (Sb.Editor.Ui.editComponent == this) {
			this.hideEdit();
		}
		this.fireEvent('remove', this);
	},

	showControls: function () {
		this.controlsEl.show();
		this.anchorControls();
	},

	anchorControls: function () {
		var topMessageBoxHeight = Sb.TopMessageBox.isVisible() ? Sb.TopMessageBox.el.getHeight() : 0,
			top = this.el.getTop() - Sb.Editor.siteView.container.getTop() - topMessageBoxHeight,
			height = this.controlsEl.getHeight();

		if (top > height) {
			this.controlsPosition = 'bl-tl';
			this.controlsEl.setStyle({
				'padding-bottom': '1px',
				'padding-top': '0'
			});
		} else {
			this.controlsPosition = 'tl-bl';
			this.controlsEl.setStyle({
				'padding-bottom': '0',
				'padding-top': '1px'
			});
		}

		this.controlsEl.anchorTo(this.el, this.controlsPosition);
	},

	onContentElementClick: function (event) {
		event.stopEvent();
		if (this.afterDD) {
			this.afterDD = false;
			return;
		}
		this.startEdit();
	},

	/**
	 * Ends the overing process and hides the frame.
	 * If completion is explicit and in history of overing components there is an component other than itself, then start overing it.
	 * @param {Boolean} [explicitCompletion] true if overing completion is explicit by mouseleave (defaults to false).
	 */
	completeOver: function (explicitCompletion) {

		if (Sb.Editor.Ui.isDragActive()
			// don't complete over components outside editing component
			|| (Sb.Editor.Ui.editComponent && !Sb.Editor.Ui.editComponent.el.contains(this.el))
			// don't complete over components that is already focused
			|| this.el.hasClass(this.getStateCls('focus'))) {
			return;
		}

		if (this.showOverTask) {
			this.showOverTask.cancel();
		}

		if (Sb.Editor.Ui.overComponent) {
			if (Sb.Editor.Ui.overComponent.showOverTask) {
				Sb.Editor.Ui.overComponent.showOverTask.cancel();
			}
			Sb.Editor.Ui.overComponent.removeOver();
		}

		if (explicitCompletion && Sb.Editor.Ui.overComponentHistory[0]) {
			var comp = Sb.Editor.Ui.overComponentHistory.shift();
			if (!comp.showOverTask) {
				comp.showOverTask = new Ext.util.DelayedTask(comp.startOver, comp);
			}
			comp.showOverTask.delay(200);
			Sb.Editor.Ui.overComponent = comp;
		}
	},

	startOver: function () {
		var el, comp;
		if (Sb.Editor.Ui.isDragActive()
			// don't start over components outside editing component
			|| (Sb.Editor.Ui.editComponent && !Sb.Editor.Ui.editComponent.el.contains(this.el))
			// don't start over components that is already focused
			|| this.el.hasClass(this.getStateCls('focus'))) {
			return;
		}

		if (Sb.Editor.Ui.overComponent) {
			// component allready overed
			if (Sb.Editor.Ui.overComponent == this) {
				return;
			}
			// complete over other overed component
			Sb.Editor.Ui.overComponent.completeOver();
		}
		if (!this.el.dom) {
			return;
		}

		Sb.Editor.Ui.overComponentHistory = [];
		this.el.addClass(this.getStateCls('over'));
		for (el = this.el.up('.sb-widget'); el; el = el.up('.sb-widget')) {
			comp = Ext.getCmp(el.id);
			if (comp.overable) {
				el.addClass(comp.getStateCls('over'));
				Sb.Editor.Ui.overComponentHistory.push(comp);
			}
		}

		Sb.Editor.Ui.overComponent = this;
		this.addClass(this.getStateCls('over'));

		if (!Sb.Editor.Ui.editComponent) {
			this.showControls();
		}

		this.toFront();
		this.fireEvent('startover', this);
	},

	/**
	 * Starts the editing process and shows the editor.
	 *
	 * @param {String} [localeCode]
	 */
	startEdit: function (localeCode) {
		var el, cmp;
		localeCode = localeCode || Sb.Editor.siteView.getCurrentSiteLocale();
		if (this.fireEvent('beforestartedit', this) === false) {
			return;
		}

		// don't start edit self again
		if (Sb.Editor.Ui.editComponent == this) {
			return;
		}

		// complete current editing
		if (Sb.Editor.Ui.editComponent) {
			var isChildComponent = false;
			// if current editing component contains this component then remember editing component in editing history
			if (Sb.Editor.Ui.editComponent.el.contains(this.el)) {
				Sb.Editor.Ui.editComponentHistory.unshift(Sb.Editor.Ui.editComponent);
				isChildComponent = true;
			} else {
				while (Sb.Editor.Ui.editComponentHistory[0] && !Sb.Editor.Ui.editComponentHistory[0].el.contains(this.el)) {
					Sb.Editor.Ui.editComponentHistory.shift();
				}
				if (Sb.Editor.Ui.editComponentHistory[0]) {
					Sb.Editor.Ui.editComponentHistory.unshift(Sb.Editor.Ui.editComponent);
				}
			}
			Sb.Editor.Ui.editComponent.completeEdit();
			if (!isChildComponent) {
				Sb.Editor.Ui.editComponentHistory.shift();
			}
		}

		this.startOver();
		if (!this.controlsEl.isVisible()) {
			this.showControls();
		}
		Sb.Editor.Ui.editComponent = this;

		this.currentLocale = localeCode;
		if (this.translatable) {
			this.lngButtons.setData(Sb.Editor.model.get('locales'));
			this.lngButtons.setValue(this.currentLocale);
			this.lngButtons.setVisible(
				Sb.Editor.model.get('locales').length > 1
			);
		}

		this.applyEl.show();
		this.reverseEl.show();
		this.editEl.hide();
		if (this.hasStylePanel) {
			this.stylePropertiesEl.show();
		}

		for (el = this.el; el; el = el.up('.sb-widget')) {
			cmp = Ext.getCmp(el.id);
			if (cmp.overable) {
				cmp.el.addClass(cmp.getStateCls((el === this.el) ? 'focus' : 'over'));
			}
		}
		this.onStartEdit();
		if (this.xtype != 'navigation_widget') {
			Ext.getBody().addClass('sb-editing');
		}
		this.el.scrollIntoView(Sb.Editor.siteView.el);

		Ext.getDoc().on('mousedown', this.onDocumentClick, this);
		this.fireEvent('startedit', this);
	},

	onStartEdit: function () {
		var editPanel = this.getEditPanel();
		if (editPanel) {
			this.propertiesEl.show();
			var isShowEditPanel = this.getShowEditPanelState();
			this.propertiesEl[isShowEditPanel ? 'addClass' : 'removeClass'](this.baseCls + '-control-item-disabled');
			if (isShowEditPanel) {
				editPanel.show(this);
			} else {
				this.propertiesEl.child('span').update(Sb.Locale.get('propertiesWidgetShow'));
			}
		}
		if (this.hasStylePanel) {
			this.refreshStylePanel();
		}
	},

	/**
	 * Ends the editing process, persists the changed value to the underlying field, and hides the editor.
	 * If completion is explicit and in history of editing components there is an component other than itself, then start editing it.
	 * @param {Boolean} explicitCompletion true if editing completion is explicit by clicking 'Ok' or other cases (defaults to false).
	 */
	completeEdit: function (explicitCompletion) {
		if (Sb.Editor.Ui.editComponent != this) {
			return;
		}
		if (Sb.Editor.siteView.getCurrentSiteLocale() !== this.currentLocale) {
			this.onCurrentLocaleChange(Sb.Editor.siteView.getCurrentSiteLocale(), this.currentLocale);
		}
		this.onCompleteEdit();
		this.hideEdit();
		Ext.getBody().removeClass('sb-editing');
		this.fireEvent('completeedit', this);

		if (Sb.Editor.Ui.editComponentHistory[0] != this) {
			if (explicitCompletion) {
				var comp = Sb.Editor.Ui.editComponentHistory.shift();
				if (comp) {
					comp.startEdit();
				}
			} else {
				Sb.Editor.Ui.editComponentHistory = [];
			}
		}
	},

	onCompleteEdit: Ext.emptyFn,

	/**
	 * Cancels the editing process and hides the editor without persisting any changes.
	 */
	cancelEdit: function () {
		if (Sb.Editor.Ui.editComponent != this) {
			return;
		}
		if (Sb.Editor.siteView.getCurrentSiteLocale() !== this.currentLocale) {
			this.onCurrentLocaleChange(Sb.Editor.siteView.getCurrentSiteLocale(), this.currentLocale);
		}
		this.onCancelEdit();
		this.hideEdit();
		this.fireEvent('canceledit', this);
	},

	onCancelEdit: Ext.emptyFn,

	/**
	 * Hides the editor.
	 */
	hideEdit: function () {
		var el = this.el,
			comp;
		Ext.getDoc().un('mousedown', this.onDocumentClick, this);
		this.onHideEdit();
		this.lngButtons.hide();
		this.applyEl.hide();
		this.reverseEl.hide();
		this.controlsEl.hide();
		this.propertiesEl.hide();
		if (this.hasStylePanel) {
			this.stylePropertiesEl.hide();
		}
		if (this.hasEditButton) {
			this.editEl.show();
		}
		this.el.removeClass(this.getStateCls('focus'));
		this.el.removeClass(this.getStateCls('over'));

		for (el = this.el; el; el = el.up('.sb-widget')) {
			comp = Ext.getCmp(el.id);
			if (comp.overable) {
				el.removeClass(comp.getStateCls('over'));
				el.removeClass(comp.getStateCls('focus'));
			}
		}
		Sb.Editor.Ui.editComponent = null;
		if (Sb.Editor.Ui.overComponent == this) {
			Sb.Editor.Ui.overComponent = null;
		}
		this.fireEvent('hideedit', this);
	},

	onHideEdit: function () {
		var editPanel = this.getEditPanel();
		if (editPanel) {
			editPanel.hide();
		}
		if (this.hasStylePanel) {
			this.getStylePanel().hide();
		}
	},

	/**
	 * Remove "over" state
	 */
	removeOver: function () {
		if ((Ext.dd.DragDropMgr.dragCurrent && Ext.dd.DragDropMgr.dragCurrent.component == this)
			|| Sb.Editor.Ui.editComponent == this
			|| (this.menu && !this.menu.hidden)
			|| (this.resizer && this.resizer.resizing)
		) {
			return;
		}

		Sb.Editor.Ui.overComponent = null;
		this.controlsEl.hide();
		this.removeClass(this.getStateCls('over'));
		this.fireEvent('removeover', this);
	},

	/**
	 * Return component edit panel.
	 *
	 * @return {Sb.editor.component.BaseEditPanel} editPanel
	 */
	getEditPanel: function () {
		if (!Sb.editor.component.Base.editPanel) {
			Sb.editor.component.Base.editPanel = new Sb.editor.component.BaseEditPanel();
		}
		return Sb.editor.component.Base.editPanel;
	},

	/**
	 * Is show panel on edit?
	 * @return {Boolean} default is true
	 */
	getShowEditPanelState: function () {
		return !!this.isShowEditPanel;
	},

	/**
	 * Change edit panel state
	 * @param {Boolean} isShowEditPanel
	 */
	toggleEditPanelState: function (isShowEditPanel) {
		this.isShowEditPanel = isShowEditPanel;
	},

	/**
	 * Return component style panel.
	 *
	 * @return {Sb.editor.component.BaseStylePanel} stylePanel
	 */
	getStylePanel: function () {
		if (!Sb.Editor.Ui.stylePanel) {
			Sb.Editor.Ui.stylePanel = new Sb.editor.component.BaseStylePanel();
		}
		return Sb.Editor.Ui.stylePanel;
	},

	onDocumentClick: function (event) {
		var editPanel		= this.getEditPanel(),
			stylePanel		= this.getStylePanel(),
			targetHasMask	= Ext.fly(event.target).hasClass('ext-el-mask'),
			clickedInsideEditableComponent = event.within(this.el);

		Ext.each(Sb.Editor.Ui.editComponentHistory, function (comp) {
			clickedInsideEditableComponent = clickedInsideEditableComponent || event.within(comp.el);
		});

		if ((editPanel && event.within(editPanel.el))
			|| (stylePanel && event.within(stylePanel.el))
			|| clickedInsideEditableComponent
			|| (this.menu && event.within(this.menu.el))
			|| (this.hideSettingsMenu && this.hideSettingsMenu.rendered && event.within(this.hideSettingsMenu.el))
			|| event.target == Sb.Editor.Ui.mainWindow.header.dom
			|| event.getTarget('#sb-editor-minimize-btn')
			|| event.getTarget('#sb-editor-help-btn')
			|| targetHasMask
			|| event.getTarget('.x-window-dlg')
			|| (Ext.fly(event.target).hasClass('sb-frame') && ((event.xy[0] >= event.target.clientWidth) || (event.xy[1] >= event.target.clientHeight)))
			|| event.button !== 0
			|| (Ext.isSafari && targetHasMask && (
					event.xy[0] > event.target.clientWidth
					|| event.xy[1] > event.target.clientHeight
					|| event.xy[0] < 0
					|| event.xy[1] < 0
				))
			|| event.getTarget('.x-combo-list')
			|| event.getTarget('.sb-color-dropdown')
		) {
			return;
		}
		this.completeEdit();
	},

	/**
	 * Gets class for some state
	 *
	 * @param {String} state
	 * @return {String} class
	 **/
	getStateCls: function (state) {
		var cls = this.baseCls,
			el = this.el.up('.sb-widget');

		if (el && Ext.getCmp(el.id).model.isInSiteWideArea()) {
			cls += '-shared';
		}
		cls += '-' + state;

		return cls;
	},

	resetStyles: function () {
		this.getEl().setStyle({
			position	: 'relative',
			left		: 'auto',
			top			: 'auto',
			width		: '100%'
		});
		this.removeClass(this.getStateCls('over'));
	},

	// support manager interface
	setZIndex: function (zindex) {
		this.getEl().setStyle('z-index', zindex);
	},

	// support manager interface
	setActive: Ext.emptyFn,

	beforeDestroy: function () {
		Sb.editor.component.Base.superclass.beforeDestroy.call(this);
		if (Sb.Editor.Ui.editComponent == this) {
			this.hideEdit();
		}
		if (this.menu) {
			Ext.destroy(this.menu);
		}
		if (this.controlsEl) {
			this.controlsEl.removeAnchor();
		}
	},

	onDestroy: function () {
		if (this.dd) {
			this.dd.destroy();
		}
		Sb.editor.component.Base.superclass.onDestroy.call(this);

		if (this.showOverTask) {
			this.showOverTask.cancel();
		}

		this.manager.unregister(this);
		if (Sb.Editor.Ui.overComponent == this) {
			if (Sb.Editor.Ui.overComponent.showOverTask) {
				Sb.Editor.Ui.overComponent.showOverTask.cancel();
			}
			Sb.Editor.Ui.overComponent = null;
		}
		if (Sb.Editor.Ui.editComponent == this) {
			Sb.Editor.Ui.editComponent = null;
		}
	},

	/**
	 * Brings this component/widget to the front of any other
	 */
	toFront: function () {
		var el = this.el.up('.sb-widget');
		if (el) {
			// Brings parent component/widget to the front
			Ext.getCmp(el.id).toFront();
		}
		this.manager.bringToFront(this);
	},

	refreshStylePanel: function () {
		if (Sb.Editor.Ui.isShowStylePanel) {
			this.getStylePanel().show(this);
			this.stylePropertiesEl.addClass(this.baseCls + '-control-item-disabled');
		} else {
			this.stylePropertiesEl.child('span').update(Sb.Locale.get('propertiesStyleWidgetShow'));
			this.stylePropertiesEl.removeClass(this.baseCls + '-control-item-disabled');
		}
	}
});

//private - custom Component DD implementation
Ext.define('Sb.editor.component.Base.DD', {
	extend: 'Ext.dd.DragSource',

	constructor: function (component, cfg) {
		this.component = component;
		cfg = Ext.apply({ddGroup: 'widgets'}, cfg);

		Sb.editor.component.Base.DD.superclass.constructor.call(this, this.component.el, cfg);
		var el = this.component.el;
		if (this.component.hasControls && this.component.ddEl) {
			el = this.component.ddEl;
			this.setOuterHandleElId(el.id);
		}
		el.setStyle('cursor', 'move');
		this.scroll = false;
	},

	getDragEl: function () {
		return Ext.getDom(this.component.getEl());
	},

	b4MouseDown: function (e) {
		var x = e.getPageX();
		var y = e.getPageY();
		this.autoOffset(x, y);
	},

	autoOffset: function (x, y) {
		x -= this.startPageX;
		y -= this.startPageY;
		this.setDelta(x, y);
	},

	startDrag: function (x, y) {
		Sb.Editor.siteView.onWidgetStartDrag();

		if (Sb.Editor.Ui.editComponent) {
			Sb.Editor.Ui.editComponent.completeEdit();
			if (!this.component.controlsEl.isVisible()) {
				this.component.showControls();
			}
		}

		// :TODO: echernodarov 20100420: dirty hack, need rework
		this.component.afterDD = true;
		this.component.setZIndex(Sb.Editor.windowManager.dragZIndex);
		if (this.component.ownerCt) {
			this.component.ownerCt.notifyOver(this, [x, y]);
		}
		this.component.el.setStyle({
			position: 'absolute',
			width: this.component.el.getWidth() + 'px'
		});
	},

	endDrag: function (e) {
		Sb.Editor.siteView.onWidgetEndDrag(this.component);
	},

	onInvalidDrop: function () {
		var animCfgObj = {
			easing		: 'elasticOut',
			duration	: 0.3,
			scope		: this,
			callback	: function () {
				this.component.resetStyles();
			}
		};

		this.el.moveTo(this.startPageX, this.startPageY, animCfgObj);
	},
	createFrame	: Ext.emptyFn,
	showFrame	: Ext.emptyFn
});