/**
 * @class Sb.editor.component.Image
 * @extends Sb.editor.component.Base
 * Class for image component
 * @constructor
 * @param {Object} config The config object
 * @xtype image_component
 */
Ext.define('Sb.editor.component.Image', {
	extend: 'Sb.editor.component.Base',
	xtype: 'image_component',

	removeConfirmation: false,
	imageUrl: '',
	imagePath: '',
	imageSize: {width: '', height: ''},
	isProportional: true,
	editPanelConfig: {},

	onRender: function () {
		Sb.editor.component.Image.superclass.onRender.apply(this, arguments);

		this.updateImage(this.imageUrl);
		this.setProportional(this.isProportional);
		this.updateAlignment(this.alignment);
	},

	initComponent: function () {
		Sb.editor.component.Image.superclass.initComponent.apply(this, arguments);
		this.removeTitle = Sb.Locale.get('hideWidgetTitle');
	},

	getEditPanel: function () {
		if (!this.editPanel) {
			this.editPanel = new Sb.editor.component.ImageEditPanel(this.editPanelConfig);
			this.mon(this.editPanel, 'show', this.onEditPanelShow, this);
			this.mon(this.editPanel, 'imagechange', this.onImageChange, this);
			this.mon(this.editPanel, 'sizechange', this.onSizeChange, this);
			this.mon(this.editPanel, 'alignmentchange', this.onAlignmentChange, this);
			this.mon(this.editPanel, 'proportionschange', this.onProportionsChange, this);
		}
		return this.editPanel;
	},

	onStartEdit: function () {
		Sb.editor.component.Image.superclass.onStartEdit.apply(this);
		this.memento = Ext.apply({
			startImage: this.imageUrl,
			isProportional: this.isProportional,
			alignment: this.alignment
		}, this.imageSize);

		this.showResizer();
	},

	onCancelEdit: function () {
		Sb.editor.component.Image.superclass.onCancelEdit.apply(this);
		this.updateImage(this.memento.startImage);
		this.setProportional(this.memento.isProportional);
		this.updateSize({width: this.memento.width, height: this.memento.height});
		this.updateAlignment(this.memento.alignment);
		this.hideResizer();
	},

	onCompleteEdit: function () {
		this.hideResizer();
	},

	onEditPanelShow: function () {
		this.updateEditPanel();
	},

	getImageSize: function () {
		if (this.imageSize.width && this.imageSize.height) {
			return this.imageSize;
		}

		if (this.imageEl) {
			return this.imageEl.getSize();
		}

		return {width: 0, height: 0};
	},

	/**
	 * Sets this Image's src
	 * @param {String} url The url to an image
	 */
	updateImage: function (url) {
		this.imageUrl = url || this.imageUrl || Ext.BLANK_IMAGE_URL;
		this.hideResizer();
		if (this.imageEl) {
			Ext.destroy(this.imageEl);
		}

		if (Sb.utils.Flash.isFlash(this.imageUrl)) {
			this.imageEl = this.createFlashEl();
			this.onImageLoad();
		} else {
			this.imageEl = this.contentElement.createChild({
				tag: 'img',
				alt: ''
			});
			this.mon(this.imageEl, 'load', this.onImageLoad, this, {single: true});
			this.imageEl.set({'src': this.imageUrl});
		}

		this.originalSize = null;
	},

	createFlashEl: function () {
		return this.contentElement.createChild({
			tag: 'div',
			html: Sb.utils.Flash.createCode(
				this.imageUrl,
				this.imageSize.width,
				this.imageSize.height
			)
		});
	},

	/**
	 * Sets this Image's size
	 * @param {Object} size like {width, height}
	 */
	updateSize: function (size) {
		if (Sb.utils.Flash.isFlash(this.imageUrl) && (!size.width || !size.height)) {
			size = Sb.utils.Flash.getSize(this.imageUrl);
		}
		this.imageSize = size;

		if (!this.imageEl) {
			return;
		}

		if (size.width && size.height) {
			Sb.utils.Flash.isFlash(this.imageUrl)
				? Sb.utils.Flash.setObjectSize(this.imageEl, size)
				: this.imageEl.setSize('100%', '100%');
		} else {
			size = this.getImageSize();
			this.imageEl.setSize('', '');
		}
		this.el.setSize(size.width, size.height);

		this.updateEditPanel();
	},

	updateAlignment: function (alignment) {
		if (alignment == this.alignment) {
			return;
		}

		this.alignment = alignment;
		this.updateEditPanel();
	},

	/**
	 * Set image to keep original aspect ratio
	 * @param {Boolean} isProportional
	 */
	setProportional: function (isProportional) {
		if (isProportional == this.isProportional) {
			return;
		}

		this.isProportional = isProportional;
		this.updateEditPanel();
	},

	/**
	 * @param {Boolean} [reanchor]
	 */
	initEditPanel: function (reanchor) {
		var editPanel = this.getEditPanel();

		if (Sb.utils.Flash.isFlash(this.imageUrl)) {
			this.originalSize = Sb.utils.Flash.getSize(this.imageUrl);
			editPanel.originalImageSize = this.originalSize;
			this.updateEditPanel(reanchor);
		} else {
			Sb.Utils.getImageOriginalSize(this.imageUrl, function (originalSize) {
				this.originalSize = originalSize;
				editPanel.originalImageSize = this.originalSize;
				this.updateEditPanel(reanchor);
			}.createDelegate(this));
		}
	},

	/**
	 * @param {Boolean} [reanchor]
	 */
	updateEditPanel: function (reanchor) {
		reanchor = undefined === reanchor || reanchor;
		if (Sb.Editor.Ui.editComponent != this || !this.getEditPanel()) {
			return;
		}

		if (!this.originalSize) {
			this.initEditPanel(reanchor);
			return;
		}

		var editPanel = this.getEditPanel();

		editPanel.updateFields({
			imageSize: this.getImageSize(),
			isProportional: this.isProportional
		});

		if (reanchor) {
			editPanel.reanchor();
		}
	},

	showResizer: function () {
		this.resizer = new Ext.Resizable(this.id, {
			handles: 'all',
			pinned: true,
			minHeight: 20,
			minWidth: 20,
			maxHeight: 1000,
			maxWidth: 1000
		});
		this.resizer.on('beforeresize', this.onBeforeMouseResize, this);
		this.resizer.on('resize', this.onMouseResize, this);
	},

	hideResizer: function () {
		if (!this.resizer) {
			return;
		}

		this.el.removeClass('x-resizable-pinned');
		this.resizer.destroy();
	},

	onImageChange: function (editPanel, fileName) {
		this.imageSize = {width: '', height: ''};
		this.updateImage(this.imagePath + fileName);
	},

	onSizeChange: function (editPanel, size) {
		this.updateSize(size);
	},

	onAlignmentChange: function (editPanel, alignment) {
		this.updateAlignment(alignment);
	},

	onImageLoad: function () {
		this.updateSize(this.imageSize);

		if (Sb.Editor.Ui.editComponent == this) {
			var editPanel = this.getEditPanel();

			this.showResizer();
			editPanel.show(this);
			this.updateEditPanel();
		}
	},

	onProportionsChange: function (editPanel, isProportional) {
		this.setProportional(isProportional);
	},

	onBeforeMouseResize: function () {
		this.lastPosition = this.el.getPositioning();
	},

	onMouseResize: function (resizer, width, height) {
		this.setProportional(false);
		this.el.setPositioning(this.lastPosition);
		this.updateSize({width: width, height: height});
	},

	initDD: function () {
		this.dd = new Ext.dd.DD(this);
		this.dd.setOuterHandleElId(this.ddEl);
		this.dd.component = this;
	}
});