/**
 * @class Sb.form.ColorDropDown
 * @extends Ext.form.TriggerField
 * @xtype colordropdown
 */
Ext.define('Sb.form.ColorDropDown', {
	extend: 'Ext.form.TriggerField',
	xtype: 'colordropdown',
	triggerClass: 'x-form-color-trigger',

	invalidText: Sb.Locale.get('colorsFormat'),

	/**
	 * @cfg {Boolean} allowBackground Is allow background image (gradient/custom/etc.)
	 */
	allowBackground: false,

	/**
	 * @cfg {Boolean} allowHeaders Is allow header library for selection background
	 */
	allowHeaders: false,

	/**
	 * @cfg {Boolean} allowOpacity Is allow opacity for solid color background and images
	 */
	allowOpacity: false,

	/**
	 * @cfg {Boolean} allowStroke True to allow select outline (defaults to false).
	 */
	allowStroke: false,

	/**
	 * @cfg {Boolean} allowInherit True to allow select inherit (Not set) color (defaults to false).
	 */
	allowInherit: false,

	/**
	 * @cfg {String} parentColor Real value for inherit (Not set) color (defaults to '#000000').
	 */
	parentColor: '#000000',

	/**
	 * @cfg {Boolean} allowColorPalette True to allow select colors from color scheme (defaults to false).
	 */
	allowColorPalette: false,

	validationEvent: false,
	validateOnBlur: false,

	constructor: function (config) {
		config = Ext.apply({
			allowBlank: false, // Is allow transparent color and show "No color" button
			isShowColorField: false,
			editable: true,
			maskRe: /[#0-9A-F]/i,
			cls: 'sb-form-color-field',
			menuCls: '',
			ignoreHideClasses: ['.x-combo-list', '.sb-selectbox-list']
		}, config);

		this.backgroundValue = {
			type: 'none'
		};

		Sb.form.ColorDropDown.superclass.constructor.call(this, config);
	},

	initComponent: function () {
		Sb.form.ColorDropDown.superclass.initComponent.call(this);

		this.menu = new Sb.menu.ColorMenu({
			cls: this.menuCls,
			allowBlank: this.allowBlank,
			isShowColorField: this.isShowColorField,
			allowBackground: this.allowBackground,
			allowHeaders: this.allowHeaders,
			value: this.value,
			allowStroke: this.allowStroke,
			allowInherit: this.allowInherit,
			parentColor: this.parentColor,
			allowOpacity: this.allowOpacity,
			allowColorPalette: this.allowColorPalette,
			listeners: {
				scope: this,
				beforehide: function () {
					// when clicking on trigger, ignore hiding the menu
					// caused by mousedown event provided by Ext.menu.MenuMgr
					var disableHide = false;

					Ext.each(this.ignoreHideClasses || [], function (cls) {
						disableHide = Ext.EventObject.getTarget(cls);
						return !disableHide;
					});

					return Ext.EventObject.type !== 'mousedown'
						|| !( Ext.EventObject.getTarget() === this.trigger.dom
							|| disableHide);
				},
				hide: function () {
					this.focus();
				},
				change: function (e, value) {
					this.setValue(value);
					this.fireEvent('change', this, value);
				},
				backgroundchange: function (colormenu, background) {
					this.backgroundValue = background;
					this.updateControl();
				}
			}
		});
		this.menu.ownerCt = this;
		/**
		 * @event backgroundchange
		 * Fires when a background is selected from the {@link #menu Sb.menu.ColorMenu}
		 * @param {Sb.menu.ColorMenu} colormenu The {@link #menu Sb.menu.ColorMenu}
		 * @param {Object} background The background properties
		 */
		this.relayEvents(this.menu, ['backgroundchange']);
	},

	initEvents: function () {
		this.mon(this.el, 'keyup', this.onKeyUp, this);
	},

	onKeyUp: function () {
		var value = this.getRawValue();
		if (Ext.ux.util.Color.isValid(value) && value != this.value) {
			this.backgroundValue.type = 'none';
			this.setValue(value);
			this.fireEvent('change', this, value);
			if (this.allowBackground) {
				this.fireEvent('backgroundchange', this.menu, this.backgroundValue);
			}
		}
	},

	getMenuParent: function () {
		return document.body;
	},

	getParentZIndex: function () {
		var zIndex;
		this.findParentBy(function (ct) {
			if (!ct.getPositionEl()) {
				return false;
			}
			zIndex = parseInt(ct.getPositionEl().getStyle('z-index'), 10);
			return !!zIndex;
		});
		return zIndex;
	},

	getZIndex: function () {
		var menuParent = Ext.getDom(this.getMenuParent()),
			zIndex = parseInt(Ext.fly(menuParent).getStyle('z-index'), 10);

		if (!zIndex) {
			zIndex = this.getParentZIndex();
		}

		return (zIndex || 12000) + 5;
	},

	setValue: function (color) {
		this.value = color;
		if (this.rendered) {
			this.updateControl();
		}
		this.menu.setRawValue(color);
	},

	getValue: function () {
		return this.value;
	},

	/**
	 * Runs ColorDropDown's validations and returns an array of any errors.
	 *
	 * @param {mixed} [value] The value to get errors for (defaults to the current field value)
	 * @return {Array} All validation errors for this field
	 */
	getErrors: function (value) {
		var errors = Sb.form.ColorDropDown.superclass.getErrors.call(this, value);

		value = Sb.Editor.model.design.getColor(this.value);

		if (value === 'transparent') {
			if (!this.allowBlank) {
				errors.push(this.blankText);
			}
			return errors;
		}

		if (value === 'inherit') {
			if (!this.allowInherit) {
				errors.push(this.invalidText);
			}
			return errors;
		}

		if (!Ext.ux.util.Color.isValid(value)) {
			errors.push(this.invalidText);
		}

		return errors;
	},

	updateControl: function () {
		var backgroundColor = this.value,
			rawValue;

		this.removeClass([
			'sb-form-color-field-gradient',
			'sb-form-color-field-image',
			'sb-form-color-field-auto',
			'sb-form-color-field-transparent',
			'sb-form-color-field-inherit'
		]);

		if (this.backgroundValue.type === 'gradient') {
			rawValue = Sb.Locale.get('backgroundGradient');
			backgroundColor = '#FFFFFF';
			this.addClass('sb-form-color-field-gradient');
		} else if (this.backgroundValue.type === 'pattern' || this.backgroundValue.type === 'image') {
			rawValue = Sb.Locale.get(
				this.backgroundValue.type === 'pattern' ? 'backgroundPattern' : 'backgroundImage'
			);
			backgroundColor = '#FFFFFF';
			this.addClass('sb-form-color-field-image');
		} else {
			if (backgroundColor === 'transparent') {
				backgroundColor = '#FFFFFF';
				rawValue = Sb.Locale.get('transparentColor');
				this.addClass('sb-form-color-field-transparent');
			} else if (backgroundColor === 'inherit') {
				backgroundColor = Sb.Editor.model.design.getColor(this.parentColor);
				rawValue = Sb.Locale.get('inheritColor');
				this.addClass('sb-form-color-field-inherit');
			} else if (!Ext.ux.util.Color.isValid(backgroundColor)) {
				rawValue = Sb.Locale.get('palette' + backgroundColor.toUpperCaseFirst());
				backgroundColor = Sb.Editor.model.design.getColor(backgroundColor);
			} else {
				rawValue = backgroundColor;
			}
		}

		this.el.setStyle({
			background: backgroundColor
		});
		this.setRawValue(rawValue);
	},

	onResize: function (aw, ah, w, h) {
		Sb.form.ColorDropDown.superclass.onResize.call(this, aw, ah, w, h);
		if (Ext.isNumber(aw)) {
			this.el.setWidth(aw - this.el.getMargins('lr') - this.getTriggerWidth());
		}
	},

	/**
	 * Show/hide menu
	 *
	 * @param {Ext.EventObject} e
	 */
	onTriggerClick: function (e) {
		if (this.menu.isVisible()) {
			this.menu.hide();
		} else {
			this.expand();
		}
	},

	/**
	 * Expands the dropdown color menu.
	 */
	expand: function () {
		this.menu.setBackgroundValue(this.backgroundValue);
		this.menu.show(this.wrap, this.menu.defaultAlign, this.parentMenu);
		this.menu.getEl().setZIndex(this.getZIndex());
	},

	setAllowBlank: function (value) {
		this.allowBlank = value;
		this.menu.setAllowBlank(value);
	},

	setParentColor: function (value) {
		this.parentColor = value;
		this.menu.setParentColor(value);
		if (this.rendered) {
			this.updateControl();
		}
	},

	setBackgroundValue: function (background) {
		this.backgroundValue = background;
		this.menu.setBackgroundValue(background);
		if (this.rendered) {
			this.updateControl();
		}
	},

	setBackgroundUploadParams: function (url, params) {
		this.menu.uploadUrl = url;
		this.menu.uploadParams = params;
	},

	setAllowHeaders: function (value) {
		this.menu.setAllowHeaders(value);
	}
});