/**
 * @class Sb.form.ColorField
 * @extends Ext.form.DisplayField
 * Class for color field
 * @constructor
 * @param {Object} config The config object
 * @xtype colorfield
 */
Ext.define('Sb.form.ColorField', {
	extend: 'Ext.form.TextField',
	xtype: 'colorfield',

	/**
	 * @cfg {String} itemCls The CSS class applies to whole control with label
	 */
	itemCls				: 'sb-colorfield-area',

	/**
	 * @cfg {Ext.ux.ColorPicker} currentPicker Current color picker this field binded to
	 */
	currentPicker		: null,

	/**
	 * @cfg {Boolean} isUseHEXHelper Does field should use HEX helper?
	 */
	isUseHEXHelper		: true,

	/**
	 * @cfg {Boolean} isResetOnBlur Does this field should be resetted on blur in case of error?
	 */
	isResetOnBlur		: false,

	regex				: /^#([0-9A-F]{3}|[0-9A-F]{6})$/i,
	maskRe				: /[#0-9A-F]/i,
	invalidText			: Sb.Locale.get('colorsFormat'),
	maxLength			: 7,
	allowBlank			: false,

	/**
	 * Default field wrapper CSS class
	 *
	 * @type String
	 * @property defaultCls
	 */
	defaultCls			: 'sb-colorfield-wrapper',

	/**
	 * Selected field wrapper CSS class
	 *
	 * @type String
	 * @property selectedCls
	 */
	selectedCls			: 'sb-colorfield-wrapper-selected',

	/**
	 * Invalid field wrapper CSS class
	 *
	 * @type String
	 * @property invalidCls
	 */
	invalidCls			: 'sb-colorfield-wrapper-invalid',

	/**
	 * Flag for selected state
	 *
	 * @type Boolean
	 * @property isSelected
	 */
	isSelected			: false,

	/**
	 * Flag for valid state
	 *
	 * @type Boolean
	 * @property isValidatedOk
	 */
	isValidatedOk				: true,

	/**
	 * Start value when user enters the field
	 *
	 * @type String
	 * @property selectedStartValue
	 */
	selectedStartValue	: null,

	initEvents: function () {
		Sb.form.ColorField.superclass.initEvents.apply(this, arguments);
		this.mon(this, 'valid', this.onValid);
		this.mon(this, 'invalid', this.onInvalid);
	},

	onRender: function (container, position) {
		Sb.form.ColorField.superclass.onRender.apply(this, arguments);
		this.el.addClass('sb-colorfield');
		this.wrap		= this.el.wrap({cls: this.defaultCls});
		this.resizeEl	= this.positionEl = this.wrap;
	},

	afterRender: function (container) {
		Sb.form.ColorField.superclass.afterRender.apply(this, arguments);
		if (this.isUseHEXHelper) {
			Ext.QuickTips.registerSelfAndChildren({
				target: this, text: Sb.Locale.get('colorCanEditHEXManually')
			});
		}
	},

	/**
	 * Updates color on success validation
	 * @param {Boolean} suppressEvent True to suppress the 'invalid' event from being fired
	 */
	unsetActiveError: function (suppressEvent) {
		this.updateColor(this.getValue());
		return Sb.form.ColorField.superclass.unsetActiveError.call(this, suppressEvent);
	},

	/**
	 * Sets colorfield value directly, bypassing validation.
	 *
	 * @param {String} color The color to set
	 * @param {Boolean} [isValidatedOk] When calling object knows, that value is valid
	 * @return {Sb.form.ColorField} this
	 */
	setRawValue: function (color, isValidatedOk) {
		this.updateColor(color);
		if (color === 'transparent') {
			color = Sb.Locale.get('transparentColor');
		}
		if (isValidatedOk) {
			this.isValidatedOk = true;
			this.activeError = '';
			this.updateWrapperClasses();
		}
		this.value = color;
		return Sb.form.ColorField.superclass.setRawValue.call(this, color);
	},

	/**
	 * Updates colorfield styles
	 */
	updateColor: function (color) {
		if (this.rendered) {
			var textColor = '#000000';
			if (color == null) {
				color = '#FFFFFF';
			} else {
				textColor = Ext.ux.util.Color.isLight(color) ? '#000000' : '#FFFFFF';
			}
			this.el.setStyle({
				'background-color': color,
				'color': textColor
			});
		}
	},

	/**
	 * Remove selected state and try to reset on error
	 */
	beforeBlur: function () {
		if (this.readOnly) {
			return;
		}
		this.isSelected = Boolean(this.leaveSelection);
		if (this.activeError && this.isResetOnBlur) {
			this.setRawValue(this.startValue);
		}
		this.updateWrapperClasses();
		Sb.form.ColorField.superclass.beforeBlur.apply(this, arguments);
	},

	/**
	 * Add selected state and try to update current picker
	 */
	preFocus: function () {
		Sb.form.ColorField.superclass.preFocus.apply(this, arguments);
		if (this.isValidatedOk) {
			this.selectedStartValue = this.getValue();
		}
		if (this.readOnly) {
			return;
		}
		this.isSelected = true;
		this.updateCurrentPicker();
		this.updateWrapperClasses();
	},

	/**
	 * Valid handler
	 */
	onValid: function (me) {
		this.isValidatedOk = true;
		this.updateCurrentPicker();
		this.updateWrapperClasses();
	},

	/**
	 * Invalid handler
	 */
	onInvalid: function (me, errorMsg) {
		this.isValidatedOk = false;
		this.updateWrapperClasses();
	},

	/**
	 * Updates wrapper classes, regarding of current state flags
	 */
	updateWrapperClasses: function () {
		if (!this.rendered) {
			return;
		}
		var className = this.defaultCls;
		if (!this.isValidatedOk) {
			className = this.invalidCls;
		} else if (this.isSelected) {
			className = this.selectedCls;
		}
		this.wrap.dom.className = className;
	},

	/**
	 * When we have current picker we update it`s raw value
	 */
	updateCurrentPicker: function () {
		if (null == this.currentPicker) {
			return;
		} else if (Ext.isString(this.currentPicker)) {
			this.currentPicker = Ext.getCmp(this.currentPicker);
		}
		this.currentPicker.setRawValue(this.isValidatedOk ? this.getValue() : this.selectedStartValue);
	}
});