/**
 * @class Sb.form.SelectBox
 * @extends Ext.form.ComboBox
 * A selectbox control.
 * @constructor
 * Create a new SelectBox.
 * @param {Object} config Configuration options
 * @xtype sbselectbox
 */
Ext.define('Sb.form.SelectBox', {
	extend: 'Ext.form.ComboBox',
	xtype: 'sbselectbox',

	defaultAutoCreate: {
		tag: 'div'
	},

	cls: 'sb-selectbox',

	listClass: 'sb-selectbox-list',

	itemClass: 'sb-selectbox-list-item',

	itemOverClass: 'sb-selectbox-list-item-over',

	itemSelectedClass: 'sb-selectbox-list-item-selected',

	/**
	 * @cfg {Number} columns The number of columns in page.
	 */
	columns: 6,

	/**
	 * @cfg {Number} rows The number of rows in page.
	 */
	rows: 3,

	/**
	 * @cfg {Number} itemAspectRatio The aspect ratio of an item describes
	 * the proportional relationship between its width and its height.
	 */
	itemAspectRatio: 1,

	constructor: function (config) {
		config = Ext.apply(config || {}, {
			editable: false,
			itemSelector: '.' + this.itemClass,
			dataWasLoaded: false
		});
		config.cls = this.cls + ' ' + (config.cls || '');
		config.listClass = this.listClass + ' ' + (config.listClass || '');
		config.itemClass = this.itemClass + ' ' + (config.itemClass || '');
		Sb.form.SelectBox.superclass.constructor.call(this, config);
	},

	initComponent: function () {
		if (this.itemRenderer) {
			this.itemTpl = '{[this.itemRenderer(values)]}';
		}
		if (this.itemTpl) {
			this.tpl = new Ext.XTemplate(
				'<tpl for="."><div' +
					' id="' + this.id + '-item-{index}"' +
					' class="' + this.itemClass +
					'{[this.backgroundColor == \'transparent\' ? \' sb-selectbox-list-item-transparent\' : \'\']}"' +
					'style="' +
					'<tpl if="values.width">width: {width}px;</tpl>' +
					'<tpl if="values.height">height: {height}px;</tpl>' +
					'{[this.backgroundColor ? \' background-color: \' + this.backgroundColor + \';\' : \'\']}' +
					'">' +
				this.itemTpl + '{[this.toolsRenderer(values)]}</div></tpl>',
				{
					itemRenderer: this.itemRenderer,
					toolsRenderer: Ext.createDelegate(function (itemData) {
						var html = '';
						if (itemData.index == 'selected') {
							return html;
						}
						Ext.iterate(this.itemTools || {}, function (toolName, tool) {
							if (tool.isAvailable(itemData)) {
								html += '<div class="sb-selectbox-list-item-' + toolName + '">&#160;</div>';
							}
						});
						return html;
					}, this)
				}
			);
		}

		Sb.Loading.updateMessage(Sb.Locale.get('loadingImages'));
		Sb.form.SelectBox.superclass.initComponent.call(this);

		this.addEvents(
			/**
			 * @event beforecollapse
			 * Fires before the dropdown list is collapsed. A handler can return false to cancel the collapse.
			 * @param {Sb.form.SelectBox} selectbox This select box
			 */
			'beforecollapse'
		);
	},

	initList: function () {
		if (this.list) {
			return;
		}

		var cls = 'x-combo-list',
			listParent = Ext.getDom(this.getListParent() || Ext.getBody());

		this.list = new Ext.Layer({
			parentEl: listParent,
			shadow: false,
			cls: [cls, this.listClass].join(' '),
			constrain: false,
			zindex: this.getZIndex(listParent)
		});

		this.listFocusEl = this.list.createChild({
			tag : 'a',
			cls: 'x-menu-focus',
			href: '#',
			onclick: 'return false;',
			tabIndex: '-1'
		});

		this.listKeyNav = new Ext.KeyNav(this.list, {
			scope: this,
			tab: function (e) {
				e.stopEvent();
			},
			esc: this.collapse,
			enter: this.collapse,
			left: this.prevPage,
			right: this.nextPage,
			up: this.prevPage,
			down: this.nextPage,
			space: this.nextPage,
			pageUp: this.firstPage,
			pageDown: this.lastPage,
			home: this.firstPage,
			end: this.lastPage
		});

		this.list.swallowEvent('mousewheel');
		this.mon(this.list, {
			mousewheel: this.onListMouseWheel,
			click: function (e) {
				if (e.getTarget('.x-form-field', this.list)) {
					return;
				}
				this.listFocusEl.focus();
			},
			scope: this
		}, this);
		this.changePageTask = new Ext.util.DelayedTask();

		this.assetHeight = 7;
		if (this.header) {
			this.header = new Ext.Container(Ext.apply({
				renderTo: this.list,
				cls: 'x-combo-list-hd',
				items: this.header
			}, this.headerCfg || {}));
			this.header.el.swallowEvent(['keypress', 'keydown']);
			this.assetHeight += this.header.getHeight();
		}

		this.innerList = this.list.createChild({cls: cls + '-inner'});

		this.mon(this.innerList, {
			click: this.onListClick,
			mouseover: this.onListMouseOver,
			mouseout: this.onListMouseOut,
			scope: this
		});

		this.paging = this.list.createChild({
			cls: 'sb-selectbox-paging',
			cn: {
				cls: 'sb-selectbox-paging-pages'
			}
		});
		this.assetHeight += this.paging.getHeight();
		this.assetHeight += this.paging.getMargins('tb');

		this.itemSize = {
			width: 0,
			height: 0
		};
		this.pagePadding = {
			top: 0,
			left: 0
		};

		this.toolTarget = this.list.createChild({cls: 'sb-selectbox-list-tools'});
		this.close = this.toolTarget.createChild({cls: 'x-tool x-tool-close', html: '&#160;'});
		this.close.addClassOnOver('x-tool-close-over');
		this.close.on('click', this.collapse, this);

		this.prev = this.innerList.createChild({
			cls: 'sb-selectbox-prev'
		});
		this.prev.addClassOnOver('sb-selectbox-prev-over');
		this.prev.on('click', this.prevPage, this);

		this.next = this.innerList.createChild({
			cls: 'sb-selectbox-next'
		});
		this.next.addClassOnOver('sb-selectbox-next-over');
		this.next.on('click', this.nextPage, this);

		this.pagesCt = this.innerList.createChild({
			cls: 'sb-selectbox-pages'
		});

		this.bindStore(this.store, true);
	},

	getZIndex: function (listParent) {
		return Math.max(
			Sb.form.SelectBox.superclass.getZIndex.call(this, listParent),
			Sb.form.SelectBox.superclass.getZIndex.call(this, Sb.Editor.Ui.mainWindow.el)
		);
	},

	bindStore: function (store, initial) {
		if (this.store) {
			this.store.un('datachanged', this.refreshPaging, this);
			this.store.un("remove", this.refreshPaging, this);
		}

		Sb.form.SelectBox.superclass.bindStore.apply(this, arguments);

		if (!store) {
			return;
		}

		this.store.on('datachanged', this.refreshPaging, this);
		this.store.on("remove", this.refreshPaging, this);
	},

	onBeforeLoad: function () {
		if (!this.hasFocus) {
			return;
		}
		this.innerList.update(this.loadingText ?
				'<div class="loading-indicator">' + this.loadingText + '</div>' : '');
		this.selectedIndex = -1;
	},

	onLoad: function () {
		if (!this.hasFocus) {
			return;
		}
		var count = this.store.getCount();
		if (count > 0 || this.listEmptyText) {
			if (count > 0) {
				this.dataWasLoaded = true;
				Sb.Loading.hide();
			}
			this.expand();
		} else {
			this.collapse();
		}
	},

	setBackgroundColor: function (color) {
		this.tpl.backgroundColor = color;
		if (this.el) {
			var els = this.el.select('.sb-selectbox-list-item');
			if (this.currPageEl) {
				els.add(this.currPageEl.select('.sb-selectbox-list-item'));
			}
			els.setStyle('background-color', color);
			els[(color == 'transparent' ? 'add' : 'remove') + 'Class']('sb-selectbox-list-item-transparent');
		}
	},

	setValue: function (value) {
		Sb.form.SelectBox.superclass.setValue.call(this, value);

		if (this.rendered) {
			var recordFoundOnTheStoreByValue = this.findRecord(this.valueField, value);

			if (!this.elData) {
				this.elData = {
					index: 'selected'
				};
			}

			if (recordFoundOnTheStoreByValue) {
				Ext.apply(this.elData, recordFoundOnTheStoreByValue.data);
			} else {
				this.elData[this.valueField] = value;
			}
			this.tpl.overwrite(this.el, this.elData);
		}

		return this;
	},

	getRawValue: function () {
		return this.getValue();
	},

	expand: function () {
		if (this.isExpanded() || !this.hasFocus) {
			return;
		}
		if (!this.dataWasLoaded) {
			Sb.Loading.show();
			return;
		}
		this.onWindowResize();
		this.list.setZIndex(this.getZIndex());
		this.list.show();
		this.listFocusEl.focus(50);
		Ext.EventManager.onWindowResize(this.onWindowResize, this);
		this.refreshPaging();

		this.fireEvent('expand', this);
	},

	collapse: function() {
		if (!this.isExpanded() || this.fireEvent('beforecollapse', this) === false) {
			return;
		}

		this.list.hide();
		Sb.Loading.hide();
		Ext.EventManager.removeResizeListener(this.onWindowResize, this);

		if (this.currPageEl) {
			this.currPageEl.remove();
			this.currPageEl = null;
		}

		this.fireEvent('collapse', this);
	},

	onWindowResize: function () {
		var vs = Ext.getBody().getViewSize(false);
		this.list.setXY([0, 0]);
		this.list.setSize(vs.width, vs.height);
		if (this.header) {
			this.header.setWidth(vs.width);
		}
		this.innerList.setSize(vs.width, vs.height - this.assetHeight);
		this.calculateItemSize();
		if (this.currPageEl) {
			this.renderPage(this.currentPage, this.currPageEl);
		}
	},

	select: function (index, scrollIntoView) {
		this.selectedIndex = index;

		if (scrollIntoView !== false) {
			this.changePage(Math.floor(index / (this.columns * this.rows)));
		}

		var node = Ext.fly(this.id + '-item-' + index);
		if (node) {
			node.addClass(this.itemSelectedClass);
		}
	},

	refreshPaging: function () {
		var pageSize = this.columns * this.rows,
			total = this.store.getCount(),
			pagesEl = this.paging.child('.sb-selectbox-paging-pages'),
			pageEl, i;

		this.pagesCount =  total < pageSize ? 1 : Math.ceil(total / pageSize);
		this.paging.setVisible(this.store.getTotalCount() > pageSize);

		pagesEl.update('');
		for (i = 0; i < this.pagesCount; i++) {
			pageEl = pagesEl.createChild({cls: 'sb-selectbox-paging-page'});
			pageEl.addClassOnOver('sb-selectbox-paging-page-over');
			pageEl.addClassOnClick('sb-selectbox-paging-page-click');
			pageEl.on('click', (function (page) {
				return function () {
					this.changePage(page);
				};
			}(i)), this);
		}

		this.selectedIndex = -1;
		if (!this.selectByValue(this.value, true)) {
			this.changePage(0);
		}
	},

	prevPage: function () {
		this.changePage(this.currentPage - 1);
	},

	nextPage: function () {
		this.changePage(this.currentPage + 1);
	},

	firstPage: function () {
		this.changePage(0);
	},

	lastPage: function () {
		this.changePage(this.pagesCount - 1);
	},

	changePage: function (page, skipRender) {
		page = page.constrain(0, this.pagesCount - 1);

		var pageEls = Ext.query('.sb-selectbox-paging-page', this.paging.dom),
			currPageEl = this.currPageEl,
			nextPageEl;

		Ext.each(pageEls, function (el, i) {
			Ext.fly(el)[i == page ? 'addClass' : 'removeClass']('sb-selectbox-paging-page-selected');
		});

		this.prev.setVisible(page > 0);
		this.next.setVisible(page < this.pagesCount - 1);

		this.currentPage = page;

		if (skipRender) {
			return true;
		}

		if (page === this.currentRenderedPage && currPageEl) {
			nextPageEl = currPageEl;
		} else {
			nextPageEl = this.pagesCt.createChild();
		}
		this.renderPage(page, nextPageEl);

		if (currPageEl && currPageEl != nextPageEl) {
			currPageEl.stopFx();
			currPageEl.slideOut(page > this.currentRenderedPage ? 'l' : 'r', {
				duration: 0.5,
				callback: function () {
					currPageEl.remove();
				}
			});
			nextPageEl.slideIn(page > this.currentRenderedPage ? 'r' : 'l', {
				duration: 0.5
			});
		}
		this.currPageEl = nextPageEl;
		this.currentRenderedPage = page;
	},

	renderPage: function (page, el) {
		el.setStyle({
			position: 'absolute',
			top: 0,
			left: 0,
			width: (this.innerList.getWidth() - this.pagePadding.left * 2) + 'px',
			height: (this.innerList.getHeight() - this.pagePadding.top * 2) + 'px',
			padding: this.pagePadding.top + 'px ' + this.pagePadding.left + 'px'
		});
		el.update('');

		var pageSize = this.columns * this.rows,
			offset = page * pageSize,
			records = this.store.getRange(offset, offset + pageSize - 1),
			data = [],
			i,
			len = records.length;

		if (!len && this.listEmptyText) {
			el.update('<div class="sb-selectbox-list-empty">' + this.listEmptyText + '</div>');
			el.first().center();
			return;
		}

		for (i = 0; i < len; i++) {
			data.push(Ext.apply({
				index: offset + i,
				width: this.itemSize.width,
				height: this.itemSize.height
			}, records[i].data));
		}
		this.tpl.overwrite(el, data);
		this.select(this.selectedIndex, false);
	},

	calculateItemSize: function () {
		var viewSize, viewAspectRatio, gridSize, gridAspectRatio,
			arrowWidth, minArrowWidth = 80;

		viewSize = this.innerList.getSize(true);
		viewSize.width -= 16* this.columns + minArrowWidth * 2;
		viewSize.height -= 16 * this.rows;
		viewAspectRatio = viewSize.width / viewSize.height;

		gridAspectRatio = this.columns / this.rows * this.itemAspectRatio;
		if (viewAspectRatio < gridAspectRatio) {
			gridSize = {
				width: viewSize.width,
				height: viewSize.width / gridAspectRatio
			};
		} else {
			gridSize = {
				width: viewSize.height * gridAspectRatio,
				height: viewSize.height
			};
		}

		this.itemSize = {
			width: Math.floor(gridSize.width / this.columns),
			height: Math.floor(gridSize.height / this.rows)
		};

		arrowWidth = Math.floor((viewSize.width - this.itemSize.width * this.columns) / 2) + minArrowWidth;
		this.prev.setWidth(arrowWidth);
		this.next.setWidth(arrowWidth);

		this.pagePadding = {
			left: arrowWidth,
			top: Math.floor((viewSize.height - this.itemSize.height * this.rows) / 2)
		};
	},

	onListMouseOver: function (e) {
		var item = e.getTarget(this.itemSelector, this.innerList);
		if (item && item !== this.lastItem) {
			this.lastItem = item;
			Ext.fly(item).addClass(this.itemOverClass);
		}
	},

	onListMouseOut: function (e) {
		if (this.lastItem) {
			if (!e.within(this.lastItem, true, true)) {
				Ext.fly(this.lastItem).removeClass(this.itemOverClass);
				delete this.lastItem;
			}
		}
	},

	onListClick: function (e) {
		var item = e.getTarget(this.itemSelector, this.innerList),
			index,
			record,
			clickedTool;

		if (item) {
			index = Ext.get(item).id.split('-').pop();
			record = this.store.getAt(index);

			Ext.iterate(this.itemTools || {}, function (toolName, tool) {
				if (e.getTarget('.sb-selectbox-list-item-' + toolName)) {
					clickedTool = toolName;
					return false;
				}
			}, this);
			if (clickedTool && this.itemTools[clickedTool].handler) {
				this.itemTools[clickedTool].handler.call(this.itemTools[clickedTool].scope || this, this, record);
			} else {
				this.onSelect(record, index);
			}
		}
	},

	onListMouseWheel: function (e) {
		var delta, nextPage;

		e.stopEvent();
		delta = e.getWheelDelta();
		nextPage = this.currentPage + (delta > 0 ? -1 : 1);

		this.changePage(nextPage, true);
		this.changePageTask.delay(250, this.changePage, this, [nextPage]);
	}

	/**
	 * @cfg {Boolean} editable @hide
	 */
});