/**
 * @class Sb.model.Document
 * @extends Ext.util.Observable
 * Base class for document model
 * @constructor
 * @param {Object} config The configuration options for the model element
 */
Ext.define('Sb.model.Document', {
	extend: 'Ext.util.Observable',

	isLoaded: false,

	store: new Ext.data.ArrayStore({
		fields: ['name', 'file', 'extension', 'size', 'date', 'type', 'shortName']
	}),

	constructor: function (config) {
		this.addEvents(
			/**
			 * @event load
			 * Fires when document model loaded
			 */
			'load',

			/**
			 * @event remove
			 * Fires when documents removed
			 */
			'remove'
		);
		Sb.model.Document.superclass.constructor.call(this);
	},

	/**
	 * Get resources data from server
	 */
	load: function () {
		Sb.SiteService.getDocuments(function (result, response) {
			if (result.success) {
				this.loadData(result.files);
				this.isLoaded = true;
			} else {
				Sb.Editor.Ui.mainWindow.statusBar.setMessageError(
					Sb.Locale.get('getDocumentsListError')
				);
			}
			this.fireEvent('load', result);
		}.createDelegate(this));
	},

	/**
	 * Load resources data to store
	 * @param resources
	 * @param append Default false Append new resources to store
	 * @returns {Sb.model.Document}
	 */
	loadData: function (resources, append) {
		append = (append) ? true : false;
		var data = [];
		if (resources instanceof Ext.data.Record) {
			resources = data.push(this.recordToArray());
		}
		if (!(resources instanceof Array)) {
			throw 'Resource should be instanceof Array or Ext.data.Record';
		}
		Ext.each(resources, function (resource) {
			data.push(this.recordToArray(this.dataToRecord(resource)));
		}, this);
		this.store.loadData(data, append);
		return this;
	},

	/**
	 *
	 * @param record
	 * @returns {Array}
	 */
	recordToArray: function (record) {
		if (!(record instanceof Ext.data.Record)) {
			throw 'Record should be instanceof Ext.data.Record';
		}
		var data = [];
		Ext.iterate(record.data, function (key, value) {
			data.push(value);
		});
		return data;
	},

	/**
	 * Converd data to Ext.data.Record for current store
	 * @param {array|object|Ext.data.Record} data
	 * @returns {Ext.data.Record}
	 */
	dataToRecord: function (data) {
		if (data instanceof Ext.data.Record) {
			return data;
		}

		var recordClassConfig = [];
		this.store.fields.each(function (field) {
			var recordField = {};
			recordField.name = field.name;
			recordField.mapping = 'map_' + field.name;
			recordClassConfig.push(recordField);
		});

		var recordClass = Ext.data.Record.create(recordClassConfig);
		var recordData = {};
		if (data instanceof Array) {
			Ext.each(data, function (fieldValue, index) {
				recordData[this.store.fields.get(index)] = fieldValue;
			}, this);
		} else if (data instanceof Object) {
			var recordDataUnsorted = {};
			Ext.iterate(data, function (key, value) {
				if (this.isField(key)) {
					recordDataUnsorted[key] = value;
				}
			}, this);
			//order recordData by fields
			this.store.fields.each(function (field) {
				recordData[field.name] = recordDataUnsorted[field.name];
			});
		} else {
			throw 'value to convert to record object should be array or object';
		}
		return new recordClass(recordData);
	},

	/**
	 * Allowed field in a store of resource manager
	 * @param fieldName
	 * @returns {Boolean}
	 */
	isField: function (fieldName) {
		return this.store.fields.containsKey(fieldName);
	},

	/**
	 * Remove resources from server
	 * @param {array} documentsToRemove
	 */
	remove: function (documentsToRemove) {
		Sb.SiteService.removeDocument(documentsToRemove, function (result, response) {
			if (!result.files.length) {
				return;
			}
			result.deletedDocuments = [];
			result.failedDocuments = [];

			Ext.iterate(result.files, function (fileInfo) {
				if (fileInfo.deleted) {
					this.store.removeAt(this.store.find('name', fileInfo.name));
					result.deletedDocuments.push(fileInfo);
				} else {
					result.failedDocuments.push(fileInfo);
				}
			}, this);
			this.fireEvent('remove', result);
		}, this);
	},

	getNameByUrl: function (url) {
		var id = this.store.find('file', url);
		return (id > -1) ? this.store.getAt(id).get('name') : url;
	},

	getUrlByName: function (name) {
		var id = this.store.find('name', name);
		return (id > -1) ? this.store.getAt(id).get('file') : name;
	}
});