/**
 * @class Sb.model.ImagegalleryWidget
 * @extends Sb.model.Widget
 * @constructor
 * @param {Object} config The configuration options for the widget model
 */
Ext.define('Sb.model.ImagegalleryWidget', {
	extend: 'Sb.model.Widget',

	type: 'imagegallery',

	translatableFields: ['widgetTitle'],

	constructor: function (config) {
		config = config || {};

		this.addEvents(
			/**
			 * @event albumsload
			 * Fires when albums loaded from store
			 * @param {Sb.model.ImagegalleryWidget} this
			 */
			'albumsload',
			/**
			 * @event albumchange
			 * Fires when albumId changed
			 * @param {Sb.model.ImagegalleryWidget} this
			 */
			'albumchange',
			/**
			 * @event imagesload
			 * Fires when images loaded from store
			 * @param {Sb.model.ImagegalleryWidget} this
			 */
			'imagesload'
		);

		/**
		 * Albums from current storage
		 * @type Array
		 * @property albums
		 */
		this.albums = [];

		/**
		 * Images from current album
		 * @type Array
		 * @property images
		 */
		this.images = [];

		/**
		 * Secure storage for auth info
		 * @type Ext.state.Provider
		 * @property secureStorage
		 */
		this.secureStorage = new Ext.state.CookieProvider({
			expires: null
		});

		Sb.model.ImagegalleryWidget.superclass.constructor.call(this, config);

		this.api = {
			save: Sb.ImageGalleryWidgetService.save
		};

		if (this.isLocal()) {
			this.loadImages(null, Ext.emptyFn);
		}
	},

	/**
	 * Shortcut to determine, when we in local storage
	 *
	 * @return {Boolean}
	 */
	isLocal: function () {
		return 'local' == this.get('storageType');
	},

	/**
	 * Shortcut to determine, when we in some remote storage
	 *
	 * @return {Boolean}
	 */
	isRemote: function () {
		return !this.isLocal();
	},

	/**
	 * Tells us when field is JSON-encoded
	 *
	 * @param fieldName
	 * @return {Boolean}
	 */
	isEncodedField: function (fieldName) {
		return ['storageAuthInfo', 'picasaDeletedImages'].indexOf(fieldName) > -1;
	},

	/**
	 * Returns the value of the given field
	 *
	 * @param {String} fieldName The field to fetch the value for
	 * @return {Mixed} The value
	 */
	get: function (fieldName) {
		var value = Sb.model.ImagegalleryWidget.superclass.get.call(this, fieldName);
		if ('storageAuthInfo' == fieldName) {
			value = '{}';
			if ('picasa' == this.get('storageType')) {
				value = this.secureStorage.get(this.get('uuid') + 'storageAuthInfo', value);
			} else if (this.isLocal()) {
				value = '{"siteUuid":"' + Sb.Editor.model.get('uuid') +  '"}';
			}
		}
		if (this.isEncodedField(fieldName)) {
			value = Ext.decode(value);
		}
		return value;
	},

	/**
	 * Sets the given field to the given value, marks the instance as dirty
	 *
	 * @param {String|Object} fieldName The field to set, or an object containing key/value pairs
	 * @param {Mixed} value The value to set
	 * @return {Boolean} Result of setting
	 */
	set: function (fieldName, value) {
		if (this.isEncodedField(fieldName)) {
			value = Ext.encode(value);
		}
		if ('storageAuthInfo' == fieldName && 'picasa' == this.get('storageType')) {
			this.secureStorage.set(this.get('uuid') + 'storageAuthInfo', value);
			return true;
		}
		return Sb.model.ImagegalleryWidget.superclass.set.call(this, fieldName, value);
	},

	/**
	 * Determines, wheteher we can change contents of active storage
	 *
	 * @return {Boolean}
	 */
	isChangeAllowed: function () {
		if ('picasa' == this.get('storageType') && (this.isNeedAuth() || !this.get('albumId'))) {
			return false;
		}

		return true;
	},

	/**
	 * Determines, wheteher we need authentification for active storage
	 *
	 * @return {Boolean}
	 */
	isNeedAuth: function () {
		if ('picasa' == this.get('storageType')) {
			return '{}' == Ext.encode(this.get('storageAuthInfo'));
		}

		return false;
	},

	/**
	 * Get current images:
	 *  - without images, that marked as deleted
	 *  - with full pathes in case of local storage
	 *
	 * @returns {Array}
	 */
	getImages: function () {
		var resultImages	= [],
			deletedImages	= this.getDeletedImages(),
			isDeleted		= (deletedImages.length > 0),
			isPrependUrl	= this.isLocal(),
			publishUrl		= Sb.Editor.model.get('publishWebSiteUrl');

		Ext.each(this.images, function (image) {
			var candidateImage = Ext.apply({}, image);
			if (isDeleted && deletedImages.indexOf(candidateImage.id) > -1) {
				return;
			}
			if (isPrependUrl) {
				candidateImage.url		= publishUrl + candidateImage.url;
				candidateImage.thumbUrl	= publishUrl + candidateImage.thumbUrl;
			}

			if (!Ext.isEmpty(candidateImage.weight) && resultImages[candidateImage.weight] === undefined) {
				resultImages[candidateImage.weight] = candidateImage;
			} else {
				resultImages.push(candidateImage);
			}
		});

		return Sb.Utils.arrayValues(resultImages);
	},

	/**
	 * Determines, wheteher we need actually clean up images from storage on delete.
	 * In false case, we don`t actually delete image, but add to deletedImages storage,
	 * that then used when we build up images array.
	 *
	 * @return {Boolean}
	 */
	isNeedClean: function () {
		if ('picasa' == this.get('storageType')) {
			return this.get('picasaIsNeedClean') ? true : false;
		}
		return true;
	},

	/**
	 * Add deleted image into special storage, that then used when we build up images array.
	 *
	 * @param imageId
	 */
	addDeletedImage: function (imageId) {
		if ('picasa' == this.get('storageType')) {
			var images = this.get('picasaDeletedImages');
			images.push(imageId);
			this.set('picasaDeletedImages', images);
		}
	},

	/**
	 * Get us deleted images storage
	 *
	 * @returns {Array}
	 */
	getDeletedImages: function () {
		if ('picasa' == this.get('storageType')) {
			return this.get('picasaDeletedImages');
		}
		return [];
	},

	/**
	 * Updates storage info and reloads it from server for update
	 *
	 * @param {Object} info Auth info object, for example {temporaryToken: 'qwerty'}
	 * @param {Function} callback Callback, that will be launched on successful authinfo update
	 */
	processStorageAuthInfo: function (info, callback) {
		var uuid = this.get('uuid'),
			type = this.get('storageType');

		info = Ext.encode(info);

		Sb.ImageGalleryWidgetService.processStorageAuthInfo(uuid, type, info, function (result, response) {
			if (!response.status) {
				return;
			}
			if (result.success) {
				this.set('storageAuthInfo', result.storageAuthInfo);
				if ('picasa' == type) {
					this.set('picasaUserDataUrl', result.properties.picasaUserDataUrl);
				}
			}
			Ext.callback(callback, null, [result]);
		}, this);
	},

	/**
	 * Get data of currently selected album
	 *
	 * @return Object
	 */
	getCurrentAlbum: function () {
		var id				= this.get('albumId'),
			currentAlbum	= {};

		Ext.each(this.albums, function (album) {
			if (id == album.id) {
				currentAlbum = album;
				return false;
			}
		});

		return currentAlbum;
	},

	/**
	 * Creates album in storage
	 *
	 * @param {String} title
	 * @param {Function} callback Callback, that will be launched on success with albumId as argument
	 * @param {Object} scope Scope for callback
	 */
	createAlbum: function (title, callback, scope) {
		var uuid = this.get('uuid'),
			type = this.get('storageType'),
			info = this.get('storageAuthInfo');

		Sb.ImageGalleryWidgetService.createAlbum(uuid, type, info, title, function (albumId, response) {
			if (response.status) {
				Ext.callback(callback, scope, [albumId]);
			}
		}, this);
	},

	/**
	 * Refresh albums by loading them or usign cache and executing callback
	 *
	 * @param {Function} callback Callback, that will be launched on refresh
	 * @param {Object} scope Scope for callback
	 */
	refreshAlbums: function (callback, scope) {
		if (this.albums.length > 0) {
			Ext.callback(callback, scope);
			return;
		}
		this.loadAlbums(callback, scope);
	},

	/**
	 * Loads albums from store
	 *
	 * @see http://code.google.com/apis/picasaweb/docs/2.0/developers_guide_protocol.html
	 *
	 * @param {Function} callback Callback, that will be launched on load
	 * @param {Object} scope Scope for callback
	 */
	loadAlbums: function (callback, scope) {
		var finish = function (result) {
				this.albums = result;
				Ext.callback(callback, scope);
				this.fireEvent('albumsload');
			}.createDelegate(this);

		if (this.isLocal()) {
			finish([]);
			return;
		}

		var uuid = this.get('uuid'),
			type = this.get('storageType'),
			info = this.get('storageAuthInfo');

		Sb.ImageGalleryWidgetService.loadAlbums(uuid, type, info, function (result, response) {
			if (response.status) {
				finish(result);
			}
		}, this);
	},

	/**
	 * Refresh images by loading them or using cache and executing callback
	 *
	 * @param {String} albumId albumId from which to refresh
	 * @param {Function} onSuccess Callback, that will be launched on refresh
	 * @param {Function} onError Callback, that will be launched on error
	 * @param {Object} scope Scope for callback
	 */
	refreshImages: function (albumId, onSuccess, onError, scope) {
		if (this.images.length > 0) {
			Ext.callback(onSuccess, scope);
			return;
		}
		this.loadImages(albumId, onSuccess, onError, scope);
	},

	/**
	 * Loads images from store
	 *
	 * @see http://code.google.com/apis/picasaweb/docs/2.0/developers_guide_protocol.html
	 *
	 * @param {String} albumId albumId from which to load
	 * @param {Function} onSuccess Callback, that will be launched on load
	 * @param {Function} [onError] Callback, that will be launched on error
	 * @param {Object} [scope] Scope for callback
	 */
	loadImages: function (albumId, onSuccess, onError, scope) {
		if (this.isLocal()) {
			this.images = this.get('localImages').slice(0);
			Ext.callback(onSuccess, scope);
			this.fireEvent('imagesload');
			return;
		}
		if (!albumId) {
			this.images = [];
			Ext.callback(onSuccess, scope);
			return;
		}

		var uuid = this.get('uuid'),
			type = this.get('storageType'),
			info = this.get('storageAuthInfo');

		if ('picasa' == type && this.isNeedAuth()) {
			Ext.ux.jsonp.Manager.request({
				url			: this.get('picasaAlbumDataUrl'),
				params		: {
					fields	: 'entry(gphoto:id,content,title,summary)',
					v		: '2.0',
					kind	: 'photo',
					alt		: 'json'
				},
				success : function (data) {
					var feed, entry, i, iMax;

					feed = data.feed || {};
					entry = feed.entry || [];
					iMax = entry.length;
					this.images = [];
					for (i = 0; i < iMax; i++) {
						var image = entry[i];
						this.images.push({
							id			: image.gphoto$id.$t,
							url			: image.content.src,
							thumbUrl	: image.content.src + '?imgmax=150',
							title		: image.title	? image.title.$t	: '',
							description	: image.summary	? image.summary.$t	: ''
						});
					}
					Ext.callback(onSuccess, scope);
					this.fireEvent('imagesload');
				},
				failure		: function () {
					Ext.callback(onError, scope, [{messageLocaleKey: 'widgetImagegalleryErrorGetImages'}]);
				},
				scope	: this
			});
			return;
		}

		Sb.ImageGalleryWidgetService.loadImages(uuid, type, info, albumId, function (result, response) {
			if (response.status) {
				if (!result.success) {
					Ext.callback(onError, scope, [{messageLocaleKey: 'widgetImagegalleryErrorGetImages'}]);
					return;
				}
				this.images = result.images;
				Ext.callback(onSuccess, scope);
				this.fireEvent('imagesload');
			}
		}, this);
	},

	/**
	 * Gets image`s index in images collection by id
	 *
	 * @param {String} imageId
	 */
	getIndexById: function (imageId) {
		var imageIndex = null;
		Ext.each(this.images, function (image, index) {
			if (image.id == imageId) {
				imageIndex = index;
				return false;
			}
		});
		return imageIndex;
	},

	/**
	 * Get image by id
	 *
	 * @param {String} id
	 * @return {Object} image
	 */
	getImage: function (id) {
		var resultImage = null;

		Ext.each(this.images, function (image) {
			if (image.id == id) {
				resultImage = image;
				return false;
			}
		});

		return resultImage;
	},

	updateImageOrder: function (imageId, relativeId, insertType) {
		var newIndex = this.getIndexById(relativeId),
			updatedImages = [];

		newIndex = (insertType == 'after') ? newIndex + 1 : newIndex;

		Ext.each(this.images, function (image, index) {
			if (image.id == imageId) {
				return true;
			}

			image = Ext.apply({}, image);

			if (index < newIndex) {
				updatedImages[index] = image;
			} else {
				updatedImages[index + 1] = image;
			}
		});
		updatedImages[newIndex] = this.getImage(imageId);

		this.images = Sb.Utils.arrayValues(updatedImages);

		Ext.each(this.images, function (image, index) {
			this.updateImageProperty(image.id, 'weight', index);
		}, this);

		this.save();
	},

	/**
	 * Remove image from store
	 *
	 * @param {String} imageId Image id to remove
	 */
	removeImage: function (imageId) {
		var uuid		= this.get('uuid'),
			type		= this.get('storageType'),
			info		= this.get('storageAuthInfo'),
			imageIndex	= this.getIndexById(imageId);

		if (null !== imageIndex) {
			if (!this.isNeedClean()) {
				this.addDeletedImage(imageId);
				return;
			}
			if (this.isRemote()) {
				Sb.ImageGalleryWidgetService.removeImage(uuid, type, info, this.get('albumId'), imageId);
			}
			this.images.splice(imageIndex, 1);
			if (this.isLocal()) {
				this.set('localImages', this.images);
			}
		}
	},

	/**
	 * Adds single image in images collection
	 *
	 * @param {Object} image
	 */
	addImage: function (image) {
		image.weight = this.images.length;
		this.images.push(image);
		if (this.isLocal()) {
			this.set('localImages', this.images);
		}
	},

	/**
	 * Updates image property in store
	 *
	 * @param {String} imageId
	 * @param {String} property
	 * @param {String} value
	 * @return false if property is not updated
	 */
	updateImageProperty: function (imageId, property, value) {
		var uuid		= this.get('uuid'),
			type		= this.get('storageType'),
			info		= this.get('storageAuthInfo'),
			imageIndex	= this.getIndexById(imageId);

		if (imageIndex !== null && this.images[imageIndex][property] != value) {
			Sb.ImageGalleryWidgetService.updateImageProperty(
				uuid, type, info, this.get('albumId'), imageId, property, value
			);
			this.images[imageIndex][property] = value;
			if (this.isLocal()) {
				this.set('localImages', this.images);
			}
			return true;
		}
		return false;
	},

	/**
	 * Restore node`s state from memento: cleanup transactions, that was started before cancel
	 *
	 * @param {Object} memento
	 */
	restoreFromMemento: function (memento) {
		Ext.iterate(Ext.Direct.transactions, function (i, t) {
			if ('ImageGalleryWidgetService' == t.action) {
				Ext.Direct.removeTransaction(t);
			}
		});
		this.images = [];
		Sb.model.ImagegalleryWidget.superclass.restoreFromMemento.call(this, memento);
	},

	/**
	 * Internal function, that fires before external albumchange
	 */
	onAlbumChange: function () {
		if ('picasa' == this.get('storageType') && this.albums.length > 0) {
			this.set('picasaAlbumDataUrl', this.getCurrentAlbum().dataUrl.replace(/^https?:/i, ''));
		}
	},

	unserialize: function (data) {
		var result = Sb.model.ImagegalleryWidget.superclass.unserialize.call(this, data);
		if (result) {
			this.updateIsConfiguredProperties();
		}
		return result;
	},

	/**
	 * Fires on changing of single property change to allow us fire up widget change events.
	 * @param {String} key
	 */
	onUnserializePropertyChange: function (key) {
		if ('albumId' == key) {
			this.onAlbumChange();
			this.fireEvent('albumchange', this);
		}
	},

	computeIsConfiguredProperties: function () {
		if (this.get('storageType') === 'picasa') {
			return !!this.get('albumId');
		}
		return this.get('localImages') && this.get('localImages').length > 0;
	},

	normalizeData: function (data) {
		var locale = Sb.Editor.model.get('locale'),
			oldLocale = this.localeCode || locale;
		data = this.callParent([data]);

		Ext.each(data.localImages, function (image, index) {
			data.localImages[index] = this.switchTranslations(
				image, locale, oldLocale, ['title', 'description']
			);
		}, this);

		return data;
	}
});