/**
 * @class Sb.model.Site
 * @extends Sb.model.Node
 * Class for site model
 * @constructor
 * @param {Object} config The configuration options for the site model
 */
Ext.define('Sb.model.Site', {
	extend: 'Sb.model.Node',

	translatableFields: [
		'companyName',
		'description',
		'keywords'
	],

	constructor: function (config) {
		config = Ext.apply({
			associations: [
				/**
				 * The model node of site root page
				 * @type Sb.model.SitePage
				 * @property rootPage
				 */
				{type: 'hasOne', model: 'SitePage', name: 'rootPage'},
				/**
				 * The model node of site design
				 * @type Sb.model.SiteDesign
				 * @property design
				 */
				{type: 'hasOne', model: 'SiteDesign', name: 'design'}
			]
		}, config || {});

		this.addEvents(
			/**
			 * @event beforeload
			 * Fires after node has been successfuly loaded and before serialized.
			 * @param {Sb.model.Node} this This node.
			 */
			'beforeload'
		);

		this.api = {
			load: Sb.SiteService.getSettings,
			save: Sb.SiteService.setSettings
		};

		/**
		 * Images, that is used in site and uploaded by user
		 *
		 * @type Ext.util.MixedCollection
		 * @property usedImages
		 */
		this.uploadedImages = new Ext.util.MixedCollection();

		Sb.model.Site.superclass.constructor.call(this, config);
	},

	/**
	 * Load callback
	 *
	 * @param {Object} result
	 * @param {Ext.Direct.Event} event
	 */
	loadCallback: function (result, event) {
		this.isLoading = false;
		if (!event.status) {
			return;
		}

		Sb.WidgetTypeManager.init();
		this.isLoaded = true;
		this.fireEvent('beforeload', this);
		(function () {
			this.unserialize(result);
			if (this.localeCode !== Sb.Editor.siteView.getCurrentSiteLocale()) {
				Sb.Editor.model.rootPage.applySelfAndRecursive(function (page) {
					page.changeLocale(Sb.Editor.siteView.getCurrentSiteLocale());
				});
			}
			this.fireEvent('load', this);
		}).defer(10, this);
	},

	unserialize: function (data) {
		var isChanged,
			rootPage = data.rootPage,
			widgetList = data.widgetList;

		delete data.rootPage;
		delete data.widgetList;
		isChanged = Sb.model.Site.superclass.unserialize.call(this, data);

		if (Ext.isObject(rootPage)) {
			rootPage.rootData = { widgetList: widgetList };
			isChanged = this.rootPage.unserialize(rootPage) || isChanged;
		}
		return isChanged;
	},

	serialize: function () {
		var data = Sb.model.Site.superclass.serialize.call(this, {});
		data.rootPage = this.rootPage.serialize(data);
		return data;
	},

	saveToMemento: function () {
		return this.normalizeData(this.getSettings());
	},

	/**
	 * Gets the site url.
	 * @return {String}
	 */
	getUrl: function () {
		return Sb.Config.sitesUrl + '/' + this.data.dirName.substr(0, 2) + '/' + this.data.dirName;
	},

	/**
	 * Gets start part of page`s URL, that will be like in published site
	 *
	 * @return {String} Published site url as of http://host/ or for undefined cases just http://site.com/
	 */
	getPublishUrl: function () {
		return this.data.publishWebSiteUrl || 'http://site.com/';
	},

	/**
	 * Gets URL that will be on published site as not found
	 *
	 * @return {String} Url as of http://host/404.html
	 */
	getNotFoundUrl: function () {
		return this.getPublishUrl() + Sb.Config.notFoundPageName;
	},

	/**
	 * Save model.
	 *
	 * @param {Object} [callbacks] Object with success and failure callbacks.
	 */
	save: function (callbacks) {
		this.api.save(this.getSettings(), this.saveCallback.createDelegate(this, [callbacks], true));
	},

	getSettings: function () {
		return Ext.clone({
			companyName: this.data.companyName,
			pagesUseName: this.data.pagesUseName,
			description: this.data.description,
			keywords: this.data.keywords,
			customHead: this.data.customHead,
			googleAnalyticsCode: this.data.googleAnalyticsCode,
			cookieBlockerCode: this.data.cookieBlockerCode,
			robotsTxt: this.data.robotsTxt,
			isFacebookIntegration: this.data.isFacebookIntegration,
			locale: this.data.locale,
			locales: this.data.locales,
			disqusId: this.data.disqusId,
			faviconFile: this.data.faviconFile,
			logoName: this.data.logoName,
			logoPath: this.data.logoPath,
			cookiePolicyWidgetEnabled: this.data.cookiePolicyWidgetEnabled,
			mobileOptimizationsEnabled: this.data.mobileOptimizationsEnabled,
			ownerInfo: this.data.ownerInfo,
			translations: this.data.translations
		});
	},

	/**
	 * Updates all used uploaded images in site
	 *
	 * @return {Ext.util.MixedCollection} updated images collection
	 */
	updateUploadedImages: function () {
		var reIsAttachmentImage		= new RegExp(
			'(?:(?:/templates/' + Sb.Editor.model.data.template.code + '/images/content_icons/[a-z_]+/)' +
			'|(?:' + Sb.Config.path.attachmentsFolder + '/' + Sb.Config.path.imageFolder + '/))' +
			'([^/]+\\.(' + Sb.Config.imageValidExtensions.concat(Sb.Config.flashValidExtensions).join('|') + '))"?', 'i'
		);
		var reIsAttachmentImages	= new RegExp(reIsAttachmentImage.source, 'ig');
		var site					= this;
		site.uploadedImages.clear();
		site.rootPage.applySelfAndRecursive(function (page) {
			page.widget.widgets.each(function (verticalContainer) {
				var textWidgets = page.widget.queryBy(function (widget) {
					return widget instanceof Sb.model.TextWidget;
				});
				Ext.each(textWidgets, function (widget) {
					if (!widget.data.content && !widget.data.translations) {
						return;
					}
					var allLocalesData = [widget.data.content];
					Ext.iterate(widget.data.translations, function (key, translation) {
						allLocalesData.push(translation.content || '');
					});
					Ext.each(allLocalesData, function (content) {
						var matches = content.match(reIsAttachmentImages);
						if (null != matches) {
							Ext.each(matches, function (usedImage) {
								try {
									usedImage = decodeURIComponent(usedImage.match(reIsAttachmentImage)[1]);
									if (!site.uploadedImages.containsKey(usedImage)) {
										site.uploadedImages.add(usedImage, 0);
									}
									site.uploadedImages.add(usedImage, site.uploadedImages.get(usedImage) + 1);
								} catch (ex) {
									// Ignore URIError: malformed URI sequence
								}
							});
						}
					});
				});
			});
		});
		return site.uploadedImages;
	},

	/**
	 * Get uploaded images src with anti-fool protect of ['empty']
	 *
	 * @return {Array}
	 */
	getUploadedImagesSources: function () {
		var files = Sb.Editor.model.uploadedImages.keys;
		if (0 === files.length) {
			files = ['empty'];
		}
		return files;
	},

	/**
	 * Checks if current site locale is RTL.
	 * @return {Boolean}
	 */
	isRtl: function () {
		return Sb.Config.locales[Sb.Editor.siteView.getCurrentSiteLocale()].isRtl;
	}
});
