/**
 * @class Sb.widget.Blog
 * @extends Sb.widget.Widget
 * Class for Blog Widget view
 * @xtype blog_widget
 */
Ext.define('Sb.widget.Blog', {
	extend: 'Sb.widget.Widget',
	xtype: 'blog_widget',

	cls			: 'sb-widget-blog',

	/**
	 * Current showed page
	 * @property currentPage
	 * @type Number
	 */
	currentPage: 1,

	afterRender: function () {
		Sb.widget.Blog.superclass.afterRender.call(this);
		this.renderedItems = [];
		this.refresh();
	},

	initEvents: function () {
		Sb.widget.Blog.superclass.initEvents.call(this);

		this.mon(Sb.Editor.model.rootPage, 'change', this.refresh, this);
		this.mon(this.model, 'propertychange', function (key, value, oldValue) {
			if (key == 'pageSize') {
				this.setCurrentPage(1);
			}
		}, this);
	},

	refresh: function () {
		Sb.Editor.siteView.container.scroll('top', this.getEl().getHeight());
		this.items = Sb.Editor.getController('Blogs').getPosts(this.model);

		var total = this.items.getCount(),
			pageSize = this.model.get('pageSize'),
			pages = total < pageSize ? 1 : Math.ceil(total / pageSize),
			start = (this.currentPage - 1) * pageSize,
			end = Math.min(start + pageSize - 1, total),
			items = this.items.getRange(start, end);

		Ext.each(this.renderedItems, function (item) {
			item.destroy();
		});
		this.update('');

		this.widgetContentEl.createChild({
			tag: 'a',
			href: '#',
			cls: 'sb-blog-add-new-post',
			html: Sb.Locale.get('widgetBlogAddNewPostText')
		}).on('click', function () {
			Sb.Editor.getController('Blogs').createPost(this.model, function (post) {
				post.ownerPage.parentPage.save();
				Sb.Editor.siteView.setCurrentPage(post.ownerPage, function () {
					Ext.getCmp('sb-widget-' + post.get('uuid')).startEdit();
				});
			});
		}, this, {stopEvent: true, single: true});

		Ext.each(items, function (post) {
			var item = new Sb.widget.BlogPost({
				model		: post,
				renderTo	: this.widgetContentEl,
				renderMode	: 'teaser'
			});
			this.renderedItems.push(item);
		}, this);

		if (this.currentPage > 1 || this.currentPage < pages) {
			var linksConfig = [],
				linksEl,
				hasPreviousBtn = false,
				hasNextBtn = false;

			if (this.currentPage < pages) {
				hasPreviousBtn = true;
				linksConfig.push({
					tag: 'td',
					cn: {
						tag: 'a',
						cls: 'blog-button-previous',
						href: '#',
						html: Sb.Locale.get('widgetBlogPreviousPageText')
					}
				});
			}
			if (this.currentPage > 1) {
				if (this.currentPage < pages) {
					linksConfig.push({tag: 'td', html: ' | '});
				}

				hasNextBtn = true;
				linksConfig.push({
					tag: 'td',
					cn: {
						tag: 'a',
						cls: 'blog-button-next',
						href: '#',
						html: Sb.Locale.get('widgetBlogNextPageText')
					}
				});
			}

			linksEl = this.widgetContentEl.createChild({
				cls: 'widget-blog-links',
				tag: 'table',
				cn: {
					tag: 'tr',
					cn: linksConfig
				}
			});

			if (hasPreviousBtn) {
				linksEl.child('a.blog-button-previous')
					.on('click', this.previousPage, this, {stopEvent: true});
			}
			if (hasNextBtn) {
				linksEl.child('a.blog-button-next')
					.on('click', this.nextPage, this, {stopEvent: true});
			}
		}

		if (Sb.Editor.Ui.editComponent === this) {
			this.getEditPanel().reanchor();
		}
	},

	removeComponentHandler: function () {
		this.items.each(function (post) {
			this.model.ownerPage.remove(post.ownerPage);
		}, this);

		this.model.ownerPage.save();

		Sb.widget.Blog.superclass.removeComponentHandler.call(this);
	},

	onContainerAdd: function () {
		Sb.WidgetTypeManager.toggleButtons();
	},

	onDestroy: function () {
		Ext.each(this.renderedItems, function (item) {
			item.destroy();
		});
		Sb.widget.Blog.superclass.onDestroy.call(this);
	},

	/**
	 * Change current showed page
	 * @param {Number} page The number of the page to show
	 */
	setCurrentPage: function (page) {
		var total = this.items.getCount(),
			pageSize = this.model.get('pageSize'),
			pages = total < pageSize ? 1 : Math.ceil(total / pageSize);

		page = page < 1 ? 1 : (page > pages ? pages : page);

		this.currentPage = page;
		this.refresh();
	},

	/**
	 * Show the next 'page'
	 */
	nextPage: function () {
		this.setCurrentPage(this.currentPage - 1);
	},

	/**
	 * Show the previous 'page'
	 */
	previousPage: function () {
		this.setCurrentPage(this.currentPage + 1);
	}
});
