/**
 * @class Sb.widget.Container
 * @extends Ext.Container
 * @constructor
 * @param {Object} config The config object
 */
Ext.define('Sb.widget.Container', {
	extend: 'Ext.Container',

	/**
	 * @cfg {Sb.model.Widget} model
	 * The {@link Sb.model.Widget} to bind this Widget to.
	 */

	droppable: true,

	/**
	 * Side paddings that define areas where dropped widget should create a column.
	 */
	sidePadding: 0.3,

	initComponent: function () {
		this.items = [];
		this.model.widgets.each(function (widget) {
			this.items.push(widget);
		}, this);
		Sb.widget.Container.superclass.initComponent.call(this);
	},

	lookupComponent: function (comp) {
		var xtype = (comp.hidden ? 'hidden' : comp.get('type')) + '_widget';
		if (Ext.ComponentMgr.isRegistered(xtype)) {
			comp = Ext.create({
				xtype: xtype,
				model: comp
			});
		} else {
			comp = new Sb.widget.Widget({
				model: comp
			});
		}

		return comp;
	},

	onBeforeAdd: function (item) {
		if (!item.clickable && !item.overable && !item.model.isConfiguredProperties()) {
			// skip not editable empty widgets
			return false;
		}
		return Sb.widget.Container.superclass.onBeforeAdd.apply(this, arguments);
	},

	afterRender: function () {
		this.dt = new Ext.dd.DropTarget(this.el.id, {
			copy	: false,
			ddGroup	: 'widgets',

			notifyDrop: function (source, e, data) {
				if (!this.isDroppable(source, e)) {
					return false;
				}

				if (source.dragData.widget && source.dragData.widget.model.ownerWidget == this.model
					&& source.dragData.position == this.model.widgets.indexOf(source.dragData.widget.model)) {
					this.removeDropSpacer();
					return false;
				}

				this.removeDropSpacer();
				this.onDrop(source);
				return true;
			}.createDelegate(this),

			notifyOver: function (source, e, data) {
				if (source.dragData.container && source.dragData.container != this) {
					source.dragData.container.removeDropSpacer();
				}

				source.dragData.container = this;

				if (!this.isDroppable(source, e)) {
					this.removeDropSpacer();
					return Ext.dd.DropZone.prototype.dropNotAllowed;
				}
				this.onNotifyOver(source, e.getXY());
				return Ext.dd.DropZone.prototype.dropAllowed;
			}.createDelegate(this),

			notifyOut: function (source, e, data) {
				this.removeDropSpacer();
				delete source.dragData.container;
				this.onNotifyOut(source);
			}.createDelegate(this)
		});
		this.dt.addToGroup('widgetButtons');

		this.mon(this.model.widgets, {
			scope: this,
			add: this.onModelWidgetAdd,
			remove: this.onModelWidgetRemove
		});

		Sb.widget.Container.superclass.afterRender.call(this);
	},

	// :TODO: There is speculation that this method was not necessary
	onModelWidgetAdd: function (index, widget, key) {
		widget = this.insert(index, widget);
		this.doLayout();

		if (widget.onContainerAdd) {
			widget.onContainerAdd();
		}
	},

	onModelWidgetRemove: function (widget) {
		this.items.each(function (item) {
			if (item.model === widget) {
				item.destroy();
			}
		});
	},

	/**
	 * Checks if we can drop to target/container
	 *
	 * @param {Ext.dd.DragSource} source
	 * @param {Event} e
	 * @return {Boolean}
	 */
	isDroppable: function (source, e) {
		var validTarget = e ? !e.getTarget('.sb-editor-panel') : true,
			isCommonParent = false,
			ownerCt;

		if (source.dragData.widget) {
			ownerCt = source.dragData.widget.ownerCt;
			isCommonParent = ownerCt === this || (ownerCt.parentWidget && ownerCt.parentWidget.ownerCt === this);
		}

		return validTarget && this.droppable
			&& (source.dragData.widgetType.name !== 'pagecontent' || isCommonParent)
			&& source.dragData.widgetType.isDroppableTo(this.model);
	},

	/**
	 * The function calls continuously while source is being dragged over the container.
	 * @param {Ext.dd.DragSource} source
	 * @param {Array} draggedPosition The xy values like [x, y]
	 */
	onNotifyOver: function (source, draggedPosition) {
		var isDdWidgetFromThisContainer = false,
			widgetsCount = this.items.getCount(),
			insertPlace = null,
			insertPosition = null,
			currentWidget = null,
			widgetHeight = 0,
			widgetWidth = 0,
			widgetPosition = [],
			widgetTop,
			widgetBottom,
			widget = null,
			isAboveFirst,
			isUnderLast,
			isInside,
			index;

		// run through all widgets in container to find over what we dragging now (currentWidget)
		for (index = 0; index < widgetsCount; index++) {
			widget =  this.items.get(index);

			if (source.dragData.widget && source.dragData.widget == widget) {
				isDdWidgetFromThisContainer = true;
				continue;
			}

			widgetHeight = widget.el.getHeight();
			widgetWidth = this.el.getWidth();
			widgetPosition = [this.el.getX(), widget.el.getY()];
			widgetTop = widgetPosition[1];
			widgetBottom = widgetPosition[1] + widgetHeight;

			isAboveFirst = index === 0 && draggedPosition[1] <= widgetBottom;
			isUnderLast = index == (widgetsCount - 1) && draggedPosition[1] >= widgetTop;
			isInside = draggedPosition[1] >= widgetTop && draggedPosition[1] <= widgetBottom;

			if (isAboveFirst || isUnderLast || isInside) {
				currentWidget = widget;
				insertPosition = index;
				break;
			}
		}

		if (insertPosition == null) {
			if (!widgetsCount || (widgetsCount == 1 && isDdWidgetFromThisContainer)) {
				insertPosition = 0;
			} else { //if drag to spacer
				return;
			}
		}

		if (currentWidget) {
			if (draggedPosition[1] <= (widgetTop + (widgetBottom - widgetTop) / 2)) {
				insertPlace = 'top';
			} else {
				insertPlace = 'bottom';
			}

			if (this.allowAddWidgetSide(currentWidget.getType()) &&
				this.allowAddWidgetSide(source.dragData.widgetType)
			) {
				if (draggedPosition[0] > widgetPosition[0] + widgetWidth * (1 - this.sidePadding)) {
					insertPlace = 'right';
				} else if (draggedPosition[0] < widgetPosition[0] + widgetWidth * this.sidePadding) {
					insertPlace = 'left';
				}
			}

			if (insertPlace == 'bottom' && !isDdWidgetFromThisContainer) {
				insertPosition++;
			}

			if (isDdWidgetFromThisContainer && insertPlace == 'top' && insertPosition > this.items.indexOf(source.dragData.widget)) {
				insertPosition--;
			}
		} else {
			insertPlace = 'top';
		}

		if (Sb.Editor.model.isRtl()) {
			insertPlace = Sb.Utils.swapLeftAndRight(insertPlace);
		}

		source.dragData = Ext.apply(source.dragData, {
			position: insertPosition,
			place: insertPlace,
			widgetUnderDrag: currentWidget
		});

		this.renderDropSpacer(insertPlace, currentWidget);
	},

	/**
	 * The function calls when source is move out the container.
	 * @param {Ext.dd.DragSource} dragSource
	 */
	onNotifyOut: Ext.emptyFn,

	/**
	 * Ability to drop dragged widget in side relative widget under Drag
	 * @param {Sb.widgetType.Base} widgetType
	 * @return {Boolean}
	 */
	allowAddWidgetSide: function (widgetType) {
		return ['column1', 'column2'].indexOf(this.model.get('region')) == -1
				&& widgetType.isDroppableToColumn();
	},

	getDdSpacerHtml: function () {
		return '<div class="sb-widget-dd-spacer sb-widget-dd-spacer-' + (this.model.isInSiteWideArea() ? 'sitewide' : 'pagewide') + '">' +
			Sb.Locale.get(this.model.isInSiteWideArea() ? 'siteWideArea' : 'pageContentArea') + '</div>';
	},

	/**
	 * Render Drop spacer for insertion in container relative defined widget
	 * @param {String} place avaliable values top|right|bottom|left
	 * @param {Sb.widget.Widget} widget (optional) relative that ddSpacer should be rendered
	 */
	renderDropSpacer: function (place, widget) {
		var contextElement = widget ? widget.el.dom : this.getLayoutTarget().dom,
			ddWidget = Ext.dd.DragDropMgr.dragCurrent.dragData.widget,
			insertPlace, node, spacerClass, prevEl, nextEl, nextIsSearch, nextIsSelf;
		this.removeDropSpacer();

		switch (place) {
			case 'top':
				insertPlace = widget ? 'beforeBegin' : 'afterBegin';
				break;
			case 'bottom':
				insertPlace = 'afterEnd';
				break;
			default:
				insertPlace = place == 'left' ? 'beforeBegin' : 'afterEnd';
				break;
		}
		node = Ext.DomHelper.insertHtml(insertPlace, contextElement, this.getDdSpacerHtml());
		if (['left', 'right'].indexOf(place) != -1) {
			var cls = 'sb-widget-insertion-column';
			Ext.fly(node).addClass(cls);
			widget.addClass(cls);
		}
		Sb.widget.Container.nodes.push(node);

		// support manager interface for node
		node.on = node.un = node.setActive = Ext.emptyFn;
		node.setZIndex = function (zindex) {
			Ext.fly(node).setStyle('z-index', zindex);
		};

		Sb.Editor.windowManager.register(node, 1);
		node.manager.bringToFront(node);
		if (ddWidget) {
			ddWidget.toFront();
		}

		spacerClass = 'sb-highlight-spacer-' + (this.model.isInSiteWideArea() ? 'sitewide' : 'pagewide');
		prevEl = Ext.get(node).prev();
		nextEl = Ext.get(node).next();

		if (nextEl) {
			nextIsSearch = nextEl.hasClass('gsc-resultsbox-invisible');
			nextIsSelf =  ddWidget && (ddWidget.id == nextEl.id);
			if (nextIsSearch || nextIsSelf) {
				nextEl = nextEl.next();
			}
		}

		if (prevEl && prevEl.hasClass(spacerClass) && ['top', 'bottom'].indexOf(place) != -1) {
			prevEl.setStyle('display', 'none');
		}

		if (nextEl && nextEl.hasClass(spacerClass) && ['top', 'bottom'].indexOf(place) != -1) {
			nextEl.setStyle('display', 'none');
		}

		this.refreshCache();
	},

	/**
	 * Remove drop spacer
	 */
	removeDropSpacer: function () {
		Ext.select('.sb-highlight-spacer-sitewide, .sb-highlight-spacer-pagewide').setStyle('display', 'block');

		if (Sb.widget.Container.nodes.length) {
			Ext.each(Sb.widget.Container.nodes, function (node) {
				if (node.manager) {
					node.manager.unregister(node);
				}
				Ext.removeNode(node);
			});
			Sb.widget.Container.nodes = [];
		}
		this.items.each(function (widget) {
			widget.removeClass('sb-widget-insertion-column');
		}, this);
		this.refreshCache();
	},

	/**
	 * Handler that one of child widget start drag
	 * @param {Ext.dd.DragSource} dragSource
	 */
	onStartDragChildWidget: Ext.emptyFn,

	/**
	 * Handler that one of drag the child widget was dropped in invalid target
	 * @param {Ext.dd.DragSource} dragSource
	 */
	onWidgetInvalidDrop: Ext.emptyFn,

	/**
	 * Handler that one of drag the child widget are dropped in some target
	 * @param {Ext.dd.DragSource} dragSource
	 */
	onEndDragChildWidget: function (dragSource) {
		this.removeDropSpacer();
	},

	/**
	 * Calls once when dragged item has been dropped on container
	 * @param {Ext.dd.DragSource} dragSource
	 */
	onDrop: function (dragSource) {
		if (['left', 'right'].indexOf(dragSource.dragData.place) != -1) {
			this.onDropSide(dragSource);
			return;
		}

		if (dragSource.dragData.widget) {
			this.model.insert(dragSource.dragData.position, dragSource.dragData.widget.model);
		} else {
			this.model.createChild(dragSource.dragData.widgetType, dragSource.dragData.position, this.checkWidgetLimits);
		}
	},

	/**
	 * Calls once when dragged item has been dropped in side relative widget in containers
	 * @param {Ext.dd.DragSource} dragSource
	 */
	onDropSide: function (dragSource) {
		this.model.createChild(Sb.WidgetTypeManager.get('columns'), dragSource.dragData.position, function (columnsModel) {

			var dropColumn, // model of the column in which we drop widget
				moveColumn; // model of the column in which the widget under the cursor moves
			if (dragSource.dragData.place == 'left') {
				dropColumn = columnsModel.widgets.items[0];
				moveColumn = columnsModel.widgets.items[1];
			} else {
				dropColumn = columnsModel.widgets.items[1];
				moveColumn = columnsModel.widgets.items[0];
			}

			if (dragSource.dragData.widget) {
				dropColumn.insert(0, dragSource.dragData.widget.model);
			} else {
				dropColumn.createChild(dragSource.dragData.widgetType, 0, this.checkWidgetLimits);
			}
			moveColumn.insert(0, dragSource.dragData.widgetUnderDrag.model);
		}, this);
	},

	/**
	 * Check created widget on limits
	 *
	 * @param {Sb.model.Widget} widget
	 */
	checkWidgetLimits: function (widget) {
		if (Sb.Limits.checkWidgets(Sb.Editor.model, widget.get('type')).isExceeded) {
			Sb.Editor.Ui.mainWindow.statusBar.setMessageWarning(
				Sb.Locale.get(
					'moduleLimitExceeded',
					widget.getType().getTranslation('title')
				)
			);
		}
	},

	/**
	 * Clean up location cache
	 */
	refreshCache: function () {
		Ext.dd.DragDropMgr.refreshCache({widgetButtons: true});
	},

	onDestroy: function () {
		if (this.dt) {
			this.dt.destroy();
		}
		Sb.widget.Container.superclass.onDestroy.call(this);
	},

	doResize: function () {
		this.items.each(function(item){
			item.doResize();
		}, this);
	}
});

Sb.widget.Container.nodes = [];