/**
 * @class Sb.widget.Navigation
 * @extends Sb.widget.Widget
 * Class for Navigation widget
 * @constructor
 * @param {Object} config The config object
 * @xtype navigation_widget
 */
Ext.define('Sb.widget.Navigation', {
	extend: 'Sb.widget.Widget',
	xtype: 'navigation_widget',

	cls: 'sb-widget-navigation',

	/**
	 * @cfg {Sb.model.NavigationWidget} model
	 * The {@link Sb.model.NavigationWidget} to bind this Widget to.
	 */

	initComponent: function () {
		this.menu = [{
			text: Sb.Locale.get('cmPages'),
			iconCls: 'sb-pages-icon',
			scope: this,
			handler: function () {
				this.completeEdit();
				Sb.Editor.Ui.mainWindow.tabs.setActiveTab('pages');
			}
		}, {
			text: Sb.Locale.get('cmWidgetNavigationItems'),
			iconCls: 'sb-menu-items-style-icon',
			scope: this,
			handler: function () {
				this.startEdit();
				if (!this.getShowEditPanelState()) {
					this.propertiesClickHandler();
				}
				this.getEditPanel().tabs.setActiveTab(1); // Items panel,
			}
		}];

		Sb.widget.Navigation.superclass.initComponent.call(this);
	},

	afterRender: function () {
		Sb.widget.Navigation.superclass.afterRender.call(this);
		this.refresh();
		this.refreshCss();
		this.refreshColorSchemeCss();
	},

	initEvents: function () {
		Sb.widget.Navigation.superclass.initEvents.call(this);

		//		Observable					Event				Handler						Scope
		this.mon(Sb.Editor.model.rootPage,	'change',			this.refresh,				this);
		this.mon(this.model,				'levelchange',		this.refresh,				this);
		this.mon(this.model,				'fontchange',		this.refreshCss,			this);
		this.mon(this.model,				'decorationchange',	this.refreshCss,			this);
		this.mon(this.model,				'backgroundchange',	this.refreshCss,			this);
		this.mon(this.model,				'colorchange',		this.refreshColorSchemeCss,	this);
		this.mon(Sb.Editor.model.design,	'colorschemechange',this.refreshColorSchemeCss,	this);
		this.mon(this.model,				'alignmentchange',	this.onAlignmentChange,		this);
	},

	onMouseEnter: function (e) {
		Sb.widget.Navigation.superclass.onMouseEnter.call(this, e);

		if (!Sb.Editor.Ui.isDragActive() && !Sb.Editor.Ui.editComponent) {
			this.toFront();
		}
	},

	/**
	 * Gets parent page for which menu is generated.
	 *
	 * @return {Sb.model.SitePage}
	 */
	getParentPage: function () {
		return Sb.Editor.siteView.currentPage.getPath()[this.model.get('level') - 1];
	},

	/**
	 * Create menu item and create all items of submenu
	 * @param {HTMLElement} parentEl
	 * @param {Sb.model.SitePage} page
	 * @param {Number} [level]
	 */
	createMenuItem: function (parentEl, page, level) {
		var cls = 'normal',
			hasVisibleChildren = false,
			itemEl, submenuEl;

		level = level || 1;
		if (this.isSelectedSubpage(page)) {
			cls = 'childselected';
		}
		if (Sb.Editor.siteView.currentPage.data.identity == page.data.identity) {
			cls = 'selected';
		}

		itemEl = parentEl.createChild({
			tag : 'li',
			cls :  cls,
			cn  : {
				tag  : 'a',
				href : '#',
				cn: [{
					tag	: 'span',
					html: '>',
					cls	: 'navigation-item-bullet'
				}, {
					tag	: 'span',
					cls	: 'navigation-item-text',
					html: Ext.util.Format.htmlEncode(page.data.title)
				}]
			}
		});
		itemEl.setVisibilityMode(Ext.Element.DISPLAY);
		itemEl.setVisible(!page.data.hidden);
		this.addInternalListener(itemEl, 'click', function (event, el, options) {
			event.stopEvent();
			Sb.Editor.siteView.setCurrentPage(options.page);
		}, null, {delegate: 'a', 'page' : page});
		this.addInternalListener(page, 'titlevisibilitychange', function () {
			var hasVisibleChildren = false,
				parentItemEl = itemEl.up('li');

			if (parentItemEl) {
				page.parentPage.pages.each(function (subPage) {
					hasVisibleChildren = hasVisibleChildren || !subPage.get('hidden');
				}, this);
				parentItemEl[(hasVisibleChildren ? 'add' : 'remove') + 'Class']('navigation-item-expand');
			}

			itemEl.child('.navigation-item-text').update(Ext.util.Format.htmlEncode(page.data.title));
			itemEl.setVisible(!page.data.hidden);
		});

		if (this.model.data.depth > level && page.pages.getCount()) {
			submenuEl = itemEl.createChild({
				tag : 'ul'
			});
			page.pages.each(function (subPage) {
				hasVisibleChildren = hasVisibleChildren || !subPage.get('hidden');
				this.createMenuItem(submenuEl, subPage, level + 1);
			}, this);

			if (hasVisibleChildren) {
				itemEl.addClass('navigation-item-expand');
			}
		}
		return itemEl;
	},

	isSelectedSubpage: function (page) {
		page.pages.each(function (subPage) {
			if (Sb.Editor.siteView.currentPage.data.identity == subPage.data.identity) {
				this.isSelected = true;
				return false;
			}
			this.isSelected = this.isSelectedSubpage(subPage);
		}, this);
		var isSelected = this.isSelected;
		this.isSelected = false;
		return isSelected;
	},

	/**
	 * Is widget has content, that could be rendered? Used in renderPlaceholder, for instance.
	 *
	 * @returns {Boolean}
	 */
	isEmpty: function () {
		var parentPage = this.getParentPage(),
			isEmpty = true;

		if (parentPage == null) {
			return isEmpty;
		}
		parentPage.pages.each(function (page) {
			if (!page.data.hidden) {
				isEmpty = false;
				return false;
			}
		}, this);

		return isEmpty;
	},

	/**
	 * Render content for widget`s placeholder if it has one
	 */
	renderPlaceholder: function () {
		Sb.widget.Navigation.superclass.renderPlaceholder.apply(this, arguments);
		this.togglePlaceholder(true);
		this.update(this.placeholderTpl.apply({
			placeholderMsg: Sb.Locale.get('navigationWidgetActiveText')
		}));

		// When we have all pages of current parent page hidden, we should rerender widget on visibility change
		var parentPage = this.getParentPage();
		if (parentPage) {
			parentPage.pages.each(function (page) {
				this.addInternalListener(page, 'titlevisibilitychange', function () {
					this.refresh();
				});
			}, this);
		}
	},

	refresh: function () {
		this.cleanInternalListeners();

		if (this.isEmpty()) {
			this.renderPlaceholder();
			return;
		}

		this.togglePlaceholder(false);
		this.update('');
		this.navigationEl = this.widgetContentEl.createChild({
			tag: 'ul',
			id:  'navigation-' + this.model.data.uuid,
			cls: 'navigation'
		});

		this.getParentPage().pages.each(function (page) {
			var itemEl = this.createMenuItem(this.navigationEl, page);
			if (this.isSelectedSubpage(page)) {
				itemEl.addClass('childselected');
			}
			// When we have all pages of current parent page hidden, we should rerender widget on visibility change
			this.addInternalListener(page, 'titlevisibilitychange', function () {
				if (this.isEmpty()) {
					this.refresh();
				}
			});

		}, this);

		this.onAlignmentChange();
	},

	onDocumentClick: function (event) {
		if (event.getTarget('.x-combo-list') || event.getTarget('.sb-color-dropdown')) {
			return;
		}

		Sb.widget.Navigation.superclass.onDocumentClick.call(this, event);
	},

	/**
	 * Updates CSS for navigation widget.
	 */
	refreshCss: function () {
		var id = 'navigation-' + this.model.get('uuid'),
			data = Ext.apply({className: '#' + id}, this.model.data),
			corners = {
				'': '0',
				all: 'p',
				top: 'p p 0 0',
				bottom: '0 0 p p',
				left: 'p 0 0 p',
				right: '0 p p 0'
			},
			borders = {
				'': 'p',
				all: 'p',
				top: 'p p 0 p',
				bottom: '0 p p p',
				left: 'p 0 p p',
				right: 'p p p 0'
			},
			regexp = /p/g;

		data.level1CornersRounding = corners[data.level1Corners].replace(regexp, data.level1CornersRounding);
		data.level2CornersRounding = corners[data.level2Corners].replace(regexp, data.level2CornersRounding);
		data.cornersRoundingActive = corners[data.cornersActive].replace(regexp, data.cornersRoundingActive);

		data.level1BorderWidth = borders[data.level1Corners].replace(regexp, data.level1BorderWidth);
		data.level2BorderWidth = borders[data.level2Corners].replace(regexp, data.level2BorderWidth);
		data.borderWidthActive = borders[data.cornersActive].replace(regexp, data.borderWidthActive);

		// Backgrounds.
		Ext.iterate (['level1', 'level2', 'active'], function (itemType) {
			var backgroundModelKey = itemType === 'active'
					? 'level1Background' + itemType.toUpperCaseFirst()
					: itemType + 'Background',
				background = data[backgroundModelKey],
				backgroundColorModelKey = itemType === 'active'
					? 'level1BackgroundColor' + itemType.toUpperCaseFirst()
					: itemType + 'BackgroundColor',
				backgroundColor = data[backgroundColorModelKey],
				backgroundType,
				backgroundImagePath,
				stretchModelKey,
				stretch,
				backgroundSizeModelKey,
				backgroundRepeatModelKey,
				gradientParams = Sb.Editor.model.data.template.decorations.gradients[background],
				backgroundRgb,
				gradientRgb,
				gradientStart,
				gradientEnd;

			if (!background) {
				return;
			}

			backgroundType = Sb.Utils.getBackgroundType(data[backgroundModelKey]);
			backgroundImagePath = backgroundType.isCustom
				? Sb.Editor.model.getUrl() + '/' + Sb.Config.path.attachmentsFolder + '/' +
					Sb.Config.path.backgroundFolder + '/' + background
				: Sb.Config.templatesUrl + '/' + Sb.Editor.model.data.template.code +
					'/decorations/background/' + background + '/images/' + background + '.png';
			stretchModelKey = itemType === 'active'
				? 'level1BackgroundStretch' + itemType.toUpperCaseFirst()
				: itemType + 'BackgroundStretch';
			stretch = data[stretchModelKey];
			backgroundSizeModelKey = itemType === 'active'
				? 'level1BackgroundSize' + itemType.toUpperCaseFirst()
				: backgroundModelKey + 'Size';
			backgroundRepeatModelKey = itemType === 'active'
				? 'level1BackgroundRepeat' + itemType.toUpperCaseFirst()
				: backgroundModelKey + 'Repeat';
			data[backgroundModelKey + 'Gradient'] = false;

			if (backgroundType.value === 'gradient') {
				gradientRgb = Ext.ux.util.Color.hex2rgba(gradientParams.startColor, gradientParams.transparency);

				if (Ext.ux.util.Color.isValid(backgroundColor)) {
					backgroundRgb = Ext.ux.util.Color.hex2rgba(backgroundColor);
					gradientStart = Ext.ux.util.Color.apply(backgroundRgb, gradientRgb);
					gradientEnd = backgroundRgb;
				} else {
					gradientStart = gradientRgb;
					gradientEnd = [0,0,0,0];
				}

				gradientStart = gradientStart.join(',');
				gradientEnd = gradientEnd.join(',');

				data[backgroundModelKey] = 'to ' + gradientParams.direction + ',rgba(' + gradientStart + '),rgba(' +
					gradientEnd + ')';
				data[backgroundModelKey + 'Gradient'] = true;
			} else {
				data[backgroundModelKey] = Sb.Utils.antiCache(backgroundImagePath, true);
				data[backgroundSizeModelKey] = 'auto';
				data[backgroundRepeatModelKey] = 'repeat';
			}

			if (backgroundType.value === 'pattern') {
				if (stretch == 'actual size') {
					data[backgroundRepeatModelKey] = 'no-repeat';
				} else if (stretch == 'crop') {
					data[backgroundSizeModelKey] = 'cover';
				} else if (stretch == 'resize') {
					data[backgroundRepeatModelKey] = 'no-repeat';
					data[backgroundSizeModelKey] = 'contain';
				}
			}
		});

		data.level1LineSize = data.level1FontSize + 10;
		data.level2LineSize = data.level2FontSize + 10;
		Sb.StyleManager.addStyleSheet(
			this.getType().getCssTemplate().apply(data),
			'widget-navigation',
			'font-' + this.model.get('uuid')
		);

		Sb.FontManager.loadFont(data.level1FontFamily);
		Sb.FontManager.loadFont(data.level2FontFamily);
	},

	/**
	 * Updates CSS for navigation widget colors.
	 */
	refreshColorSchemeCss: function () {
		var id = 'navigation-' + this.model.get('uuid'),
			data = Ext.apply({className: '#' + id}, this.model.data);

		Ext.iterate(data, function (name, value) {
			data[name] = Sb.Editor.model.design.getColor(value);
		});

		Sb.StyleManager.addStyleSheet(
			this.getType().getColorSchemeCssTemplate().apply(data),
			'widget-navigation',
			'color-' + this.model.get('uuid')
		);
	},

	/**
	 * Updates CSS for navigation widget alignment.
	 */
	onAlignmentChange: function () {
		if (this.navigationEl && this.model.data.level1Alignment) {
			var align = Sb.Editor.model.isRtl()
				? Sb.Utils.swapLeftAndRight(this.model.data.level1Alignment)
				: this.model.data.level1Alignment;
			this.navigationEl.setStyle('text-align', align.toLowerCase());
		}
	}
});