/**
 * @class Sb.widget.Widget
 * @extends Sb.editor.component.Base
 * @property {Sb.model.Widget} model
 * Base class for widgets
 * @constructor
 * @param {Object} config The config object
 */
Ext.define('Sb.widget.Widget', {
	extend: 'Sb.editor.component.Base',

	/**
	 * @cfg {Sb.model.Widget} model
	 * The {@link Sb.model.Widget} to bind this Widget to.
	 */

	hasStylePanel: true,

	/**
	 * @cfg {Boolean} hasVisibilityControl
	 * <p>True to display visibility controls buttons, false to avoid it (defaults to <code>true</code>).</p>
	 */
	hasVisibilityControl: true,

	/**
	 * @cfg {Boolean} hasResizer
	 * <p>True to display resizer, false to avoid it (defaults to <code>true</code>).</p>
	 */
	hasResizer: null,

	/**
	 * @cfg {Boolean} isShowWhenDrag
	 * <p>True to show widget when drag (defaults to <code>false</code>).</p>
	 */
	isShowWhenDrag: false,

	/**
	 * The template to use when rendering the Widget.
	 *
	 * @property tpl
	 * @type Ext.Template
	 */
	tpl: new Ext.Template(
		'<div id="widget-{uuid}" class="widget widget-{type}">',
			'<div class="widget-content">{content}</div>',
		'</div>',
		{compiled: true}
	),

	/**
	 * The template to use when rendering the placeholder of widget.
	 *
	 * @property placeholderTpl
	 * @type Ext.Template
	 */
	placeholderTpl: new Ext.Template(
		'<table cellpadding="0" cellspacing="0">',
			'<tr>',
				'<td><img src="' + Ext.BLANK_IMAGE_URL + '" align="absmiddle"/></td>',
				'<td>{placeholderMsg}</td>',
			'</tr>',
		'</table>',
		{compiled: true}
	),

	initComponent: function () {
		Sb.widget.Widget.superclass.initComponent.apply(this, arguments);

		this.removalConfirmationMessage = Sb.Locale.get('confirmRemovalWidget');

		if (this.hasResizer === null) {
			this.hasResizer = this.model.data.height !== undefined;
		}

		/**
		 * Internal listeners that could be cleaned up when we need in some intermediate stage of component lifetime
		 *
		 * @type Array
		 * @property internalListeners
		 *
		 */
		this.internalListeners = [];
	},

	onRender: function () {
		if (!this.model.isNew) {
			this.model.changeLocale(Sb.Editor.siteView.getCurrentSiteLocale());
		}

		Sb.widget.Widget.superclass.onRender.apply(this, arguments);

		if (this.hasResizer) {
			this.resizer = new Ext.Resizable(this.getEl(), {
				widget: this,
				handles: 's',
				minHeight: 50,
				pinned: true,
				dynamic: true,
				handleCls: 'sb-widget-resizable-handler',
				resizeElement: function () {
					var box = this.proxy.getBox(),
						height = box.height - this.widget.widgetEl.getPadding('tb');

					if (!this.infoEl) {
						this.infoEl = this.widget.getEl().insertFirst({
							cls: 'sb-widget-columns-column-info',
							cn: {}
						});
					}

					this.infoEl.first().update(height + 'px').alignTo(this.south.el, 'b-t');

					this.widget.model.set('height', height);
					this.widget.anchorControls();
					return box;
				},
				listeners: {
					scope: this,
					resize: function (resizer, width, height) {
						if (resizer.infoEl) {
							resizer.infoEl.remove();
							delete resizer.infoEl;

						}
						if (Sb.Editor.Ui.editComponent != this && resizer.startBox.height != height) {
							this.model.save();
						}
					}
				}
			});
		}

		this.refreshContent();

		if (this.hasVisibilityControl) {
			this.renderVisibilityControl();
			this.updateVisibilityState();
		}

		if (this.getType().hasWidgetTitle) {
			this.createWidgetTitle();
		}
	},

	updateVisibilityState: function () {
		var isFirstPage;
		if (this.model.isInSiteWideArea()) {
			isFirstPage = Sb.Editor.siteView.currentPage === Sb.Editor.model.rootPage.getFirstVisiblePage();
			if ((this.model.get('visibilityMode') === 'except-first' && isFirstPage)
					|| (this.model.get('visibilityMode') === 'first' && !isFirstPage)) {
				this.hide();
			} else {
				this.show();
			}
		}
	},

	removeOver: function () {
		if (!this.hideSettingsMenu || this.hideSettingsMenu.hidden) {
			this.callParent();
		}
	},

	renderVisibilityControl: function () {
		this.hideSettingsEl = this.controlsTargetEl.createChild({
			tag: 'span',
			cls: this.baseCls + '-control-item ' + this.baseCls + '-hide-properties ',
			cn: {
				tag: 'span'
			}
		}).setVisibilityMode(Ext.Element.DISPLAY);
		this.setVisibilityTitle(this.model.get('visibilityMode'));

		if (!this.model.isInSiteWideArea()) {
			this.hideSettingsEl.hide();
		}

		this.hideSettingsEl.on('click', function () {
			if (!this.hideSettingsMenu.isVisible() && !this.hideSettingsMenu.isIgnoreNextShowEvent) {
				this.hideSettingsMenu.show(this.hideSettingsEl, 'bl?');
			}
		}, this);

		this.hideSettingsMenu = new Ext.menu.Menu({
			isIgnoreNextShowEvent: false,
			cls: 'sb-hide-settings-menu',
			defaults: {
				scope: this,
				handler: function (item) {
					this.model.set('visibilityMode', item.value);
					if (!this.memento) {
						this.model.save();
						this.updateVisibilityState();
					}
				}
			},
			items: [{
				text: Sb.Locale.get('widgetVisibilityAll'),
				value: 'all'
			}, {
				text: Sb.Locale.get('widgetVisibilityFirst'),
				value: 'first'
			}, {
				text: Sb.Locale.get('widgetVisibilityExceptFirst'),
				value: 'except-first'
			}],
			listeners: {
				hide: function () {
					this.isIgnoreNextShowEvent = true;
					this.isIgnoreNextShowEvent = Ext.defer(function() {
						this.isIgnoreNextShowEvent = false;
					}, 200, this);
				}
			}
		});
	},

	setVisibilityTitle: function (value) {
		var span = this.hideSettingsEl.child('span');

		span.update(Sb.Locale.get('widgetVisibility' + value.camelize().toUpperCaseFirst()));
		span[value === 'all' ? 'removeClass' : 'addClass']('not-always-visible');
	},

	showVisibilityLabel: function () {
		if (this.hasVisibilityControl && this.model.isInSiteWideArea()) {
			this.visibilityLabelEl = this.contentElement.createChild({
				cls: 'sb-widget-visibility-label',
				html: Sb.Locale.get('widgetVisibility' + this.model.get('visibilityMode').camelize().toUpperCaseFirst())
			});
		}
	},

	hideVisibilityLabel: function () {
		if (this.visibilityLabelEl && this.hasVisibilityControl && this.model.isInSiteWideArea()) {
			this.visibilityLabelEl.remove();
		}
	},

	refreshContent: function () {
		this.tpl.overwrite(this.contentElement, {
			uuid: this.model.get('uuid'),
			type: this.model.get('type')
		});
		this.widgetEl = this.contentElement.child('.widget-' + this.model.get('type'));
		this.widgetContentEl = this.contentElement.child('.widget-content');

		if (!this.model.isConfiguredProperties()) {
			this.renderPlaceholder();
		} else {
			this.renderContent();
			this.afterRenderContent();
		}
		if (this.model.get('styles')) {
			this.onStylesChange(this.model.get('styles'));
		}
	},

	renderContent: function () {
		var content = '', template = Sb.Editor.model.get('template').partials[this.getType().getPhpType()];
		if (Ext.isDefined(template)) {
			var partialTemplate = twig({data: template});
			content = partialTemplate.render(this.prepareData());
		}
		this.getContentTarget().update(content);
	},

	/**
	 * Editor specific logic
	 */
	afterRenderContent: Ext.emptyFn,

	createWidgetTitle: function (config) {
		this.widgetTitle = new Sb.editor.component.Text(Ext.apply({
			cls: 'sb-widget-title',
			emptyText: Sb.Locale.get('titleEmptyMessage'),
			hasControls: false,
			hasEditPanel: false,
			removable: false,
			movable: false,
			clickable: this.clickable,
			overable: this.overable,
			isMultiline: false,
			listeners: {
				'completeedit': {
					scope: this,
					fn: function (titleModel) {
						this.setWidgetTitle(titleModel.data.content.replace(/&nbsp;/g," "));
					}
				},
				'afterrender': function () {
					this.contentElement = this.controlsEl.next().createChild({
						tag: 'h2',
						cls: 'widget-title'
					});
				}
			}
		}, config || {}));
		this.widgetTitle.render(this.widgetEl, 0);
		this.widgetTitle.el.setVisibilityMode(Ext.Element.DISPLAY);
		this.onWidgetTitleChange();
	},

	initEvents: function () {
		Sb.widget.Widget.superclass.initEvents.call(this);

		this.mon(this.model, 'propertychange', function (key, value, oldValue) {
			if (key == 'widgetTitle') {
				this.onWidgetTitleChange();
			} else if (key === 'visibilityMode') {
				this.setVisibilityTitle(value);
			} else if (this.getXType() === undefined) {
				this.refreshContent();
			}
		}, this);

		this.mon(this.model, 'styleschange', this.onStylesChange, this);
		this.mon(Sb.Editor.model.design, 'colorschemechange', function () {
			this.onStylesChange({
				backgroundColor: this.model.get('styles').backgroundColor
			});
		}, this);

		if (this.translatable) {
			this.mon(Sb.Editor.model, 'propertychange', function(key, value) {
				if (key === 'locales') {
					this.model.updateTranslationList(value);
				}
			}, this);
		}

		if (this.hasVisibilityControl) {
			this.mon(Sb.Editor.model.rootPage, 'change', function () {
				if (this.model.isInSiteWideArea()) {
					this.updateVisibilityState();
				}
			}, this);
		}
	},

	/**
	 * Prepare data for template processing
	 * @return {Object} Generated data
	 */
	prepareData: function () {
		return this.model.serialize();
	},

	/**
	 * Enables/disables placeholder mode by adding/removing class to main element
	 * @deprecated Remove this method once all widgets will use rendering with templates
	 * @param {Boolean} isEnable
	 */
	togglePlaceholder: function (isEnable) {
		this.getContentTarget()[(isEnable ? 'add' : 'remove' ) + 'Class']('sb-widget-placeholder');
		this.getContentTarget()[(isEnable ? 'add' : 'remove' ) + 'Class'](this.getStateCls('inactive'));
	},

	/**
	 * Render content for widget`s placeholder if it has one
	 * @return {Sb.widget.Widget} self
	 */
	renderPlaceholder: function (msg, state) {
		msg = msg || this.model.getType().getTranslation('inactiveText');
		state = Ext.isDefined(state) ? state : 'inactive';

		this.getContentTarget().addClass('sb-widget-placeholder');
		if (state !== false) {
			this.getContentTarget().addClass(this.getStateCls(state));
		}

		this.update(
			this.placeholderTpl.apply({
				placeholderMsg: msg
			})
		);

		return this;
	},

	getContentTarget: function () {
		return this.widgetContentEl;
	},

	/**
	 * Returns the <code>id</code> of this widget (sb-widget-UUID)
	 * @return {String} id
	 */
	getId: function () {
		return this.id || (this.id = 'sb-widget-' + this.model.get('uuid'));
	},

	/**
	 * Returns the <code>type</code> of this widget`s model
	 *
	 * @return {Sb.widgetType.Base}
	 */
	getType: function () {
		return this.model.getType();
	},

	/**
	 * Tries to get edit panel for current type
	 * @return {Sb.editor.component.BaseEditPanel} editPanel
	 */
	getEditPanel: function () {
		return this.getType().getEditPanel();
	},

	/**
	 * Is show panel on edit?
	 * @return {Boolean} default is true
	 */
	getShowEditPanelState: function () {
		return !!this.model.isShowEditPanel;
	},

	/**
	 * Change edit panel state
	 * @param {Boolean} isShowEditPanel
	 */
	toggleEditPanelState: function (isShowEditPanel) {
		this.model.isShowEditPanel = isShowEditPanel;
	},

	/**
	 * Tries to get style panel for current type
	 * @return {Sb.widget.WidgetStylePanel} stylePanel
	 */
	getStylePanel: function () {
		if (!Sb.Editor.Ui.stylePanel) {
			Sb.Editor.Ui.stylePanel = new Sb.widget.WidgetStylePanel();
		}
		return Sb.Editor.Ui.stylePanel;
	},

	getStateCls: function (state) {
		var cls = this.baseCls;
		if (this.model.isInSiteWideArea()) {
			cls += '-shared';
		}
		cls += '-' + state;
		return cls;
	},

	afterRender: function () {
		Sb.widget.Widget.superclass.afterRender.call(this);

		if (this.model.isNew) {
			this.model.isNew = false;
			this.startEdit(Sb.Editor.model.get('locale'));
		}
	},

	getWidgetTitle: function () {
		return this.model.get('widgetTitle') || '';
	},

	setWidgetTitle: function (newTitle) {
		if (this.getWidgetTitle() != newTitle) {
			this.model.set('widgetTitle', newTitle);
			this.model.save();
		}
	},

	onWidgetTitleChange: function () {
		this.widgetTitle.setData({
			content: this.getWidgetTitle()
		});

		if (Sb.Editor.Ui.editComponent == this) {
			this.getEditPanel().reanchor();
		}
	},

	removeComponentHandler: function () {
		Sb.widget.Widget.superclass.removeComponentHandler.call(this);
		this.ownerCt.model.removeChild(this.model);
	},

	onStartEdit: function () {
		Sb.widget.Widget.superclass.onStartEdit.call(this);
		if (this.hasVisibilityControl && this.model.isInSiteWideArea()) {
			this.hideSettingsEl.show();
		}
		this.memento = this.getMemento();
	},

	/**
	 * @return {Object}
	 */
	getMemento: function () {
		return this.model.saveToMemento();
	},

	onCancelEdit: function () {
		if (this.memento) {
			this.model.restoreFromMemento(this.memento);
		}
	},

	onCompleteEdit: function () {
		if (this.model.get('visibilityMode') !== this.memento.visibilityMode) {
			this.updateVisibilityState();
		}
		this.model.saveIfChanged(this.memento, {
			success: function () {
				Sb.Editor.Ui.mainWindow.statusBar.setMessage(
					Sb.Locale.get('messageChangesSaved')
				);
			}
		});
	},

	onHideEdit: function () {
		delete this.memento;
		this.callParent();
	},

	/**
	 * Add internal listener
	 *
	 * @param {Object} item
	 * @param {String} ename
	 * @param {Function} fn
	 * @param {Object} [scope]
	 * @param {Object} [opt]
	 */
	addInternalListener: function (item, ename, fn, scope, opt) {
		scope	= scope || this;
		opt		= opt || {};
		this.mon(item, ename, fn, scope, opt);
		this.internalListeners.push({item: item, ename: ename, fn: fn, scope: scope, opt: opt});
	},

	/**
	 * Cleans up internal listeners and unsubscribes all of them
	 */
	cleanInternalListeners: function () {
		Ext.each(this.internalListeners, function (internalListener) {
			this.mun(internalListener.item, internalListener.ename, internalListener.fn, internalListener.scope);
		}, this);
	},

	onDestroy: function () {
		if (this.resizer) {
			this.resizer.destroy();
		}
		Sb.widget.Widget.superclass.onDestroy.call(this);
		if (Sb.Editor.Ui.overComponent == this) {
			if (Sb.Editor.Ui.overComponent.showOverTask) {
				Sb.Editor.Ui.overComponent.showOverTask.cancel();
			}
			Sb.Editor.Ui.overComponent = null;
		}
		if (Sb.Editor.Ui.editComponent == this) {
			Sb.Editor.Ui.editComponent = null;
		}
	},

	initDD: function () {
		this.dd = new Sb.widget.DD(this);
	},

	/**
	 * Reanchor edit panel of current widget if needed
	 */
	reanchorEditPanel: function () {
		if (Sb.Editor.Ui.editComponent == this) {
			this.getEditPanel().reanchor();
		}
	},

	/**
	 * Reanchor style panel of current widget if needed
	 */
	reanchorStylePanel: function () {
		if (Sb.Editor.Ui.editComponent == this) {
			this.getStylePanel().reanchor();
		}
	},

	onStylesChange: function (styles) {
		var backgroundConfig = {},
			modelStyles = this.model.get('styles'),
			backgroundType = Sb.Utils.getBackgroundType(modelStyles.background),
			stretchKeyToCss = {
				'tile': {'background-repeat': 'repeat'},
				'crop': {'background-size': 'cover'},
				'resize': {'background-size': 'contain'},
				'actual size': {}
			};

		Ext.iterate (styles, function (key, value) {
			var backgroundColor;

			if(value === undefined) {
				return true;
			}

			if (['margin', 'padding', 'borderRadius'].indexOf(key) > -1) {
				this.widgetEl.setStyle(key, value);
			}

			if (['borderRadius', 'boxShadow'].indexOf(key) > -1) {
				backgroundConfig[key] = value;
			}

			if (key == 'backgroundColor') {
				if (backgroundType.value === 'none') {
					value = value || 'transparent';
					value = Sb.Editor.model.design.getColor(value);

					if (Ext.ux.util.Color.isValid(value)) {
						value = Ext.ux.util.Color.hex2rgba(value, modelStyles.backgroundOpacity);
						backgroundConfig['background-color'] = 'rgba(' + value.join(',') + ')';
					} else {
						backgroundConfig['background-color'] = value;
					}

					backgroundConfig['background-image'] = '';
				}

				if (backgroundType.value === 'gradient') {
					Ext.apply(backgroundConfig, this.getGradientStyles());
				}
			}

			if (key == 'backgroundOpacity') {
				if (backgroundType.value === 'none') {
					backgroundColor = Sb.Editor.model.design.getColor(modelStyles.backgroundColor);
					if (Ext.ux.util.Color.isValid(backgroundColor)) {
						backgroundConfig['background-color'] = 'rgba(' +
							Ext.ux.util.Color.hex2rgba(backgroundColor, value).join(',') +
						')';
					} else {
						backgroundConfig['background-color'] = backgroundColor;
					}
				}

				if (backgroundType.value === 'gradient') {
					Ext.apply(backgroundConfig, this.getGradientStyles());
				}
			}

			if (key == 'background') {
				if (backgroundType.value === 'none') {
					backgroundColor = Sb.Editor.model.design.getColor(modelStyles.backgroundColor);
					if (Ext.ux.util.Color.isValid(backgroundColor)) {
						backgroundConfig['background-color'] = 'rgba(' +
							Ext.ux.util.Color.hex2rgba(backgroundColor, modelStyles.backgroundOpacity).join(',') +
							')';
					} else {
						backgroundConfig['background-color'] = backgroundColor;
					}
					backgroundConfig['background-image'] = '';
				}

				if (backgroundType.value === 'gradient') {
					Ext.apply(backgroundConfig, this.getGradientStyles());
					backgroundConfig['background-size'] = '100% 100%';
					backgroundConfig['background-color'] = '';
				}

				if (backgroundType.value === 'pattern') {
					backgroundConfig['background-color'] = '';
					backgroundConfig['background-position'] = modelStyles.backgroundPosition;
					backgroundConfig['background-size'] = 'auto';
					backgroundConfig['background-repeat'] = 'no-repeat';
					backgroundConfig = Ext.apply(backgroundConfig, stretchKeyToCss[modelStyles.backgroundStretch]);
				}
			}

			if (key == 'backgroundPosition') {
				if (backgroundType.value === 'pattern') {
					backgroundConfig['background-position'] = value;
				}
			}

			if (key == 'backgroundStretch') {
				if (backgroundType.value === 'pattern') {
					backgroundConfig['background-size'] = 'auto';
					backgroundConfig['background-repeat'] = 'no-repeat';
					backgroundConfig = Ext.apply(backgroundConfig, stretchKeyToCss[modelStyles.backgroundStretch]);
				}
			}
		}, this);

		if (styles.backgroundOpacity !== undefined || styles.background !== undefined ||
			styles.backgroundColor !== undefined
		) {
			if (backgroundType.value === 'pattern') {
				Sb.Editor.siteView.applyBackgroundImage(
					this.widgetEl,
					this.getBackgroundUrl(modelStyles.background, backgroundType.isCustom),
					modelStyles.backgroundOpacity,
					Sb.Editor.model.design.getColor(modelStyles.backgroundColor)
				);
			}
		}

		this.widgetEl.setStyle(backgroundConfig);

		var id = '#widget-' + this.model.get('uuid'),
			design = Sb.Editor.model.design,
			css = '',
			blackStrokeClasses = [],
			whiteStrokeClasses = [],
			data = {
				className: id,
				textColor: design.getColor(modelStyles.textColor),
				linkColor: design.getColor(modelStyles.linkColor),
				h1Color: design.getColor(modelStyles.h1Color),
				h2Color: design.getColor(modelStyles.h2Color)
			};

		if (modelStyles.textStroke && data.textColor != 'inherit') {
			(Ext.ux.util.Color.isLight(data.textColor) ? blackStrokeClasses : whiteStrokeClasses).push(id);
		}
		if (modelStyles.linkStroke && data.linkColor != 'inherit') {
			(Ext.ux.util.Color.isLight(data.linkColor) ? blackStrokeClasses : whiteStrokeClasses).push(id + ' a');
		}
		if (modelStyles.h1Stroke && data.h1Color != 'inherit') {
			(Ext.ux.util.Color.isLight(data.h1Color) ? blackStrokeClasses : whiteStrokeClasses).push(id + ' h1');
		}
		if (modelStyles.h2Stroke && data.h2Color != 'inherit') {
			(Ext.ux.util.Color.isLight(data.h2Color) ? blackStrokeClasses : whiteStrokeClasses).push(id + ' h2');
		}
		if (blackStrokeClasses.length) {
			css += blackStrokeClasses.join(',') + '{ text-shadow:'
				+ 'rgba(0, 0, 0, 0.4) -1px -1px 1px,'
				+ 'rgba(0, 0, 0, 0.4) 1px -1px 1px,'
				+ 'rgba(0, 0, 0, 0.4) -1px 1px 1px,'
				+ 'rgba(0, 0, 0, 0.4) 1px 1px 1px,'
				+ 'rgba(0, 0, 0, 0.9) 0 1px 2px; }';
		}
		if (whiteStrokeClasses.length) {
			css += whiteStrokeClasses.join(',') + '{ text-shadow:'
				+ 'rgba(255, 255, 255, .9) -1px -1px 0px,'
				+ 'rgba(255, 255, 255, .9) -1px 1px 0px,'
				+ 'rgba(255, 255, 255, .9) 1px -1px 0px,'
				+ 'rgba(255, 255, 255, .9) 1px 1px 0px,'
				+ 'rgba(255, 255, 255, .8) -2px -2px 1px,'
				+ 'rgba(255, 255, 255, .8) -2px 2px 1px,'
				+ 'rgba(255, 255, 255, .8) 2px -2px 1px,'
				+ 'rgba(255, 255, 255, .8) 2px 2px 1px,'
				+ 'rgba(255, 255, 255, .8) 2px 0px 1px,'
				+ 'rgba(255, 255, 255, .8) 0px 2px 1px,'
				+ 'rgba(255, 255, 255, .8) -2px 0px 1px,'
				+ 'rgba(255, 255, 255, .8) 0px -2px 1px; }';
		}

		css += Sb.SiteView.fontColorCssTemplate.apply(data);

		Sb.StyleManager.addStyleSheet(css, 'text-color', id);
	},

	getBackgroundUrl: function (value, isCustom) {
		var url;

		if (isCustom) {
			url = this.model.getResourcesUrl() + '/' + Sb.Config.path.attachmentsFolder + '/' +
				Sb.Config.path.backgroundFolder + '/' + value;
		} else {
			url = Sb.Config.templatesUrl + '/' + Sb.Editor.model.data.template.code +
				'/decorations/background/' + value + '/images/' + value + '.png';
		}

		return Sb.Utils.antiCache(url, true);
	},

	onCurrentLocaleChange: function (newLocale, oldLocale) {
		this.callParent([newLocale, oldLocale]);
		this.model.changeLocale(newLocale, oldLocale);
	},

	doResize: Ext.emptyFn,

	getGradientStyles: function () {
		var	modelStyles = this.model.get('styles'),
			gradientParams = Sb.Editor.model.data.template.decorations.gradients[modelStyles.background],
			backgroundColor = Sb.Editor.model.design.getColor(modelStyles.backgroundColor),
			backgroundOpacity = (modelStyles.backgroundOpacity / 100).toFixed(2),
			gradientRgb = Ext.ux.util.Color.hex2rgba(gradientParams.startColor, gradientParams.transparency),
			backgroundRgb,
			gradientStart,
			gradientEnd;

		if (Ext.ux.util.Color.isValid(backgroundColor)) {
			backgroundRgb = Ext.ux.util.Color.hex2rgba(backgroundColor);
			gradientStart = Ext.ux.util.Color.apply(backgroundRgb, gradientRgb);
			gradientStart[3] = backgroundOpacity;
			gradientEnd = backgroundRgb;
			gradientEnd[3] = backgroundOpacity;
		} else {
			gradientStart = gradientRgb;
			gradientStart[3] *= backgroundOpacity;
			gradientEnd = [0,0,0,0];
		}

		gradientStart = gradientStart.join(',');
		gradientEnd = gradientEnd.join(',');

		return {
			'background-image': 'linear-gradient(' +
				'to ' + gradientParams.direction + ',rgba(' + gradientStart + '),rgba(' + gradientEnd + ')' +
			')'
		};
	}
});

//private - custom Widget DD implementation
Ext.define('Sb.widget.DD', {
	extend: 'Ext.dd.DragSource',

	constructor: function (widget, cfg) {
		cfg = Ext.apply({
			ddGroup: 'widgets',
			dragData: {
				widget: widget,
				widgetType: widget.getType()
			}
		}, cfg);

		this.callParent([widget.el, cfg]);

		if (this.dragData.widget.hasControls && this.dragData.widget.ddEl) {
			this.setOuterHandleElId(this.dragData.widget.ddEl.id);
		}
		this.scroll = false;

		this.proxy.ghost.addClass('sb-dd-drag-ghost');
		if (!widget.isShowWhenDrag) {
			this.proxy.update(Ext.DomHelper.createDom(
				Sb.Utils.getWidgetButtonCfg(
					this.dragData.widgetType.name,
					this.dragData.widgetType.getTranslation('title')
				)
			));
		}

		this.dragData.widget.ddEl.on('click', this.handleMouseDown, this);

		// See Sb.editor.component.Base.removeOver()
		this.component = widget;
	},

	startDrag: function (x, y) {
		var widget = this.dragData.widget,
			startTop = widget.el.getTop(),
			elCopy;

		this.startScrollTop = Sb.Editor.siteView.el.getScroll().top;
		Sb.Editor.siteView.onWidgetStartDrag();

		Sb.Editor.siteView.el.scroll('b', widget.el.getTop() - startTop);

		this.startDragXY = [x, y];
		this.isMoved = false;

		if (Sb.Editor.Ui.editComponent) {
			Sb.Editor.Ui.editComponent.completeEdit.call(Sb.Editor.Ui.editComponent);
			if (!widget.controlsEl.isVisible()) {
				widget.showControls();
			}
		}

		if (widget.isShowWhenDrag) {
			elCopy = widget.el.dom.cloneNode(true);
			elCopy.id = Ext.id();
			elCopy.style.width = widget.el.getWidth() + 'px';

			this.proxy.update(elCopy);
		}

		// :TODO: echernodarov 20100420: dirty hack, need rework
		widget.afterDD = true;
		widget.ownerCt.onNotifyOver(this, [x, y]);
		widget.removeClass(this.dragData.widget.getStateCls('over'));
		widget.ownerCt.onStartDragChildWidget(this);
		widget.hide();
	},

	endDrag: function (e) {
		var widget = this.dragData.widget;

		Sb.Editor.siteView.onWidgetEndDrag(widget);
		widget.ownerCt.onEndDragChildWidget(this);
		if (!widget.isVisible()) {
			widget.show();
		}

		// :INFO: ekaragodin 20120423: workaround for bug #105271
		var currentXY = e.getXY();
		if ((this.startDragXY[0] === currentXY[0]) && (this.startDragXY[1] === currentXY[1])) {
			this.onInvalidDrop();
		}
	},

	onDrag: function () {
		this.isMoved = true;
	},

	beforeDragDrop: function (target, e, id) {
		if (target.id != id || !this.isMoved) {
			return false;
		}

		if (!this.dragData.container) { // drop to usual (not column) container?
			return true;
		}
		return (this.dragData.container.el.id == id);
	},

	onInvalidDrop: function () {
		Sb.Highlighting.hide();

		var animCfgObj = {
			easing: 'elasticOut',
			duration: 0.3,
			scope: this,
			callback: function () {
				this.proxy.hide();
			}
		};

		this.proxy.getEl().moveTo(this.startPageX, this.startPageY, animCfgObj);
		this.dragData.widget.ownerCt.onWidgetInvalidDrop(this);
		Sb.Editor.siteView.el.scrollTo('top', this.startScrollTop);
	},

	afterDragDrop: function () {
		if (this.dragData.widget.isShowWhenDrag) {
			this.proxy.update('');
		}
	}
});